@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.biclustering;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -64,7 +64,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * @param <V> Vector type.
  */
 @Reference(authors = "Y. Cheng, G. M. Church", title = "Biclustering of expression data", booktitle = "Proc. 8th International Conference on Intelligent Systems for Molecular Biology (ISMB)")
-public class ChengAndChurch<V extends NumberVector<?>> extends AbstractBiclustering<V, BiclusterWithInversionsModel> {
+public class ChengAndChurch<V extends NumberVector> extends AbstractBiclustering<V, BiclusterWithInversionsModel> {
   /**
    * The logger for this class.
    */
@@ -555,9 +555,7 @@ public class ChengAndChurch<V extends NumberVector<?>> extends AbstractBicluster
         // LOG.verbose("Total number of masked values: " + maskedVals.size() +
         // "\n");
       }
-      if(prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     // Add a noise cluster, full-dimensional.
     if(!noise.isEmpty()) {
@@ -565,9 +563,7 @@ public class ChengAndChurch<V extends NumberVector<?>> extends AbstractBicluster
       BiclusterWithInversionsModel model = new BiclusterWithInversionsModel(colsBitsetToIDs(allcols), DBIDUtil.EMPTYDBIDS);
       result.addToplevelCluster(new Cluster<>(noise, true, model));
     }
-    if(prog != null) {
-      prog.ensureCompleted(LOG);
-    }
+    LOG.ensureCompleted(prog);
     return result;
   }
 
@@ -805,7 +801,7 @@ public class ChengAndChurch<V extends NumberVector<?>> extends AbstractBicluster
    * 
    * @param <V> Vector type
    */
-  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector> extends AbstractParameterizer {
     /**
      * Parameter to specify the distribution of replacement values when masking
      * a cluster.
