@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.correlation.cash;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -77,7 +77,7 @@ public class ParameterizationFunction {
   /**
    * The actual vector.
    */
-  private NumberVector<?> vec;
+  private NumberVector vec;
 
   /**
    * Provides a new parameterization function describing all lines in a
@@ -85,7 +85,7 @@ public class ParameterizationFunction {
    * 
    * @param vec Existing vector
    */
-  public ParameterizationFunction(NumberVector<?> vec) {
+  public ParameterizationFunction(NumberVector vec) {
     super();
     this.vec = vec;
     determineGlobalExtremum();
@@ -403,7 +403,7 @@ public class ParameterizationFunction {
       if(d != 0) {
         result.append(" + \n").append(FormatUtil.whitespace(offset));
       }
-      result.append(FormatUtil.format(vec.doubleValue(d)));
+      result.append(vec.doubleValue(d));
       for(int j = 0; j < d; j++) {
         result.append(" * sin(a_").append(j + 1).append(')');
       }
