@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.hierarchical;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,7 +24,6 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.hierarchical;
  */
 import de.lmu.ifi.dbs.elki.algorithm.Algorithm;
 import de.lmu.ifi.dbs.elki.database.Database;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
  * Interface for hierarchical clustering algorithms.
@@ -35,17 +34,8 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
  * @author Erich Schubert
  * 
  * @apiviz.has PointerHierarchyRepresentationResult
- * 
- * @param <D> Distance type
  */
-public interface HierarchicalClusteringAlgorithm<D extends Distance<D>> extends Algorithm {
+public interface HierarchicalClusteringAlgorithm extends Algorithm {
   @Override
-  public PointerHierarchyRepresentationResult<D> run(Database db);
-
-  /**
-   * Return the distance type that will be used by the algorithm.
-   * 
-   * @return Distance factory.
-   */
-  public D getDistanceFactory();
+  public PointerHierarchyRepresentationResult run(Database db);
 }
