@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,7 +23,6 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.BitSet;
 import java.util.Random;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
@@ -47,12 +46,12 @@ import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.SubspaceMaximumDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
-import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
+import de.lmu.ifi.dbs.elki.math.random.RandomFactory;
+import de.lmu.ifi.dbs.elki.utilities.BitsUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
@@ -66,14 +65,14 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 
 /**
  * <p>
- * Provides the DOC algorithm, and it's heuristic variant, FastDOC. DOC is a
- * sampling based subspace clustering algorithm.
+ * The DOC algorithm, and it's heuristic variant, FastDOC. DOC is a sampling
+ * based subspace clustering algorithm.
  * </p>
  * 
  * <p>
  * Reference: <br/>
  * C. M. Procopiuc, M. Jones, P. K. Agarwal, T. M. Murali<br />
- * A Monte Carlo algorithm for fast projective clustering. <br/>
+ * A Monte Carlo algorithm for fast projective clustering. <br />
  * In: Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD '02).
  * </p>
  * 
@@ -84,8 +83,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
  * @param <V> the type of NumberVector handled by this Algorithm.
  */
 @Title("DOC: Density-based Optimal projective Clustering")
-@Reference(authors = "C. M. Procopiuc, M. Jones, P. K. Agarwal, T. M. Murali", title = "A Monte Carlo algorithm for fast projective clustering", booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD '02)", url = "http://dx.doi.org/10.1145/564691.564739")
-public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
+@Reference(authors = "C. M. Procopiuc, M. Jones, P. K. Agarwal, T. M. Murali", //
+title = "A Monte Carlo algorithm for fast projective clustering", //
+booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD '02)", //
+url = "http://dx.doi.org/10.1145/564691.564739")
+public class DOC<V extends NumberVector> extends AbstractAlgorithm<Clustering<SubspaceModel>> implements SubspaceClusteringAlgorithm<SubspaceModel> {
   /**
    * The logger for this class.
    */
@@ -152,7 +154,7 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
    * @param database Database
    * @param relation Data relation
    */
-  public Clustering<SubspaceModel<V>> run(Database database, Relation<V> relation) {
+  public Clustering<SubspaceModel> run(Database database, Relation<V> relation) {
     // Dimensionality of our set.
     final int d = RelationUtil.dimensionality(relation);
 
@@ -173,7 +175,7 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
     int minClusterSize = (int) (alpha * S.size());
 
     // List of all clusters we found.
-    Clustering<SubspaceModel<V>> result = new Clustering<>("DOC Clusters", "DOC");
+    Clustering<SubspaceModel> result = new Clustering<>("DOC Clusters", "DOC");
 
     // Inform the user about the number of actual clusters found so far.
     IndefiniteProgress cprogress = LOG.isVerbose() ? new IndefiniteProgress("Number of clusters", LOG) : null;
@@ -181,7 +183,7 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
     // To not only find a single cluster, we continue running until our set
     // of points is empty.
     while(S.size() > minClusterSize) {
-      Cluster<SubspaceModel<V>> C;
+      Cluster<SubspaceModel> C;
       if(heuristics) {
         C = runFastDOC(relation, S, d, n, m, (int) r);
       }
@@ -206,15 +208,10 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
 
     // Add the remainder as noise.
     if(S.size() > 0) {
-      BitSet alldims = new BitSet();
-      alldims.set(0, d);
-      result.addToplevelCluster(new Cluster<>(S, true, new SubspaceModel<>(new Subspace(alldims), Centroid.make(relation, S).toVector(relation))));
+      long[] alldims = BitsUtil.ones(d);
+      result.addToplevelCluster(new Cluster<>(S, true, new SubspaceModel(new Subspace(alldims), Centroid.make(relation, S))));
     }
-
-    if(cprogress != null) {
-      cprogress.setCompleted(LOG);
-    }
-
+    LOG.setCompleted(cprogress);
     return result;
   }
 
@@ -230,12 +227,11 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
    * @param minClusterSize Minimum size a cluster must have to be accepted.
    * @return a cluster, if one is found, else <code>null</code>.
    */
-  private Cluster<SubspaceModel<V>> runDOC(Relation<V> relation, ArrayModifiableDBIDs S, final int d, int n, int m, int r, int minClusterSize) {
-    final DoubleDistance wd = new DoubleDistance(w);
+  private Cluster<SubspaceModel> runDOC(Relation<V> relation, ArrayModifiableDBIDs S, final int d, int n, int m, int r, int minClusterSize) {
     // Best cluster for the current run.
     DBIDs C = null;
     // Relevant attributes for the best cluster.
-    BitSet D = null;
+    long[] D = null;
     // Quality of the best cluster.
     double quality = Double.NEGATIVE_INFINITY;
 
@@ -244,9 +240,9 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
     // double[d], new double[d]);
 
     // Weights for distance (= rectangle query)
-    SubspaceMaximumDistanceFunction df = new SubspaceMaximumDistanceFunction(new BitSet(d));
-    DistanceQuery<V, DoubleDistance> dq = relation.getDatabase().getDistanceQuery(relation, df);
-    RangeQuery<V, DoubleDistance> rq = relation.getDatabase().getRangeQuery(dq);
+    SubspaceMaximumDistanceFunction df = new SubspaceMaximumDistanceFunction(BitsUtil.zero(d));
+    DistanceQuery<V> dq = relation.getDatabase().getDistanceQuery(relation, df);
+    RangeQuery<V> rq = relation.getDatabase().getRangeQuery(dq);
 
     // Inform the user about the progress in the current iteration.
     FiniteProgress iprogress = LOG.isVerbose() ? new FiniteProgress("Iteration progress for current cluster", m * n, LOG) : null;
@@ -263,22 +259,22 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
         DBIDs randomSet = DBIDUtil.randomSample(S, Math.min(S.size(), r), random);
 
         // Initialize cluster info.
-        BitSet nD = new BitSet(d);
+        long[] nD = BitsUtil.zero(d);
 
         // Test each dimension and build bounding box.
         for(int k = 0; k < d; ++k) {
           if(dimensionIsRelevant(k, relation, randomSet)) {
-            nD.set(k);
+            BitsUtil.setI(nD, k);
           }
         }
-        if(nD.cardinality() > 0) {
+        if(BitsUtil.cardinality(nD) > 0) {
           // Get all points in the box.
           df.setSelectedDimensions(nD);
           // TODO: add filtering capabilities into query API!
-          DBIDs nC = DBIDUtil.intersection(S, rq.getRangeForDBID(iter, wd));
+          DBIDs nC = DBIDUtil.intersection(S, rq.getRangeForDBID(iter, w));
 
           if(LOG.isDebuggingFiner()) {
-            LOG.finer("Testing a cluster candidate, |C| = " + nC.size() + ", |D| = " + nD.cardinality());
+            LOG.finer("Testing a cluster candidate, |C| = " + nC.size() + ", |D| = " + BitsUtil.cardinality(nD));
           }
 
           // Is the cluster large enough?
@@ -290,7 +286,7 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
           }
           else {
             // Better cluster than before?
-            double nQuality = computeClusterQuality(nC.size(), nD.cardinality());
+            double nQuality = computeClusterQuality(nC.size(), BitsUtil.cardinality(nD));
             if(nQuality > quality) {
               if(LOG.isDebuggingFiner()) {
                 LOG.finer("... and it's the best so far: " + nQuality + " vs. " + quality);
@@ -306,23 +302,12 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
             }
           }
         }
-
-        if(iprogress != null) {
-          iprogress.incrementProcessed(LOG);
-        }
+        LOG.incrementProcessed(iprogress);
       }
     }
+    LOG.ensureCompleted(iprogress);
 
-    if(iprogress != null) {
-      iprogress.ensureCompleted(LOG);
-    }
-
-    if(C != null) {
-      return makeCluster(relation, C, D);
-    }
-    else {
-      return null;
-    }
+    return (C != null) ? makeCluster(relation, C, D) : null;
   }
 
   /**
@@ -336,9 +321,9 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
    * @param n Number of outer iterations (seed points).
    * @return a cluster, if one is found, else <code>null</code>.
    */
-  private Cluster<SubspaceModel<V>> runFastDOC(Relation<V> relation, ArrayModifiableDBIDs S, int d, int n, int m, int r) {
+  private Cluster<SubspaceModel> runFastDOC(Relation<V> relation, ArrayModifiableDBIDs S, int d, int n, int m, int r) {
     // Relevant attributes of highest cardinality.
-    BitSet D = null;
+    long[] D = null;
     // The seed point for the best dimensions.
     DBIDVar dV = DBIDUtil.newVar();
 
@@ -357,57 +342,46 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
         DBIDs randomSet = DBIDUtil.randomSample(S, Math.min(S.size(), r), random);
 
         // Initialize cluster info.
-        BitSet nD = new BitSet(d);
+        long[] nD = BitsUtil.zero(d);
 
         // Test each dimension.
         for(int k = 0; k < d; ++k) {
           if(dimensionIsRelevant(k, relation, randomSet)) {
-            nD.set(k);
+            BitsUtil.setI(nD, k);
           }
         }
 
-        if(D == null || nD.cardinality() > D.cardinality()) {
+        if(D == null || BitsUtil.cardinality(nD) > BitsUtil.cardinality(D)) {
           D = nD;
           dV.set(iter);
 
-          if(D.cardinality() >= d_zero) {
+          if(BitsUtil.cardinality(D) >= d_zero) {
             if(iprogress != null) {
               iprogress.setProcessed(iprogress.getTotal(), LOG);
             }
             break outer;
           }
         }
-
-        if(iprogress != null) {
-          iprogress.incrementProcessed(LOG);
-        }
+        LOG.incrementProcessed(iprogress);
       }
     }
-
-    if(iprogress != null) {
-      iprogress.ensureCompleted(LOG);
-    }
+    LOG.ensureCompleted(iprogress);
 
     // If no relevant dimensions were found, skip it.
-    if(D == null || D.cardinality() == 0) {
+    if(D == null || BitsUtil.cardinality(D) == 0) {
       return null;
     }
 
     // Get all points in the box.
     SubspaceMaximumDistanceFunction df = new SubspaceMaximumDistanceFunction(D);
-    DistanceQuery<V, DoubleDistance> dq = relation.getDatabase().getDistanceQuery(relation, df);
-    RangeQuery<V, DoubleDistance> rq = relation.getDatabase().getRangeQuery(dq, DatabaseQuery.HINT_SINGLE);
+    DistanceQuery<V> dq = relation.getDatabase().getDistanceQuery(relation, df);
+    RangeQuery<V> rq = relation.getDatabase().getRangeQuery(dq, DatabaseQuery.HINT_SINGLE);
 
     // TODO: add filtering capabilities into query API!
-    DBIDs C = DBIDUtil.intersection(S, rq.getRangeForDBID(dV, new DoubleDistance(w)));
+    DBIDs C = DBIDUtil.intersection(S, rq.getRangeForDBID(dV, w));
 
     // If we have a non-empty cluster, return it.
-    if(C.size() > 0) {
-      return makeCluster(relation, C, D);
-    }
-    else {
-      return null;
-    }
+    return (C.size() > 0) ? makeCluster(relation, C, D) : null;
   }
 
   /**
@@ -421,12 +395,11 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
    * @return <code>true</code> if the dimension is relevant.
    */
   private boolean dimensionIsRelevant(int dimension, Relation<V> relation, DBIDs points) {
-    double min = Double.POSITIVE_INFINITY;
-    double max = Double.NEGATIVE_INFINITY;
+    double min = Double.POSITIVE_INFINITY, max = Double.NEGATIVE_INFINITY;
     for(DBIDIter iter = points.iter(); iter.valid(); iter.advance()) {
-      V xV = relation.get(iter);
-      min = Math.min(min, xV.doubleValue(dimension));
-      max = Math.max(max, xV.doubleValue(dimension));
+      double xV = relation.get(iter).doubleValue(dimension);
+      min = (xV < min) ? xV : min;
+      max = (xV > max) ? xV : max;
       if(max - min > w) {
         return false;
       }
@@ -443,10 +416,10 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
    * @param D the relevant dimensions.
    * @return an object representing the subspace cluster.
    */
-  private Cluster<SubspaceModel<V>> makeCluster(Relation<V> relation, DBIDs C, BitSet D) {
+  private Cluster<SubspaceModel> makeCluster(Relation<V> relation, DBIDs C, long[] D) {
     DBIDs ids = DBIDUtil.newHashSet(C); // copy, also to lose distance values!
-    Cluster<SubspaceModel<V>> cluster = new Cluster<>(ids);
-    cluster.setModel(new SubspaceModel<>(new Subspace(D), Centroid.make(relation, ids).toVector(relation)));
+    Cluster<SubspaceModel> cluster = new Cluster<>(ids);
+    cluster.setModel(new SubspaceModel(new Subspace(D), Centroid.make(relation, ids)));
     return cluster;
   }
 
@@ -483,7 +456,7 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector> extends AbstractParameterizer {
     /**
      * Relative density threshold parameter Alpha.
      */
@@ -549,26 +522,26 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
       super.makeOptions(config);
 
       {
-        DoubleParameter param = new DoubleParameter(ALPHA_ID, 0.2);
-        param.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_DOUBLE);
-        param.addConstraint(CommonConstraints.LESS_EQUAL_ONE_DOUBLE);
+        DoubleParameter param = new DoubleParameter(ALPHA_ID, 0.2) //
+        .addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_DOUBLE) //
+        .addConstraint(CommonConstraints.LESS_EQUAL_ONE_DOUBLE);
         if(config.grab(param)) {
           alpha = param.getValue();
         }
       }
 
       {
-        DoubleParameter param = new DoubleParameter(BETA_ID, 0.8);
-        param.addConstraint(CommonConstraints.GREATER_THAN_ZERO_DOUBLE);
-        param.addConstraint(CommonConstraints.LESS_THAN_ONE_DOUBLE);
+        DoubleParameter param = new DoubleParameter(BETA_ID, 0.8) //
+        .addConstraint(CommonConstraints.GREATER_THAN_ZERO_DOUBLE) //
+        .addConstraint(CommonConstraints.LESS_THAN_ONE_DOUBLE);
         if(config.grab(param)) {
           beta = param.getValue();
         }
       }
 
       {
-        DoubleParameter param = new DoubleParameter(W_ID, 0.05);
-        param.addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_DOUBLE);
+        DoubleParameter param = new DoubleParameter(W_ID, 0.05) //
+        .addConstraint(CommonConstraints.GREATER_EQUAL_ZERO_DOUBLE);
         if(config.grab(param)) {
           w = param.getValue();
         }
@@ -582,8 +555,8 @@ public class DOC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering
       }
 
       if(heuristics) {
-        IntParameter param = new IntParameter(D_ZERO_ID, 5);
-        param.addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
+        IntParameter param = new IntParameter(D_ZERO_ID, 5) //
+        .addConstraint(CommonConstraints.GREATER_EQUAL_ONE_INT);
         if(config.grab(param)) {
           d_zero = param.getValue();
         }
