@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier.meta;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,8 +40,8 @@ import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
-import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.DoubleRelation;
+import de.lmu.ifi.dbs.elki.database.relation.MaterializedDoubleRelation;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
@@ -111,13 +111,9 @@ public class SimpleOutlierEnsemble extends AbstractAlgorithm<OutlierResult> impl
           results.add(or);
           ids.addDBIDs(or.getScores().getDBIDs());
         }
-        if (prog != null) {
-          prog.incrementProcessed(LOG);
-        }
-      }
-      if (prog != null) {
-        prog.ensureCompleted(LOG);
+        LOG.incrementProcessed(prog);
       }
+      LOG.ensureCompleted(prog);
     }
     // Combine
     WritableDoubleDataStore sumscore = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_STATIC);
@@ -128,8 +124,8 @@ public class SimpleOutlierEnsemble extends AbstractAlgorithm<OutlierResult> impl
         double[] scores = new double[num];
         int i = 0;
         for (OutlierResult r : results) {
-          Double score = r.getScores().get(id);
-          if (score != null) {
+          double score = r.getScores().doubleValue(id);
+          if (!Double.isNaN(score)) {
             scores[i] = score;
             i++;
           } else {
@@ -147,16 +143,12 @@ public class SimpleOutlierEnsemble extends AbstractAlgorithm<OutlierResult> impl
         } else {
           LOG.warning("DBID " + id + " was not given any score at all.");
         }
-        if (cprog != null) {
-          cprog.incrementProcessed(LOG);
-        }
-      }
-      if (cprog != null) {
-        cprog.ensureCompleted(LOG);
+        LOG.incrementProcessed(cprog);
       }
+      LOG.ensureCompleted(cprog);
     }
     OutlierScoreMeta meta = new BasicOutlierScoreMeta(minmax.getMin(), minmax.getMax());
-    Relation<Double> scores = new MaterializedRelation<>("Simple Outlier Ensemble", "ensemble-outlier", TypeUtil.DOUBLE, sumscore, ids);
+    DoubleRelation scores = new MaterializedDoubleRelation("Simple Outlier Ensemble", "ensemble-outlier", sumscore, ids);
     return new OutlierResult(meta, scores);
   }
 
