@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier.spatial.neighborhood;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -156,13 +156,9 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
           todo = ntodo;
         }
         store.put(iter, res);
-        if(progress != null) {
-          progress.incrementProcessed(LOG);
-        }
-      }
-      if(progress != null) {
-        progress.ensureCompleted(LOG);
+        LOG.incrementProcessed(progress);
       }
+      LOG.ensureCompleted(progress);
 
       return store;
     }
