@@ -1,26 +1,27 @@
 package de.lmu.ifi.dbs.elki.algorithm.outlier.spatial.neighborhood;
+
 /*
-This file is part of ELKI:
-Environment for Developing KDD-Applications Supported by Index-Structures
+ This file is part of ELKI:
+ Environment for Developing KDD-Applications Supported by Index-Structures
 
-Copyright (C) 2013
-Ludwig-Maximilians-Universität München
-Lehr- und Forschungseinheit für Datenbanksysteme
-ELKI Development Team
+ Copyright (C) 2014
+ Ludwig-Maximilians-Universität München
+ Lehr- und Forschungseinheit für Datenbanksysteme
+ ELKI Development Team
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as published by
+ the Free Software Foundation, either version 3 of the License, or
+ (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU Affero General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.database.QueryUtil;
@@ -32,11 +33,10 @@ import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
+import de.lmu.ifi.dbs.elki.database.ids.KNNList;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -48,10 +48,8 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * Neighborhoods based on k nearest neighbors.
  * 
  * @author Ahmed Hettab
- * 
- * @param <D> Distance to use
  */
-public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> extends AbstractPrecomputedNeighborhood {
+public class PrecomputedKNearestNeighborNeighborhood extends AbstractPrecomputedNeighborhood {
   /**
    * Logger
    */
@@ -88,11 +86,10 @@ public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> exte
    * 
    * @apiviz.stereotype factory
    * @apiviz.has PrecomputedKNearestNeighborNeighborhood
-   *
+   * 
    * @param <O> Object type
-   * @param <D> Distance type
    */
-  public static class Factory<O, D extends Distance<D>> implements NeighborSetPredicate.Factory<O> {
+  public static class Factory<O> implements NeighborSetPredicate.Factory<O> {
     /**
      * parameter k
      */
@@ -101,12 +98,12 @@ public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> exte
     /**
      * distance function to use
      */
-    private DistanceFunction<? super O, D> distFunc;
+    private DistanceFunction<? super O> distFunc;
 
     /**
      * Factory Constructor
      */
-    public Factory(int k, DistanceFunction<? super O, D> distFunc) {
+    public Factory(int k, DistanceFunction<? super O> distFunc) {
       super();
       this.k = k;
       this.distFunc = distFunc;
@@ -114,19 +111,19 @@ public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> exte
 
     @Override
     public NeighborSetPredicate instantiate(Relation<? extends O> relation) {
-      KNNQuery<?, D> knnQuery = QueryUtil.getKNNQuery(relation, distFunc);
+      KNNQuery<?> knnQuery = QueryUtil.getKNNQuery(relation, distFunc);
 
       // TODO: use bulk?
       WritableDataStore<DBIDs> s = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC, DBIDs.class);
       for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-        KNNList<D> neighbors = knnQuery.getKNNForDBID(iditer, k);
+        KNNList neighbors = knnQuery.getKNNForDBID(iditer, k);
         ArrayModifiableDBIDs neighbours = DBIDUtil.newArray(neighbors.size());
-        for (DBIDIter neighbor = neighbors.iter(); neighbor.valid(); neighbor.advance()) {
+        for(DBIDIter neighbor = neighbors.iter(); neighbor.valid(); neighbor.advance()) {
           neighbours.add(neighbor);
         }
         s.put(iditer, neighbours);
       }
-      return new PrecomputedKNearestNeighborNeighborhood<D>(s);
+      return new PrecomputedKNearestNeighborNeighborhood(s);
     }
 
     @Override
@@ -142,9 +139,8 @@ public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> exte
      * @apiviz.exclude
      * 
      * @param <O> Object type
-     * @param <D> Distance type
      */
-    public static class Parameterizer<O, D extends Distance<D>> extends AbstractParameterizer {
+    public static class Parameterizer<O> extends AbstractParameterizer {
       /**
        * Parameter k
        */
@@ -163,7 +159,7 @@ public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> exte
       /**
        * Distance function
        */
-      DistanceFunction<? super O, D> distFunc;
+      DistanceFunction<? super O> distFunc;
 
       @Override
       protected void makeOptions(Parameterization config) {
@@ -172,14 +168,14 @@ public class PrecomputedKNearestNeighborNeighborhood<D extends Distance<D>> exte
         if(config.grab(kP)) {
           k = kP.getValue();
         }
-        final ObjectParameter<DistanceFunction<? super O, D>> distP = new ObjectParameter<>(DISTANCEFUNCTION_ID, DistanceFunction.class);
+        final ObjectParameter<DistanceFunction<? super O>> distP = new ObjectParameter<>(DISTANCEFUNCTION_ID, DistanceFunction.class);
         if(config.grab(distP)) {
           distFunc = distP.instantiateClass(config);
         }
       }
 
       @Override
-      protected PrecomputedKNearestNeighborNeighborhood.Factory<O, D> makeInstance() {
+      protected PrecomputedKNearestNeighborNeighborhood.Factory<O> makeInstance() {
         return new PrecomputedKNearestNeighborNeighborhood.Factory<>(k, distFunc);
       }
     }
