@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.statistics;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -47,7 +47,6 @@ import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.StepProgress;
@@ -76,12 +75,10 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Parameter;
  * @author Erich Schubert
  * 
  * @param <O> Object type
- * @param <D> Distance type
  */
-// TODO: optimize for double distances.
 @Title("Distance Histogram")
 @Description("Computes a histogram over the distances occurring in the data set.")
-public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm<O, D, CollectionResult<DoubleVector>> {
+public class DistanceStatisticsWithClasses<O> extends AbstractDistanceBasedAlgorithm<O, CollectionResult<DoubleVector>> {
   /**
    * The logger for this class.
    */
@@ -125,7 +122,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
    * @param exact Exactness flag
    * @param sampling Sampling flag
    */
-  public DistanceStatisticsWithClasses(DistanceFunction<? super O, D> distanceFunction, int numbins, boolean exact, boolean sampling) {
+  public DistanceStatisticsWithClasses(DistanceFunction<? super O> distanceFunction, int numbins, boolean exact, boolean sampling) {
     super(distanceFunction);
     this.numbin = numbins;
     this.exact = exact;
@@ -135,7 +132,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
   @Override
   public HistogramResult<DoubleVector> run(Database database) {
     final Relation<O> relation = database.getRelation(getInputTypeRestriction()[0]);
-    final DistanceQuery<O, D> distFunc = database.getDistanceQuery(relation, getDistanceFunction());
+    final DistanceQuery<O> distFunc = database.getDistanceQuery(relation, getDistanceFunction());
 
     final StepProgress stepprog = LOG.isVerbose() ? new StepProgress("Distance statistics", 2) : null;
 
@@ -159,9 +156,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     MeanVariance modif = new MeanVariance();
     // Histogram
     final ObjHistogram<long[]> histogram;
-    if(stepprog != null) {
-      stepprog.beginStep(1, "Prepare histogram.", LOG);
-    }
+    LOG.beginStep(stepprog, 1, "Prepare histogram.");
     if(exact) {
       gminmax = exactMinMax(relation, distFunc);
       histogram = new LongArrayStaticHistogram(numbin, gminmax.getMin(), gminmax.getMax(), 2);
@@ -206,9 +201,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
       };
     }
 
-    if(stepprog != null) {
-      stepprog.beginStep(2, "Build histogram.", LOG);
-    }
+    LOG.beginStep(stepprog, 2, "Build histogram.");
     final FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Distance computations", relation.size(), LOG) : null;
     // iterate per cluster
     final long[] incFirst = new long[] { 1L, 0L };
@@ -222,7 +215,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
           if(DBIDUtil.equal(id1, iter2)) {
             continue;
           }
-          double d = distFunc.distance(id1, iter2).doubleValue();
+          double d = distFunc.distance(id1, iter2);
 
           histogram.putData(d, incFirst);
 
@@ -247,7 +240,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
             if(DBIDUtil.equal(id1, iter2)) {
               continue;
             }
-            double d = distFunc.distance(id1, iter2).doubleValue();
+            double d = distFunc.distance(id1, iter2);
 
             histogram.putData(d, incSecond);
 
@@ -261,21 +254,15 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
         // min/max
         gominmax.put(ominmax.getMin());
         gominmax.put(ominmax.getMax());
-        if(progress != null) {
-          progress.incrementProcessed(LOG);
-        }
+        LOG.incrementProcessed(progress);
       }
     }
-    if(progress != null) {
-      progress.ensureCompleted(LOG);
-    }
+    LOG.ensureCompleted(progress);
     // Update values (only needed for sampling case).
     gminmax.setFirst(Math.min(giminmax.getMin(), gominmax.getMin()));
     gminmax.setSecond(Math.max(giminmax.getMax(), gominmax.getMax()));
 
-    if(stepprog != null) {
-      stepprog.setCompleted(LOG);
-    }
+    LOG.setCompleted(stepprog);
 
     // count the number of samples we have in the data
     long inum = 0;
@@ -316,7 +303,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
    * @param distFunc Distance function to use
    * @return Minimum and maximum
    */
-  private DoubleMinMax sampleMinMax(Relation<O> relation, DistanceQuery<O, D> distFunc) {
+  private DoubleMinMax sampleMinMax(Relation<O> relation, DistanceQuery<O> distFunc) {
     int size = relation.size();
     Random rnd = new Random();
     // estimate minimum and maximum.
@@ -344,12 +331,12 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
         if(DBIDUtil.equal(iter, pair)) {
           continue;
         }
-        double d = distFunc.distance(iter, pair).doubleValue();
+        double d = distFunc.distance(iter, pair);
         np.add(DBIDUtil.newPair(d, iter));
         np.add(DBIDUtil.newPair(d, pair));
       }
       for(DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
-        double d = distFunc.distance(iter, iter2).doubleValue();
+        double d = distFunc.distance(iter, iter2);
         np.add(DBIDUtil.newPair(d, iter));
         np.add(DBIDUtil.newPair(d, iter2));
       }
@@ -363,12 +350,12 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
         if(DBIDUtil.equal(iter, pair)) {
           continue;
         }
-        double d = distFunc.distance(iter, pair).doubleValue();
+        double d = distFunc.distance(iter, pair);
         np2.add(DBIDUtil.newPair(d, iter));
         np2.add(DBIDUtil.newPair(d, pair));
       }
       for(DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
-        double d = distFunc.distance(iter, iter2).doubleValue();
+        double d = distFunc.distance(iter, iter2);
         np.add(DBIDUtil.newPair(d, iter));
         np.add(DBIDUtil.newPair(d, iter2));
       }
@@ -393,7 +380,8 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
    * @param distFunc Distance function
    * @return Exact maximum and minimum
    */
-  private DoubleMinMax exactMinMax(Relation<O> relation, DistanceQuery<O, D> distFunc) {
+  private DoubleMinMax exactMinMax(Relation<O> relation, DistanceQuery<O> distFunc) {
+    final FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Exact fitting distance computations", relation.size(), LOG) : null;
     DoubleMinMax minmax = new DoubleMinMax();
     // find exact minimum and maximum first.
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
@@ -402,10 +390,12 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
         if(DBIDUtil.equal(iditer, iditer2)) {
           continue;
         }
-        double d = distFunc.distance(iditer, iditer2).doubleValue();
+        double d = distFunc.distance(iditer, iditer2);
         minmax.put(d);
       }
+      LOG.incrementProcessed(progress);
     }
+    LOG.ensureCompleted(progress);
     return minmax;
   }
 
@@ -447,8 +437,10 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
    * @author Erich Schubert
    * 
    * @apiviz.exclude
+   * 
+   * @param <O> Object type
    */
-  public static class Parameterizer<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<O, D> {
+  public static class Parameterizer<O> extends AbstractDistanceBasedAlgorithm.Parameterizer<O> {
     /**
      * Number of bins to use in sampling.
      */
@@ -490,7 +482,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     }
 
     @Override
-    protected DistanceStatisticsWithClasses<O, D> makeInstance() {
+    protected DistanceStatisticsWithClasses<O> makeInstance() {
       return new DistanceStatisticsWithClasses<>(distanceFunction, numbin, exact, sampling);
     }
   }
