@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application.cache;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,23 +28,24 @@ import java.io.IOException;
 
 import de.lmu.ifi.dbs.elki.application.AbstractApplication;
 import de.lmu.ifi.dbs.elki.database.Database;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRange;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.external.DiskCacheBasedFloatDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.persistent.OnDiskUpperTriangleMatrix;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
+import de.lmu.ifi.dbs.elki.utilities.io.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.FileParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 import de.lmu.ifi.dbs.elki.workflow.InputStep;
 
 /**
- * Precompute an on-disk distance matrix, using double precision.
+ * Precompute an on-disk distance matrix, using float precision.
  * 
  * @author Erich Schubert
  * 
@@ -52,9 +53,8 @@ import de.lmu.ifi.dbs.elki.workflow.InputStep;
  * @apiviz.has DistanceFunction
  * 
  * @param <O> Object type
- * @param <D> Distance type
  */
-public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>> extends AbstractApplication {
+public class CacheFloatDistanceInOnDiskMatrix<O> extends AbstractApplication {
   /**
    * The logger for this class.
    */
@@ -65,11 +65,6 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
    */
   private static final boolean debugExtraCheckSymmetry = false;
 
-  /**
-   * Storage size: 4 bytes floats
-   */
-  private static final int FLOAT_SIZE = 4;
-
   /**
    * Data source to process.
    */
@@ -78,7 +73,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
   /**
    * Distance function that is to be cached.
    */
-  private DistanceFunction<O, D> distance;
+  private DistanceFunction<O> distance;
 
   /**
    * Output file.
@@ -92,7 +87,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
    * @param distance Distance function
    * @param out Matrix output file
    */
-  public CacheFloatDistanceInOnDiskMatrix(InputStep input, DistanceFunction<O, D> distance, File out) {
+  public CacheFloatDistanceInOnDiskMatrix(InputStep input, DistanceFunction<O> distance, File out) {
     super();
     this.input = input;
     this.distance = distance;
@@ -103,40 +98,35 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
   public void run() {
     Database database = input.getDatabase();
     Relation<O> relation = database.getRelation(distance.getInputTypeRestriction());
-    DistanceQuery<O, D> distanceQuery = database.getDistanceQuery(relation, distance);
+    DistanceQuery<O> distanceQuery = database.getDistanceQuery(relation, distance);
 
-    int matrixsize = 0;
-    for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      final int intid = DBIDUtil.asInteger(iditer);
-      matrixsize = Math.max(matrixsize, intid + 1);
-      if (intid < 0) {
-        throw new AbortException("OnDiskMatrixCache does not allow negative DBIDs.");
-      }
-    }
+    DBIDRange ids = DBIDUtil.assertRange(relation.getDBIDs());
+    int matrixsize = ids.size();
 
     OnDiskUpperTriangleMatrix matrix;
     try {
-      matrix = new OnDiskUpperTriangleMatrix(out, DiskCacheBasedFloatDistanceFunction.FLOAT_CACHE_MAGIC, 0, FLOAT_SIZE, matrixsize);
-    } catch (IOException e) {
+      matrix = new OnDiskUpperTriangleMatrix(out, DiskCacheBasedFloatDistanceFunction.FLOAT_CACHE_MAGIC, 0, ByteArrayUtil.SIZE_FLOAT, matrixsize);
+    }
+    catch(IOException e) {
       throw new AbortException("Error creating output matrix.", e);
     }
 
-    for (DBIDIter id1 = relation.iterDBIDs(); id1.valid(); id1.advance()) {
-      for (DBIDIter id2 = relation.iterDBIDs(); id2.valid(); id2.advance()) {
-        if (DBIDUtil.asInteger(id2) >= DBIDUtil.asInteger(id1)) {
-          float d = distanceQuery.distance(id1, id2).floatValue();
-          if (debugExtraCheckSymmetry) {
-            float d2 = distanceQuery.distance(id2, id1).floatValue();
-            if (Math.abs(d - d2) > 0.0000001) {
-              LOG.warning("Distance function doesn't appear to be symmetric!");
-            }
-          }
-          try {
-            matrix.getRecordBuffer(DBIDUtil.asInteger(id1), DBIDUtil.asInteger(id2)).putFloat(d);
-          } catch (IOException e) {
-            throw new AbortException("Error writing distance record " + id1 + "," + id2 + " to matrix.", e);
+    DBIDArrayIter id1 = ids.iter(), id2 = ids.iter();
+    for(; id1.valid(); id1.advance()) {
+      for(id2.seek(id1.getOffset()); id2.valid(); id2.advance()) {
+        float d = (float) distanceQuery.distance(id1, id2);
+        if(debugExtraCheckSymmetry) {
+          float d2 = (float) distanceQuery.distance(id2, id1);
+          if(Math.abs(d - d2) > 0.0000001) {
+            LOG.warning("Distance function doesn't appear to be symmetric!");
           }
         }
+        try {
+          matrix.getRecordBuffer(id1.getOffset(), id2.getOffset()).putFloat(d);
+        }
+        catch(IOException e) {
+          throw new AbortException("Error writing distance record " + DBIDUtil.toString(id1) + "," + DBIDUtil.toString(id2) + " to matrix.", e);
+        }
       }
     }
   }
@@ -148,7 +138,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<O, D extends NumberDistance<D, ?>> extends AbstractApplication.Parameterizer {
+  public static class Parameterizer<O> extends AbstractApplication.Parameterizer {
     /**
      * Data source to process.
      */
@@ -157,7 +147,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
     /**
      * Distance function that is to be cached.
      */
-    private DistanceFunction<O, D> distance = null;
+    private DistanceFunction<O> distance = null;
 
     /**
      * Output file.
@@ -169,19 +159,19 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
       super.makeOptions(config);
       input = config.tryInstantiate(InputStep.class);
       // Distance function parameter
-      final ObjectParameter<DistanceFunction<O, D>> dpar = new ObjectParameter<>(CacheDoubleDistanceInOnDiskMatrix.Parameterizer.DISTANCE_ID, DistanceFunction.class);
-      if (config.grab(dpar)) {
+      final ObjectParameter<DistanceFunction<O>> dpar = new ObjectParameter<>(CacheDoubleDistanceInOnDiskMatrix.Parameterizer.DISTANCE_ID, DistanceFunction.class);
+      if(config.grab(dpar)) {
         distance = dpar.instantiateClass(config);
       }
       // Output file parameter
       final FileParameter cpar = new FileParameter(CacheDoubleDistanceInOnDiskMatrix.Parameterizer.CACHE_ID, FileParameter.FileType.OUTPUT_FILE);
-      if (config.grab(cpar)) {
+      if(config.grab(cpar)) {
         out = cpar.getValue();
       }
     }
 
     @Override
-    protected CacheFloatDistanceInOnDiskMatrix<O, D> makeInstance() {
+    protected CacheFloatDistanceInOnDiskMatrix<O> makeInstance() {
       return new CacheFloatDistanceInOnDiskMatrix<>(input, distance, out);
     }
   }
