@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application.geo;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -175,13 +175,9 @@ public class VisualizeGeodesicDistances extends AbstractApplication {
         }
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
-    }
-    if (prog != null) {
-      prog.ensureCompleted(LOG);
+      LOG.incrementProcessed(prog);
     }
+    LOG.ensureCompleted(prog);
 
     try {
       ImageIO.write(img, "png", out);
