@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -33,15 +33,14 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.NumberArrayAdapter
  * 
  * @author Arthur Zimek
  * 
- * @param <N> the type of the attribute values
- * 
  * @apiviz.landmark
  * @apiviz.has Vector
+ * @apiviz.exclude de.lmu.ifi.dbs.elki.(datasource.filter|data.projection).*
  */
-public interface NumberVector<N extends Number> extends FeatureVector<N>, SpatialComparable {
+public interface NumberVector extends FeatureVector<Number>, SpatialComparable {
   @Deprecated
   @Override
-  N getValue(int dimension);
+  Number getValue(int dimension);
   
   /**
    * Returns the value in the specified dimension as double.
@@ -139,9 +138,8 @@ public interface NumberVector<N extends Number> extends FeatureVector<N>, Spatia
    * @apiviz.has NumberVector
    * 
    * @param <V> Vector type
-   * @param <N> Data type of vector
    */
-  interface Factory<V extends NumberVector<? extends N>, N extends Number> extends FeatureVector.Factory<V, N> {
+  interface Factory<V extends NumberVector> extends FeatureVector.Factory<V, Number> {
     /**
      * Returns a new NumberVector of N for the given values.
      * 
@@ -150,6 +148,14 @@ public interface NumberVector<N extends Number> extends FeatureVector<N>, Spatia
      */
     V newNumberVector(double[] values);
 
+    /**
+     * Returns a new NumberVector of N for the given values.
+     * 
+     * @param values Existing number vector
+     * @return a new NumberVector of N for the given values
+     */
+    V newNumberVector(NumberVector values);
+
     /**
      * Instantiate from any number-array like object.
      * 
