@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data.model;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -38,7 +38,7 @@ import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
  * 
  * @param <V> Vector type
  */
-public class CorrelationModel<V extends FeatureVector<?>> extends BaseModel implements TextWriteable {
+public class CorrelationModel<V extends FeatureVector<?>> extends AbstractModel implements TextWriteable {
   /**
    * The computed PCA result of this cluster.
    */
@@ -104,7 +104,7 @@ public class CorrelationModel<V extends FeatureVector<?>> extends BaseModel impl
    */
   @Override
   public void writeToText(TextWriterStream out, String label) {
-    out.commentPrintLn(TextWriterStream.SER_MARKER + " " + CorrelationModel.class.getName());
+    super.writeToText(out, label);
     out.commentPrintLn("Centroid: " + out.normalizationRestore(getCentroid()).toString());
     out.commentPrintLn("Strong Eigenvectors:");
     String strong = getPCAResult().getStrongEigenvectors().toString();
@@ -118,6 +118,6 @@ public class CorrelationModel<V extends FeatureVector<?>> extends BaseModel impl
       weak = weak.substring(0, weak.length() - 1);
     }
     out.commentPrintLn(weak);
-    out.commentPrintLn("Eigenvalues: " + FormatUtil.format(getPCAResult().getEigenvalues(), " ", 2));
+    out.commentPrintLn("Eigenvalues: " + FormatUtil.format(getPCAResult().getEigenvalues(), " ", FormatUtil.NF2));
   }
 }
