@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data.model;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,56 +23,33 @@ package de.lmu.ifi.dbs.elki.data.model;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.data.FeatureVector;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.result.textwriter.TextWriteable;
-import de.lmu.ifi.dbs.elki.result.textwriter.TextWriterStream;
 
 /**
  * Cluster model that stores a mean for the cluster.
  * 
  * @author Erich Schubert
- * 
- * @param <V> Vector type
  */
-public class MeanModel<V extends FeatureVector<?>> extends BaseModel implements TextWriteable{
-  /**
-   * Cluster mean
-   */
-  private V mean;
-
+public class MeanModel extends PrototypeModel<Vector> implements TextWriteable {
   /**
    * Constructor with mean
    * 
    * @param mean Cluster mean
    */
-  public MeanModel(V mean) {
-    super();
-    this.mean = mean;
+  public MeanModel(Vector mean) {
+    super(mean);
   }
 
   /**
    * @return mean
    */
-  public V getMean() {
-    return mean;
+  public Vector getMean() {
+    return prototype;
   }
 
-  /**
-   * @param mean Mean vector
-   */
-  public void setMean(V mean) {
-    this.mean = mean;
-  }
-
-  /**
-   * Implementation of {@link TextWriteable} interface.
-   */
   @Override
-  public void writeToText(TextWriterStream out, String label) {
-    if(label != null) {
-      out.commentPrintLn(label);
-    }
-    out.commentPrintLn(TextWriterStream.SER_MARKER + " " + getClass().getName());
-    out.commentPrintLn("Cluster Mean: " + mean.toString());
+  protected String getPrototypeType() {
+    return "Mean";
   }
 }
