@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.data.model;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,52 +25,31 @@ package de.lmu.ifi.dbs.elki.data.model;
 
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.result.textwriter.TextWriteable;
-import de.lmu.ifi.dbs.elki.result.textwriter.TextWriterStream;
 
 /**
  * Cluster model that stores a mean for the cluster.
  * 
  * @author Erich Schubert
  */
-public class MedoidModel extends BaseModel implements TextWriteable {
-  /**
-   * Cluster medoid
-   */
-  private DBID medoid;
-
+public class MedoidModel extends PrototypeModel<DBID> implements TextWriteable {
   /**
    * Constructor with medoid
    * 
    * @param medoid Cluster medoid
    */
   public MedoidModel(DBID medoid) {
-    super();
-    this.medoid = medoid;
+    super(medoid);
   }
 
   /**
    * @return medoid
    */
   public DBID getMedoid() {
-    return medoid;
+    return prototype;
   }
 
-  /**
-   * @param medoid Medoid object
-   */
-  public void setMedoid(DBID medoid) {
-    this.medoid = medoid;
-  }
-
-  /**
-   * Implementation of {@link TextWriteable} interface.
-   */
   @Override
-  public void writeToText(TextWriterStream out, String label) {
-    if(label != null) {
-      out.commentPrintLn(label);
-    }
-    out.commentPrintLn(TextWriterStream.SER_MARKER + " " + getClass().getName());
-    out.commentPrintLn("Cluster Medoid: " + medoid.toString());
+  protected String getPrototypeType() {
+    return "Medoid";
   }
 }
