@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.datastore;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -61,4 +61,18 @@ public interface WritableDoubleDataStore extends DoubleDataStore, WritableDataSt
    * @return previous value
    */
   public double put(DBIDRef id, double value);
+
+
+  /**
+   * Increment the specified value with the specified id in this storage.
+   * 
+   * @param id Database ID.
+   * @param value Value to add to the previous value.
+   */
+  public void increment(DBIDRef id, double value);
+
+  /**
+   * Reinitialize (reset to default value).
+   */
+  public void clear();
 }
\ No newline at end of file
