@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,5 +30,15 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.iterator.ArrayIter;
  * @author Erich Schubert
  */
 public interface DBIDArrayIter extends DBIDIter, ArrayIter {
-  // Nothing added - see {@link ArrayIter}!
+  @Override
+  DBIDArrayIter advance();
+
+  @Override
+  DBIDArrayIter advance(int count);
+
+  @Override
+  DBIDArrayIter retract();
+
+  @Override
+  DBIDArrayIter seek(int off);
 }
