@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,8 +23,6 @@ package de.lmu.ifi.dbs.elki.database.ids;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.utilities.pairs.PairInterface;
-
 /**
  * Immutable pair of two DBIDs. This can be stored more efficiently than when
  * using {@link de.lmu.ifi.dbs.elki.utilities.pairs.Pair}
@@ -33,21 +31,24 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.PairInterface;
  * 
  * @apiviz.composedOf de.lmu.ifi.dbs.elki.database.ids.DBID
  */
-// TODO: implement DBIDs?
-public interface DBIDPair extends PairInterface<DBID, DBID> {
+public interface DBIDPair extends DBIDs {
   /**
-   * Getter for first
+   * Getter for first.
    * 
    * @return first element in pair
+   * @deprecated This method can be expensive. The use of a {@link DBIDVar} is
+   *             recommended when many such accesses are needed.
    */
-  @Override
+  @Deprecated
   public DBID getFirst();
 
   /**
    * Getter for second element in pair
    * 
    * @return second element in pair
+   * @deprecated This method can be expensive. The use of a {@link DBIDVar} is
+   *             recommended when many such accesses are needed.
    */
-  @Override
+  @Deprecated
   public DBID getSecond();
 }
\ No newline at end of file
