@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -39,4 +39,30 @@ public interface DBIDVar extends DBIDRef, ArrayDBIDs, SetDBIDs {
    * @param ref Reference
    */
   void set(DBIDRef ref);
+
+  /**
+   * Clear the contents.
+   */
+  void unset();
+
+  /**
+   * Test if the variable is well-defined.
+   * 
+   * @return {@code true} when assigned.
+   */
+  boolean isSet();
+
+  /**
+   * Assign the first pair member to this variable.
+   * 
+   * @param pair Pair
+   */
+  void setFirst(DBIDPair pair);
+
+  /**
+   * Assign the second pair member to this variable.
+   * 
+   * @param pair Pair
+   */
+  void setSecond(DBIDPair pair);
 }
