@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids.generic;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -35,6 +35,8 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 /**
  * View on an ArrayDBIDs masked using a BitMask for efficient mask changing.
  * 
+ * FIXME: switch to long[] instead of BitSet
+ * 
  * @author Erich Schubert
  * 
  * @apiviz.uses DBIDs
@@ -137,8 +139,9 @@ public class MaskedDBIDs implements DBIDs {
     }
 
     @Override
-    public void advance() {
+    public DBIDIter advance() {
       pos = bits.nextSetBit(pos + 1);
+      return this;
     }
 
     @Override
@@ -180,8 +183,9 @@ public class MaskedDBIDs implements DBIDs {
     }
 
     @Override
-    public void advance() {
+    public DBIDIter advance() {
       pos = bits.nextClearBit(pos + 1);
+      return this;
     }
 
     @Override
