@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids.generic;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -109,8 +109,9 @@ public class UnmodifiableDBIDs implements StaticDBIDs {
     }
 
     @Override
-    public void advance() {
+    public DBIDIter advance() {
       it.advance();
+      return this;
     }
 
     @Override
