@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids.integer;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -32,9 +32,9 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDVar;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
-import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
-import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
-import de.lmu.ifi.dbs.elki.persistent.FixedSizeByteBufferSerializer;
+import de.lmu.ifi.dbs.elki.utilities.io.ByteArrayUtil;
+import de.lmu.ifi.dbs.elki.utilities.io.ByteBufferSerializer;
+import de.lmu.ifi.dbs.elki.utilities.io.FixedSizeByteBufferSerializer;
 
 /**
  * Database ID object.
@@ -185,23 +185,27 @@ final class IntegerDBID implements DBID, IntegerDBIDRef {
     int pos = 0;
 
     @Override
-    public void advance() {
+    public Itr advance() {
       pos++;
+      return this;
     }
 
     @Override
-    public void advance(int count) {
+    public Itr advance(int count) {
       pos += count;
+      return this;
     }
 
     @Override
-    public void retract() {
+    public Itr retract() {
       pos--;
+      return this;
     }
 
     @Override
-    public void seek(int off) {
+    public Itr seek(int off) {
       pos = off;
+      return this;
     }
 
     @Override
@@ -248,7 +252,7 @@ final class IntegerDBID implements DBID, IntegerDBIDRef {
     /**
      * Constructor. Protected: use static instance!
      */
-    protected DynamicSerializer() {
+    public DynamicSerializer() {
       super();
     }
 
@@ -277,7 +281,7 @@ final class IntegerDBID implements DBID, IntegerDBIDRef {
     /**
      * Constructor. Protected: use static instance!
      */
-    protected StaticSerializer() {
+    public StaticSerializer() {
       super();
     }
 
