@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids.integer;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,6 +24,7 @@ package de.lmu.ifi.dbs.elki.database.ids.integer;
  */
 
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDMIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDVar;
@@ -65,7 +66,7 @@ public class UnmodifiableIntegerArrayDBIDs implements IntegerArrayStaticDBIDs {
   @Override
   public IntegerDBIDArrayIter iter() {
     IntegerDBIDArrayIter it = inner.iter();
-    if (it instanceof DBIDMIter) {
+    if(it instanceof DBIDMIter) {
       return new UnmodifiableDBIDIter(it);
     }
     return it;
@@ -128,23 +129,27 @@ public class UnmodifiableIntegerArrayDBIDs implements IntegerArrayStaticDBIDs {
     }
 
     @Override
-    public void advance() {
+    public DBIDArrayIter advance() {
       it.advance();
+      return this;
     }
 
     @Override
-    public void advance(int count) {
+    public DBIDArrayIter advance(int count) {
       it.advance(count);
+      return this;
     }
 
     @Override
-    public void retract() {
+    public DBIDArrayIter retract() {
       it.retract();
+      return this;
     }
 
     @Override
-    public void seek(int off) {
+    public DBIDArrayIter seek(int off) {
       it.seek(off);
+      return this;
     }
 
     @Override
