@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.query;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,17 +25,15 @@ package de.lmu.ifi.dbs.elki.database.query;
 
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.similarity.SimilarityQuery;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
- * Interface that is a combination of distance and a similarity function.
- * For combined implementations of both.
+ * Interface that is a combination of distance and a similarity function. For
+ * combined implementations of both.
  * 
  * @author Erich Schubert
- *
+ * 
  * @param <O> Object type
- * @param <D> Distance type
  */
-public interface DistanceSimilarityQuery<O, D extends Distance<D>> extends DistanceQuery<O, D>, SimilarityQuery<O, D> {
+public interface DistanceSimilarityQuery<O> extends DistanceQuery<O>, SimilarityQuery<O> {
   // Empty
 }
