@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.query.knn;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,10 +30,8 @@ import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
-import de.lmu.ifi.dbs.elki.database.query.AbstractDataBasedQuery;
+import de.lmu.ifi.dbs.elki.database.ids.KNNList;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.preprocessed.knn.AbstractMaterializeKNNPreprocessor;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
@@ -42,12 +40,19 @@ import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
  * Instance for a particular database, invoking the preprocessor.
  * 
  * @author Erich Schubert
+ * 
+ * @param <O> Data object type
  */
-public class PreprocessorKNNQuery<O, D extends Distance<D>, T extends KNNList<D>> extends AbstractDataBasedQuery<O> implements KNNQuery<O, D> {
+public class PreprocessorKNNQuery<O> implements KNNQuery<O> {
+  /**
+   * The data to use for this query
+   */
+  final protected Relation<? extends O> relation;
+
   /**
    * The last preprocessor result
    */
-  final private AbstractMaterializeKNNPreprocessor<O, D, T> preprocessor;
+  final private AbstractMaterializeKNNPreprocessor<O> preprocessor;
 
   /**
    * Warn only once.
@@ -57,36 +62,27 @@ public class PreprocessorKNNQuery<O, D extends Distance<D>, T extends KNNList<D>
   /**
    * Constructor.
    * 
-   * @param database Database to query
+   * @param relation Relation to query
    * @param preprocessor Preprocessor instance to use
    */
-  public PreprocessorKNNQuery(Relation<O> database, AbstractMaterializeKNNPreprocessor<O, D, T> preprocessor) {
-    super(database);
+  public PreprocessorKNNQuery(Relation<O> relation, AbstractMaterializeKNNPreprocessor<O> preprocessor) {
+    super();
+    this.relation = relation;
     this.preprocessor = preprocessor;
   }
 
-  /**
-   * Constructor.
-   * 
-   * @param database Database to query
-   * @param preprocessor Preprocessor to use
-   */
-  public PreprocessorKNNQuery(Relation<O> database, AbstractMaterializeKNNPreprocessor.Factory<O, D, T> preprocessor) {
-    this(database, preprocessor.instantiate(database));
-  }
-
   @Override
-  public KNNList<D> getKNNForDBID(DBIDRef id, int k) {
+  public KNNList getKNNForDBID(DBIDRef id, int k) {
     if(!warned && k > preprocessor.getK()) {
       LoggingUtil.warning("Requested more neighbors than preprocessed!");
     }
     if(!warned && k < preprocessor.getK()) {
-      KNNList<D> dr = preprocessor.get(id);
+      KNNList dr = preprocessor.get(id);
       int subk = k;
-      D kdist = dr.get(subk - 1).getDistance();
+      double kdist = dr.get(subk - 1).doubleValue();
       while(subk < dr.size()) {
-        D ndist = dr.get(subk).getDistance();
-        if(kdist.equals(ndist)) {
+        double ndist = dr.get(subk).doubleValue();
+        if(kdist == ndist) {
           // Tie - increase subk.
           subk++;
         }
@@ -105,19 +101,19 @@ public class PreprocessorKNNQuery<O, D extends Distance<D>, T extends KNNList<D>
   }
 
   @Override
-  public List<KNNList<D>> getKNNForBulkDBIDs(ArrayDBIDs ids, int k) {
+  public List<KNNList> getKNNForBulkDBIDs(ArrayDBIDs ids, int k) {
     if(!warned && k > preprocessor.getK()) {
       LoggingUtil.warning("Requested more neighbors than preprocessed!");
     }
-    List<KNNList<D>> result = new ArrayList<>(ids.size());
+    List<KNNList> result = new ArrayList<>(ids.size());
     if(k < preprocessor.getK()) {
-      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {      
-        KNNList<D> dr = preprocessor.get(iter);
+      for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+        KNNList dr = preprocessor.get(iter);
         int subk = k;
-        D kdist = dr.get(subk - 1).getDistance();
+        double kdist = dr.get(subk - 1).doubleValue();
         while(subk < dr.size()) {
-          D ndist = dr.get(subk).getDistance();
-          if(kdist.equals(ndist)) {
+          double ndist = dr.get(subk).doubleValue();
+          if(kdist == ndist) {
             // Tie - increase subk.
             subk++;
           }
@@ -134,7 +130,7 @@ public class PreprocessorKNNQuery<O, D extends Distance<D>, T extends KNNList<D>
       }
     }
     else {
-      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {      
+      for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
         result.add(preprocessor.get(iter));
       }
     }
@@ -142,7 +138,7 @@ public class PreprocessorKNNQuery<O, D extends Distance<D>, T extends KNNList<D>
   }
 
   @Override
-  public KNNList<D> getKNNForObject(O obj, int k) {
+  public KNNList getKNNForObject(O obj, int k) {
     throw new AbortException("Preprocessor KNN query only supports ID queries.");
   }
 
@@ -151,7 +147,7 @@ public class PreprocessorKNNQuery<O, D extends Distance<D>, T extends KNNList<D>
    * 
    * @return preprocessor instance
    */
-  public AbstractMaterializeKNNPreprocessor<O, D, T> getPreprocessor() {
+  public AbstractMaterializeKNNPreprocessor<O> getPreprocessor() {
     return preprocessor;
   }
 }
\ No newline at end of file
