@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.query.range;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,10 +25,12 @@ package de.lmu.ifi.dbs.elki.database.query.range;
 
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
-import de.lmu.ifi.dbs.elki.database.ids.generic.GenericDistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.ModifiableDoubleDBIDList;
 import de.lmu.ifi.dbs.elki.database.query.distance.PrimitiveDistanceQuery;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.PrimitiveDistanceFunction;
 
 /**
  * Default linear scan range query class.
@@ -41,43 +43,57 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
  * @apiviz.uses PrimitiveDistanceQuery
  * 
  * @param <O> Database object type
- * @param <D> Distance type
  */
-public class LinearScanPrimitiveDistanceRangeQuery<O, D extends Distance<D>> extends AbstractDistanceRangeQuery<O, D> {
+public class LinearScanPrimitiveDistanceRangeQuery<O> extends AbstractDistanceRangeQuery<O> {
+  /**
+   * Unboxed distance function.
+   */
+  private PrimitiveDistanceFunction<? super O> rawdist;
+
   /**
    * Constructor.
    * 
    * @param distanceQuery Distance function to use
    */
-  public LinearScanPrimitiveDistanceRangeQuery(PrimitiveDistanceQuery<O, D> distanceQuery) {
+  public LinearScanPrimitiveDistanceRangeQuery(PrimitiveDistanceQuery<O> distanceQuery) {
     super(distanceQuery);
+    rawdist = distanceQuery.getDistanceFunction();
   }
 
   @Override
-  public DistanceDBIDList<D> getRangeForDBID(DBIDRef id, D range) {
+  public DoubleDBIDList getRangeForDBID(DBIDRef id, double range) {
     // Note: subtle optimization. Get "id" only once!
     final O obj = relation.get(id);
-    GenericDistanceDBIDList<D> result = new GenericDistanceDBIDList<>();
-    for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
-      D currentDistance = distanceQuery.distance(obj, iter);
-      if(currentDistance.compareTo(range) <= 0) {
-        result.add(currentDistance, iter);
-      }
-    }
+    ModifiableDoubleDBIDList result = DBIDUtil.newDistanceDBIDList();
+    linearScan(relation, relation.iterDBIDs(), obj, range, result);
     result.sort();
     return result;
   }
 
   @Override
-  public DistanceDBIDList<D> getRangeForObject(O obj, D range) {
-    GenericDistanceDBIDList<D> result = new GenericDistanceDBIDList<>();
-    for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
-      D currentDistance = distanceQuery.distance(obj, iter);
-      if(currentDistance.compareTo(range) <= 0) {
-        result.add(currentDistance, iter);
-      }
-    }
+  public DoubleDBIDList getRangeForObject(O obj, double range) {
+    ModifiableDoubleDBIDList result = DBIDUtil.newDistanceDBIDList();
+    linearScan(relation, relation.iterDBIDs(), obj, range, result);
     result.sort();
     return result;
   }
+
+  /**
+   * Main loop for linear scan,
+   * 
+   * @param relation Data relation
+   * @param iter Iterator
+   * @param obj Query object
+   * @param range Query radius
+   * @param result Output data structure
+   */
+  private void linearScan(Relation<? extends O> relation, DBIDIter iter, O obj, double range, ModifiableDoubleDBIDList result) {
+    while(iter.valid()) {
+      final double distance = rawdist.distance(obj, relation.get(iter));
+      if(distance <= range) {
+        result.add(distance, iter);
+      }
+      iter.advance();
+    }
+  }
 }
\ No newline at end of file
