@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.query.range;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,9 +24,8 @@ package de.lmu.ifi.dbs.elki.database.query.range;
  */
 
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDList;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
  * The interface for range queries
@@ -34,12 +33,11 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
  * @author Erich Schubert
  * 
  * @apiviz.landmark
- * @apiviz.uses DistanceDBIDList oneway - - «create»
+ * @apiviz.uses DoubleDBIDList oneway - - «create»
  * 
  * @param <O> Object type
- * @param <D> Distance type
  */
-public interface RangeQuery<O, D extends Distance<D>> extends DatabaseQuery {
+public interface RangeQuery<O> extends DatabaseQuery {
   /**
    * Get the nearest neighbors for a particular id in a given query range
    * 
@@ -47,7 +45,7 @@ public interface RangeQuery<O, D extends Distance<D>> extends DatabaseQuery {
    * @param range Query range
    * @return neighbors
    */
-  public DistanceDBIDList<D> getRangeForDBID(DBIDRef id, D range);
+  public DoubleDBIDList getRangeForDBID(DBIDRef id, double range);
 
   /**
    * Get the nearest neighbors for a particular object in a given query range
@@ -56,12 +54,5 @@ public interface RangeQuery<O, D extends Distance<D>> extends DatabaseQuery {
    * @param range Query range
    * @return neighbors
    */
-  public DistanceDBIDList<D> getRangeForObject(O obj, D range);
-
-  /**
-   * Get the distance factory for the given distance type.
-   * 
-   * @return Distance factory.
-   */
-  public D getDistanceFactory();
+  public DoubleDBIDList getRangeForObject(O obj, double range);
 }
