@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.query.rknn;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,32 +24,37 @@ package de.lmu.ifi.dbs.elki.database.query.rknn;
  */
 
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
-import de.lmu.ifi.dbs.elki.database.query.AbstractDataBasedQuery;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDList;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.database.relation.Relation;
 
 /**
  * Instance for the query on a particular database.
  * 
  * @author Erich Schubert
  */
-public abstract class AbstractRKNNQuery<O, D extends Distance<D>> extends AbstractDataBasedQuery<O> implements RKNNQuery<O, D> {
+public abstract class AbstractRKNNQuery<O> implements RKNNQuery<O> {
+  /**
+   * The data to use for this query
+   */
+  final protected Relation<? extends O> relation;
+
   /**
    * Hold the distance function to be used.
    */
-  protected final DistanceQuery<O, D> distanceQuery;
+  final protected DistanceQuery<O> distanceQuery;
 
   /**
    * Constructor.
    * 
    * @param distanceQuery distance query
    */
-  public AbstractRKNNQuery(DistanceQuery<O, D> distanceQuery) {
-    super(distanceQuery.getRelation());
+  public AbstractRKNNQuery(DistanceQuery<O> distanceQuery) {
+    super();
+    this.relation = distanceQuery.getRelation();
     this.distanceQuery = distanceQuery;
   }
 
   @Override
-  abstract public DistanceDBIDList<D> getRKNNForDBID(DBIDRef id, int k);
+  abstract public DoubleDBIDList getRKNNForDBID(DBIDRef id, int k);
 }
\ No newline at end of file
