@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.filter;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -34,11 +34,11 @@ import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
  * @param <I> Input vector type
  * @param <O> Output vector type
  */
-public abstract class AbstractVectorConversionFilter<I, O extends NumberVector<?>> extends AbstractConversionFilter<I, O> {
+public abstract class AbstractVectorConversionFilter<I, O extends NumberVector> extends AbstractConversionFilter<I, O> {
   /**
    * Number vector factory.
    */
-  protected NumberVector.Factory<O, ?> factory;
+  protected NumberVector.Factory<O>  factory;
 
   /**
    * Initialize factory from a data type.
