@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,7 +24,6 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction;
  */
 
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.Index;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 
@@ -36,8 +35,10 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
  * @apiviz.landmark
  * @apiviz.stereotype factory
  * @apiviz.has Instance oneway - - «create»
+ * 
+ * @param <O> Object type
  */
-public interface IndexBasedDistanceFunction<O, D extends Distance<D>> extends DistanceFunction<O, D> {
+public interface IndexBasedDistanceFunction<O> extends DistanceFunction<O> {
   /**
    * OptionID for the index parameter
    */
@@ -49,9 +50,8 @@ public interface IndexBasedDistanceFunction<O, D extends Distance<D>> extends Di
    * @author Erich Schubert
    * 
    * @param <T> Object type
-   * @param <D> Distance type
    */
-  public static interface Instance<T, I extends Index, D extends Distance<D>> extends DistanceQuery<T, D> {
+  public static interface Instance<T, I extends Index> extends DistanceQuery<T> {
     /**
      * Get the index used.
      * 
