@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -24,17 +24,26 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction;
  */
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
- * Base interface for the common case of distance functions defined on numerical vectors.
+ * Base interface for the common case of distance functions defined on numerical
+ * vectors.
  * 
  * @author Erich Schubert
  * 
  * @apiviz.landmark
- *
- * @param <D> Distance type
+ * 
+ * @param <O> vector type, usually NumberVector or a parent type
  */
-public interface NumberVectorDistanceFunction<D extends Distance<D>> extends PrimitiveDistanceFunction<NumberVector<?>, D> {
-  // Empty - marker interface
+public interface NumberVectorDistanceFunction<O> extends PrimitiveDistanceFunction<O> {
+  /**
+   * Computes the distance between two given vectors according to this distance
+   * function.
+   * 
+   * @param o1 first DatabaseObject
+   * @param o2 second DatabaseObject
+   * @return the distance between two given vectors according to this distance
+   *         function
+   */
+  double distance(NumberVector o1, NumberVector o2);
 }
