@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.preprocessed;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,7 +40,7 @@ import de.lmu.ifi.dbs.elki.math.linearalgebra.ProjectionResult;
  * @param <V> Vector type
  * @param <P> Projection result type
  */
-public interface LocalProjectionIndex<V extends NumberVector<?>, P extends ProjectionResult> extends Index {
+public interface LocalProjectionIndex<V extends NumberVector, P extends ProjectionResult> extends Index {
   /**
    * Get the precomputed local projection for a particular object ID.
    * 
@@ -60,7 +60,7 @@ public interface LocalProjectionIndex<V extends NumberVector<?>, P extends Proje
    * @param <V> Vector type
    * @param <I> Index type
    */
-  public static interface Factory<V extends NumberVector<?>, I extends LocalProjectionIndex<V, ?>> extends IndexFactory<V, I> {
+  public static interface Factory<V extends NumberVector, I extends LocalProjectionIndex<V, ?>> extends IndexFactory<V, I> {
     /**
      * Instantiate the index for a given database.
      * 
