@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.knn;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,14 +28,13 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
+import de.lmu.ifi.dbs.elki.database.ids.KNNList;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.PreprocessorKNNQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.minkowski.EuclideanDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.IndexFactory;
 import de.lmu.ifi.dbs.elki.index.KNNIndex;
 import de.lmu.ifi.dbs.elki.index.preprocessed.AbstractPreprocessorIndex;
@@ -52,10 +51,8 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * @author Erich Schubert
  * 
  * @param <O> Object type
- * @param <D> Distance type
- * @param <T> Result type
  */
-public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D>, T extends KNNList<D>> extends AbstractPreprocessorIndex<O, T> implements KNNIndex<O> {
+public abstract class AbstractMaterializeKNNPreprocessor<O> extends AbstractPreprocessorIndex<O, KNNList> implements KNNIndex<O> {
   /**
    * The query k value.
    */
@@ -64,12 +61,12 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
   /**
    * The distance function to be used.
    */
-  protected final DistanceFunction<? super O, D> distanceFunction;
+  protected final DistanceFunction<? super O> distanceFunction;
 
   /**
    * The distance query we used.
    */
-  protected final DistanceQuery<O, D> distanceQuery;
+  protected final DistanceQuery<O> distanceQuery;
 
   /**
    * Constructor.
@@ -78,28 +75,19 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
    * @param distanceFunction Distance function
    * @param k k
    */
-  public AbstractMaterializeKNNPreprocessor(Relation<O> relation, DistanceFunction<? super O, D> distanceFunction, int k) {
+  public AbstractMaterializeKNNPreprocessor(Relation<O> relation, DistanceFunction<? super O> distanceFunction, int k) {
     super(relation);
     this.k = k;
     this.distanceFunction = distanceFunction;
     this.distanceQuery = distanceFunction.instantiate(relation);
   }
 
-  /**
-   * Get the distance factory.
-   * 
-   * @return distance factory
-   */
-  public D getDistanceFactory() {
-    return distanceFunction.getDistanceFactory();
-  }
-
   /**
    * The distance query we used.
    * 
    * @return Distance query
    */
-  public DistanceQuery<O, D> getDistanceQuery() {
+  public DistanceQuery<O> getDistanceQuery() {
     return distanceQuery;
   }
 
@@ -123,7 +111,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
    * @param id Object ID
    * @return Neighbors
    */
-  public KNNList<D> get(DBIDRef id) {
+  public KNNList get(DBIDRef id) {
     if(storage == null) {
       if(getLogger().isDebugging()) {
         getLogger().debug("Running kNN preprocessor: " + this.getClass());
@@ -137,7 +125,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
    * Create the default storage.
    */
   void createStorage() {
-    WritableDataStore<T> s = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT, KNNList.class);
+    WritableDataStore<KNNList> s = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT, KNNList.class);
     storage = s;
   }
 
@@ -155,7 +143,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
 
   @SuppressWarnings("unchecked")
   @Override
-  public <S extends Distance<S>> KNNQuery<O, S> getKNNQuery(DistanceQuery<O, S> distQ, Object... hints) {
+  public KNNQuery<O> getKNNQuery(DistanceQuery<O> distQ, Object... hints) {
     if(!this.distanceFunction.equals(distQ.getDistanceFunction())) {
       return null;
     }
@@ -169,8 +157,8 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
       }
     }
     // To make compilers happy:
-    AbstractMaterializeKNNPreprocessor<?, ?, ?> tmp = this;
-    return new PreprocessorKNNQuery<>(relation, (AbstractMaterializeKNNPreprocessor<O, S, KNNList<S>>) tmp);
+    AbstractMaterializeKNNPreprocessor<?> tmp = this;
+    return new PreprocessorKNNQuery<>(relation, (AbstractMaterializeKNNPreprocessor<O>) tmp);
   }
 
   /**
@@ -183,9 +171,8 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
    * @apiviz.uses AbstractMaterializeKNNPreprocessor oneway - - «create»
    * 
    * @param <O> The object type
-   * @param <D> The distance type
    */
-  public abstract static class Factory<O, D extends Distance<D>, T extends KNNList<D>> implements IndexFactory<O, KNNIndex<O>> {
+  public abstract static class Factory<O> implements IndexFactory<O, KNNIndex<O>> {
     /**
      * Parameter to specify the number of nearest neighbors of an object to be
      * materialized. must be an integer greater than 1.
@@ -216,7 +203,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
     /**
      * Hold the distance function to be used.
      */
-    protected DistanceFunction<? super O, D> distanceFunction;
+    protected DistanceFunction<? super O> distanceFunction;
 
     /**
      * Index factory.
@@ -224,14 +211,14 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
      * @param k k parameter
      * @param distanceFunction distance function
      */
-    public Factory(int k, DistanceFunction<? super O, D> distanceFunction) {
+    public Factory(int k, DistanceFunction<? super O> distanceFunction) {
       super();
       this.k = k;
       this.distanceFunction = distanceFunction;
     }
 
     @Override
-    public abstract AbstractMaterializeKNNPreprocessor<O, D, T> instantiate(Relation<O> relation);
+    public abstract AbstractMaterializeKNNPreprocessor<O> instantiate(Relation<O> relation);
 
     /**
      * Get the distance function.
@@ -239,19 +226,10 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
      * @return Distance function
      */
     // TODO: hide this?
-    public DistanceFunction<? super O, D> getDistanceFunction() {
+    public DistanceFunction<? super O> getDistanceFunction() {
       return distanceFunction;
     }
 
-    /**
-     * Get the distance factory.
-     * 
-     * @return Distance factory
-     */
-    public D getDistanceFactory() {
-      return distanceFunction.getDistanceFactory();
-    }
-
     @Override
     public TypeInformation getInputTypeRestriction() {
       return distanceFunction.getInputTypeRestriction();
@@ -263,8 +241,10 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
      * @author Erich Schubert
      * 
      * @apiviz.exclude
+     * 
+     * @param <O> Object type
      */
-    public abstract static class Parameterizer<O, D extends Distance<D>> extends AbstractParameterizer {
+    public abstract static class Parameterizer<O> extends AbstractParameterizer {
       /**
        * Holds the value of {@link #K_ID}.
        */
@@ -273,7 +253,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
       /**
        * Hold the distance function to be used.
        */
-      protected DistanceFunction<? super O, D> distanceFunction;
+      protected DistanceFunction<? super O> distanceFunction;
 
       @Override
       protected void makeOptions(Parameterization config) {
@@ -286,14 +266,14 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
         }
 
         // distance function
-        final ObjectParameter<DistanceFunction<? super O, D>> distanceFunctionP = new ObjectParameter<>(DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
+        final ObjectParameter<DistanceFunction<? super O>> distanceFunctionP = new ObjectParameter<>(DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
         if(config.grab(distanceFunctionP)) {
           distanceFunction = distanceFunctionP.instantiateClass(config);
         }
       }
 
       @Override
-      protected abstract Factory<O, D, ?> makeInstance();
+      protected abstract Factory<O> makeInstance();
     }
   }
 }
\ No newline at end of file
