@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.preference;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,17 +25,17 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.preference;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.BitSet;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import de.lmu.ifi.dbs.elki.algorithm.APRIORI;
-import de.lmu.ifi.dbs.elki.data.Bit;
+import de.lmu.ifi.dbs.elki.algorithm.itemsetmining.APRIORI;
+import de.lmu.ifi.dbs.elki.algorithm.itemsetmining.Itemset;
 import de.lmu.ifi.dbs.elki.data.BitVector;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
+import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.HashmapDatabase;
 import de.lmu.ifi.dbs.elki.database.UpdatableDatabase;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
@@ -43,23 +43,21 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDList;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
 import de.lmu.ifi.dbs.elki.database.query.distance.PrimitiveDistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.datasource.bundle.SingleObjectBundle;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.DimensionSelectingDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.OnedimensionalDistanceFunction;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.result.AprioriResult;
+import de.lmu.ifi.dbs.elki.utilities.BitsUtil;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.ExceptionMessages;
-import de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.WrongParameterValueException;
@@ -78,7 +76,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  * @param <V> Vector type
  */
 @Description("Computes the preference vector of objects of a certain database according to the DiSH algorithm.")
-public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends AbstractPreferenceVectorIndex<V> implements PreferenceVectorIndex<V> {
+public class DiSHPreferenceVectorIndex<V extends NumberVector> extends AbstractPreferenceVectorIndex<V> implements PreferenceVectorIndex<V> {
   /**
    * Logger to use.
    */
@@ -103,7 +101,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
   /**
    * The epsilon value for each dimension.
    */
-  protected DoubleDistance[] epsilon;
+  protected double[] epsilon;
 
   /**
    * Threshold for minimum number of points in the neighborhood.
@@ -123,7 +121,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
    * @param minpts MinPts value
    * @param strategy Strategy
    */
-  public DiSHPreferenceVectorIndex(Relation<V> relation, DoubleDistance[] epsilon, int minpts, Strategy strategy) {
+  public DiSHPreferenceVectorIndex(Relation<V> relation, double[] epsilon, int minpts, Strategy strategy) {
     super(relation);
     this.epsilon = epsilon;
     this.minpts = minpts;
@@ -136,7 +134,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
       throw new IllegalArgumentException(ExceptionMessages.DATABASE_EMPTY);
     }
 
-    storage = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP, BitSet.class);
+    storage = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP, long[].class);
 
     if(LOG.isDebugging()) {
       StringBuilder msg = new StringBuilder();
@@ -152,13 +150,13 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
     // only one epsilon value specified
     int dim = RelationUtil.dimensionality(relation);
     if(epsilon.length == 1 && dim != 1) {
-      DoubleDistance eps = epsilon[0];
-      epsilon = new DoubleDistance[dim];
+      double eps = epsilon[0];
+      epsilon = new double[dim];
       Arrays.fill(epsilon, eps);
     }
 
     // epsilons as string
-    RangeQuery<V, DoubleDistance>[] rangeQueries = initRangeQueries(relation, dim);
+    RangeQuery<V>[] rangeQueries = initRangeQueries(relation, dim);
 
     for(DBIDIter it = relation.iterDBIDs(); it.valid(); it.advance()) {
       StringBuilder msg = new StringBuilder();
@@ -172,7 +170,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
       // determine neighbors in each dimension
       ModifiableDBIDs[] allNeighbors = new ModifiableDBIDs[dim];
       for(int d = 0; d < dim; d++) {
-        DistanceDBIDList<DoubleDistance> qrList = rangeQueries[d].getRangeForDBID(it, epsilon[d]);
+        DoubleDBIDList qrList = rangeQueries[d].getRangeForDBID(it, epsilon[d]);
         allNeighbors[d] = DBIDUtil.newHashSet(qrList);
       }
 
@@ -184,24 +182,15 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
         }
       }
 
-      try {
-        storage.put(it, determinePreferenceVector(relation, allNeighbors, msg));
-      }
-      catch(UnableToComplyException e) {
-        throw new IllegalStateException(e);
-      }
+      storage.put(it, determinePreferenceVector(relation, allNeighbors, msg));
 
       if(LOG.isDebugging()) {
         LOG.debugFine(msg.toString());
       }
 
-      if(progress != null) {
-        progress.incrementProcessed(LOG);
-      }
-    }
-    if(progress != null) {
-      progress.ensureCompleted(LOG);
+      LOG.incrementProcessed(progress);
     }
+    LOG.ensureCompleted(progress);
 
     long end = System.currentTimeMillis();
     // TODO: re-add timing code!
@@ -218,10 +207,8 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
    * @param neighborIDs the list of ids of the neighbors in each dimension
    * @param msg a string buffer for debug messages
    * @return the preference vector
-   * 
-   * @throws de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException
    */
-  private BitSet determinePreferenceVector(Relation<V> relation, ModifiableDBIDs[] neighborIDs, StringBuilder msg) throws UnableToComplyException {
+  private long[] determinePreferenceVector(Relation<V> relation, ModifiableDBIDs[] neighborIDs, StringBuilder msg) {
     if(strategy.equals(Strategy.APRIORI)) {
       return determinePreferenceVectorByApriori(relation, neighborIDs, msg);
     }
@@ -240,31 +227,25 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
    * @param neighborIDs the list of ids of the neighbors in each dimension
    * @param msg a string buffer for debug messages
    * @return the preference vector
-   * 
-   * @throws de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException
-   * 
    */
-  private BitSet determinePreferenceVectorByApriori(Relation<V> relation, ModifiableDBIDs[] neighborIDs, StringBuilder msg) throws UnableToComplyException {
+  private long[] determinePreferenceVectorByApriori(Relation<V> relation, ModifiableDBIDs[] neighborIDs, StringBuilder msg) {
     int dimensionality = neighborIDs.length;
 
     // database for apriori
     UpdatableDatabase apriori_db = new HashmapDatabase();
-    SimpleTypeInformation<?> bitmeta = new VectorFieldTypeInformation<>(BitVector.class, dimensionality);
+    SimpleTypeInformation<?> bitmeta = VectorFieldTypeInformation.typeRequest(BitVector.class, dimensionality, dimensionality);
     for(DBIDIter it = relation.iterDBIDs(); it.valid(); it.advance()) {
-      Bit[] bits = new Bit[dimensionality];
+      long[] bits = BitsUtil.zero(dimensionality);
       boolean allFalse = true;
       for(int d = 0; d < dimensionality; d++) {
         if(neighborIDs[d].contains(it)) {
-          bits[d] = Bit.TRUE;
+          BitsUtil.setI(bits, d);
           allFalse = false;
         }
-        else {
-          bits[d] = Bit.FALSE;
-        }
       }
       if(!allFalse) {
         SingleObjectBundle oaa = new SingleObjectBundle();
-        oaa.append(bitmeta, new BitVector(bits));
+        oaa.append(bitmeta, new BitVector(bits, dimensionality));
         apriori_db.insert(oaa);
       }
     }
@@ -272,27 +253,24 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
     AprioriResult aprioriResult = apriori.run(apriori_db);
 
     // result of apriori
-    List<BitSet> frequentItemsets = aprioriResult.getSolution();
-    Map<BitSet, Integer> supports = aprioriResult.getSupports();
+    List<Itemset> frequentItemsets = aprioriResult.getItemsets();
     if(LOG.isDebugging()) {
       msg.append("\n Frequent itemsets: ").append(frequentItemsets);
-      msg.append("\n All supports: ").append(supports);
     }
     int maxSupport = 0;
     int maxCardinality = 0;
-    BitSet preferenceVector = new BitSet();
-    for(BitSet bitSet : frequentItemsets) {
-      int cardinality = bitSet.cardinality();
-      if((maxCardinality < cardinality) || (maxCardinality == cardinality && maxSupport == supports.get(bitSet))) {
-        preferenceVector = bitSet;
-        maxCardinality = cardinality;
-        maxSupport = supports.get(bitSet);
+    long[] preferenceVector = BitsUtil.zero(dimensionality);
+    for(Itemset itemset : frequentItemsets) {
+      if((maxCardinality < itemset.length()) || (maxCardinality == itemset.length() && maxSupport == itemset.getSupport())) {
+        preferenceVector = itemset.getItems();
+        maxCardinality = itemset.length();
+        maxSupport = itemset.getSupport();
       }
     }
 
     if(LOG.isDebugging()) {
       msg.append("\n preference ");
-      msg.append(FormatUtil.format(dimensionality, preferenceVector));
+      msg.append(BitsUtil.toStringLow(preferenceVector, dimensionality));
       msg.append('\n');
       LOG.debugFine(msg.toString());
     }
@@ -307,9 +285,9 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
    * @param msg a string buffer for debug messages
    * @return the preference vector
    */
-  private BitSet determinePreferenceVectorByMaxIntersection(ModifiableDBIDs[] neighborIDs, StringBuilder msg) {
+  private long[] determinePreferenceVectorByMaxIntersection(ModifiableDBIDs[] neighborIDs, StringBuilder msg) {
     int dimensionality = neighborIDs.length;
-    BitSet preferenceVector = new BitSet(dimensionality);
+    long[] preferenceVector = BitsUtil.zero(dimensionality);
 
     Map<Integer, ModifiableDBIDs> candidates = new HashMap<>(dimensionality);
     for(int i = 0; i < dimensionality; i++) {
@@ -325,7 +303,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
     if(!candidates.isEmpty()) {
       int i = max(candidates);
       ModifiableDBIDs intersection = candidates.remove(i);
-      preferenceVector.set(i);
+      BitsUtil.setI(preferenceVector, i);
       while(!candidates.isEmpty()) {
         ModifiableDBIDs newIntersection = DBIDUtil.newHashSet();
         i = maxIntersection(candidates, intersection, newIntersection);
@@ -338,14 +316,14 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
           break;
         }
         else {
-          preferenceVector.set(i);
+          BitsUtil.setI(preferenceVector, i);
         }
       }
     }
 
     if(LOG.isDebugging()) {
       msg.append("\n preference ");
-      msg.append(FormatUtil.format(dimensionality, preferenceVector));
+      msg.append(BitsUtil.toStringLow(preferenceVector, dimensionality));
       msg.append('\n');
       LOG.debug(msg.toString());
     }
@@ -405,11 +383,12 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
    * @return the dimension selecting distancefunctions to determine the
    *         preference vectors
    */
-  private RangeQuery<V, DoubleDistance>[] initRangeQueries(Relation<V> relation, int dimensionality) {
-    Class<RangeQuery<V, DoubleDistance>> rqcls = ClassGenericsUtil.uglyCastIntoSubclass(RangeQuery.class);
-    RangeQuery<V, DoubleDistance>[] rangeQueries = ClassGenericsUtil.newArrayOfNull(dimensionality, rqcls);
+  private RangeQuery<V>[] initRangeQueries(Relation<V> relation, int dimensionality) {
+    Database db = relation.getDatabase();
+    Class<RangeQuery<V>> rqcls = ClassGenericsUtil.uglyCastIntoSubclass(RangeQuery.class);
+    RangeQuery<V>[] rangeQueries = ClassGenericsUtil.newArrayOfNull(dimensionality, rqcls);
     for(int d = 0; d < dimensionality; d++) {
-      rangeQueries[d] = relation.getDatabase().getRangeQuery(new PrimitiveDistanceQuery<>(relation, new DimensionSelectingDistanceFunction(d)));
+      rangeQueries[d] = db.getRangeQuery(new PrimitiveDistanceQuery<>(relation, new OnedimensionalDistanceFunction(d)));
     }
     return rangeQueries;
   }
@@ -444,11 +423,11 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
    * 
    * @param <V> Vector type
    */
-  public static class Factory<V extends NumberVector<?>> extends AbstractPreferenceVectorIndex.Factory<V, DiSHPreferenceVectorIndex<V>> {
+  public static class Factory<V extends NumberVector> extends AbstractPreferenceVectorIndex.Factory<V, DiSHPreferenceVectorIndex<V>> {
     /**
      * The default value for epsilon.
      */
-    public static final DoubleDistance DEFAULT_EPSILON = new DoubleDistance(0.001);
+    public static final double DEFAULT_EPSILON = 0.001;
 
     /**
      * A comma separated list of positive doubles specifying the maximum radius
@@ -509,7 +488,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
     /**
      * The epsilon value for each dimension.
      */
-    protected DoubleDistance[] epsilon;
+    protected double[] epsilon;
 
     /**
      * Threshold for minimum number of points in the neighborhood.
@@ -528,7 +507,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
      * @param minpts Minpts
      * @param strategy Strategy
      */
-    public Factory(DoubleDistance[] epsilon, int minpts, Strategy strategy) {
+    public Factory(double[] epsilon, int minpts, Strategy strategy) {
       super();
       this.epsilon = epsilon;
       this.minpts = minpts;
@@ -556,11 +535,11 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
      * 
      * @apiviz.exclude
      */
-    public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
+    public static class Parameterizer<V extends NumberVector> extends AbstractParameterizer {
       /**
        * The epsilon value for each dimension.
        */
-      protected DoubleDistance[] epsilon;
+      protected double[] epsilon;
 
       /**
        * Threshold for minimum number of points in the neighborhood.
@@ -584,16 +563,16 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?>> extends Abstra
         // parameter epsilon
         // todo: constraint auf positive werte
         List<Double> defaultEps = new ArrayList<>();
-        defaultEps.add(DEFAULT_EPSILON.doubleValue());
+        defaultEps.add(DEFAULT_EPSILON);
         final DoubleListParameter epsilonP = new DoubleListParameter(EPSILON_ID, true);
         epsilonP.setDefaultValue(defaultEps);
         if(config.grab(epsilonP)) {
           List<Double> eps_list = epsilonP.getValue();
-          epsilon = new DoubleDistance[eps_list.size()];
+          epsilon = new double[eps_list.size()];
 
           for(int d = 0; d < eps_list.size(); d++) {
-            epsilon[d] = new DoubleDistance(eps_list.get(d));
-            if(epsilon[d].doubleValue() < 0) {
+            epsilon[d] = eps_list.get(d);
+            if(epsilon[d] < 0) {
               config.reportError(new WrongParameterValueException(epsilonP, eps_list.toString()));
             }
           }
