@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.preference;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,8 +23,6 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.preference;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.BitSet;
-
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
@@ -38,14 +36,14 @@ import de.lmu.ifi.dbs.elki.index.IndexFactory;
  * 
  * @param <NV> Vector type
  */
-public interface PreferenceVectorIndex<NV extends NumberVector<?>> extends Index {
+public interface PreferenceVectorIndex<NV extends NumberVector> extends Index {
   /**
    * Get the precomputed preference vector for a particular object ID.
    * 
    * @param id Object ID
    * @return Matrix
    */
-  public BitSet getPreferenceVector(DBIDRef id);
+  public long[] getPreferenceVector(DBIDRef id);
 
   /**
    * Factory interface
@@ -58,7 +56,7 @@ public interface PreferenceVectorIndex<NV extends NumberVector<?>> extends Index
    * @param <V> vector type
    * @param <I> index type
    */
-  public static interface Factory<V extends NumberVector<?>, I extends PreferenceVectorIndex<V>> extends IndexFactory<V, I> {
+  public static interface Factory<V extends NumberVector, I extends PreferenceVectorIndex<V>> extends IndexFactory<V, I> {
     /**
      * Instantiate the index for a given database.
      * 
