@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,7 +30,6 @@ import java.util.List;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.index.tree.BreadthFirstEnumeration;
 import de.lmu.ifi.dbs.elki.index.tree.IndexTreePath;
 import de.lmu.ifi.dbs.elki.index.tree.TreeIndexPathComponent;
@@ -54,12 +53,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleIntPair;
  * @apiviz.excludeSubtypes
  * 
  * @param <O> the type of DatabaseObject to be stored in the metrical index
- * @param <D> the type of Distance used in the metrical index
  * @param <N> the type of MetricalNode used in the metrical index
  * @param <E> the type of MetricalEntry used in the metrical index
  * @param <S> the type to store settings in.
  */
-public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends AbstractMTreeNode<O, D, N, E>, E extends MTreeEntry, S extends MTreeSettings<O, D, N, E>> extends MetricalIndexTree<O, D, N, E> {
+public abstract class AbstractMTree<O, N extends AbstractMTreeNode<O, N, E>, E extends MTreeEntry, S extends MTreeSettings<O, N, E>> extends MetricalIndexTree<O, N, E> {
   /**
    * Debugging flag: do extra integrity checks.
    */
@@ -87,19 +85,10 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
   }
 
   @Override
-  public final DistanceFunction<? super O, D> getDistanceFunction() {
+  public final DistanceFunction<? super O> getDistanceFunction() {
     return settings.distanceFunction;
   }
 
-  /**
-   * Get the distance factory.
-   * 
-   * @return the distance factory used
-   */
-  public final D getDistanceFactory() {
-    return settings.distanceFunction.getDistanceFactory();
-  }
-
   /**
    * Returns a string representation of this M-Tree by performing a breadth
    * first enumeration on the tree and adding the string representation of the
@@ -117,8 +106,8 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
 
     N node = getRoot();
 
-    while (!node.isLeaf()) {
-      if (node.getNumEntries() > 0) {
+    while(!node.isLeaf()) {
+      if(node.getNumEntries() > 0) {
         E entry = node.getEntry(0);
         node = getNode(entry);
         levels++;
@@ -126,20 +115,22 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
     }
 
     BreadthFirstEnumeration<N, E> enumeration = new BreadthFirstEnumeration<>(this, getRootPath());
-    while (enumeration.hasMoreElements()) {
+    while(enumeration.hasMoreElements()) {
       IndexTreePath<E> path = enumeration.nextElement();
       E entry = path.getLastPathComponent().getEntry();
-      if (entry.isLeafEntry()) {
+      if(entry.isLeafEntry()) {
         objects++;
         result.append("\n    ").append(entry.toString());
-      } else {
+      }
+      else {
         node = getNode(entry);
         result.append("\n\n").append(node).append(", numEntries = ").append(node.getNumEntries());
         result.append("\n").append(entry.toString());
 
-        if (node.isLeaf()) {
+        if(node.isLeaf()) {
           leafNodes++;
-        } else {
+        }
+        else {
           dirNodes++;
         }
       }
@@ -165,27 +156,27 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
    */
   // todo: implement a bulk load for M-Tree and remove this method
   public void insert(E entry, boolean withPreInsert) {
-    if (getLogger().isDebugging()) {
+    if(getLogger().isDebugging()) {
       getLogger().debugFine("insert " + entry.getRoutingObjectID() + "\n");
     }
 
-    if (!initialized) {
+    if(!initialized) {
       initialize(entry);
     }
 
     // choose subtree for insertion
     IndexTreePath<E> subtree = settings.insertStrategy.choosePath(this, entry);
-    if (getLogger().isDebugging()) {
+    if(getLogger().isDebugging()) {
       getLogger().debugFine("insertion-subtree " + subtree + "\n");
     }
 
     // determine parent distance
     E parentEntry = subtree.getLastPathComponent().getEntry();
-    double parentDistance = distance(parentEntry.getRoutingObjectID(), entry.getRoutingObjectID()).doubleValue();
+    double parentDistance = distance(parentEntry.getRoutingObjectID(), entry.getRoutingObjectID());
     entry.setParentDistance(parentDistance);
 
     // create leaf entry and do pre insert
-    if (withPreInsert) {
+    if(withPreInsert) {
       preInsert(entry);
     }
 
@@ -198,8 +189,8 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
     adjustTree(subtree);
 
     // test
-    if (EXTRA_INTEGRITY_CHECKS) {
-      if (withPreInsert) {
+    if(EXTRA_INTEGRITY_CHECKS) {
+      if(withPreInsert) {
         getRoot().integrityCheck(this, getRootEntry());
       }
     }
@@ -211,10 +202,10 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
    * @param entries Entries to insert
    */
   public void insertAll(List<E> entries) {
-    if (!initialized && entries.size() > 0) {
+    if(!initialized && entries.size() > 0) {
       initialize(entries.get(0));
     }
-    for (E entry : entries) {
+    for(E entry : entries) {
       insert(entry, false);
     }
   }
@@ -236,9 +227,9 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
   protected final List<DoubleIntPair> getSortedEntries(N node, DBID q) {
     List<DoubleIntPair> result = new ArrayList<>();
 
-    for (int i = 0; i < node.getNumEntries(); i++) {
+    for(int i = 0; i < node.getNumEntries(); i++) {
       E entry = node.getEntry(i);
-      double distance = distance(entry.getRoutingObjectID(), q).doubleValue();
+      double distance = distance(entry.getRoutingObjectID(), q);
       double radius = entry.getCoveringRadius();
       double minDist = (radius > distance) ? 0.0 : distance - radius;
 
@@ -256,7 +247,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
    * @param id2 the second id
    * @return the distance between the two specified ids
    */
-  public abstract D distance(DBIDRef id1, DBIDRef id2);
+  public abstract double distance(DBIDRef id1, DBIDRef id2);
 
   /**
    * Returns the distance between the routing object of two entries.
@@ -265,7 +256,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
    * @param e2 Second entry
    * @return the distance between the two routing objects
    */
-  public final D distance(E e1, E e2) {
+  public final double distance(E e1, E e2) {
     return distance(e1.getRoutingObjectID(), e2.getRoutingObjectID());
   }
 
@@ -286,7 +277,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
    * @param subtree the subtree to be adjusted
    */
   private void adjustTree(IndexTreePath<E> subtree) {
-    if (getLogger().isDebugging()) {
+    if(getLogger().isDebugging()) {
       getLogger().debugFine("Adjust tree " + subtree + "\n");
     }
 
@@ -295,7 +286,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
     N node = getNode(subtree.getLastPathComponent().getEntry());
 
     // overflow in node; split the node
-    if (hasOverflow(node)) {
+    if(hasOverflow(node)) {
       // do the split
       Assignments<E> assignments = settings.splitStrategy.split(this, node);
       final N newNode = node.isLeaf() ? createNewLeafNode() : createNewDirectoryNode();
@@ -303,12 +294,12 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
       List<E> entries1 = new ArrayList<>(assignments.getFirstAssignments().size());
       List<E> entries2 = new ArrayList<>(assignments.getSecondAssignments().size());
       // Store final parent distances:
-      for (DistanceEntry<E> ent : assignments.getFirstAssignments()) {
+      for(DistanceEntry<E> ent : assignments.getFirstAssignments()) {
         final E e = ent.getEntry();
         e.setParentDistance(ent.getDistance());
         entries1.add(e);
       }
-      for (DistanceEntry<E> ent : assignments.getSecondAssignments()) {
+      for(DistanceEntry<E> ent : assignments.getSecondAssignments()) {
         final E e = ent.getEntry();
         e.setParentDistance(ent.getDistance());
         entries2.add(e);
@@ -319,14 +310,14 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
       writeNode(node);
       writeNode(newNode);
 
-      if (getLogger().isDebugging()) {
+      if(getLogger().isDebugging()) {
         String msg = "Split Node " + node.getPageID() + " (" + this.getClass() + ")\n" + "      newNode " + newNode.getPageID() + "\n" + "      firstPromoted " + assignments.getFirstRoutingObject() + "\n" + "      firstAssignments(" + node.getPageID() + ") " + assignments.getFirstAssignments() + "\n" + "      firstCR " + assignments.getFirstCoveringRadius() + "\n" + "      secondPromoted " + assignments.getSecondRoutingObject() + "\n" + "      secondAssignments(" + newNode.getPageID() + ") " + assignments.getSecondAssignments() + "\n" + "      secondCR " + assignments.getSecondCoveringRadius() + "\n";
         getLogger().debugFine(msg);
       }
 
       // if root was split: create a new root that points the two split
       // nodes
-      if (isRoot(node)) {
+      if(isRoot(node)) {
         // FIXME: stimmen die parentDistance der Kinder in node & splitNode?
         IndexTreePath<E> newRootPath = createNewRoot(node, newNode, assignments.getFirstRoutingObject(), assignments.getSecondRoutingObject());
         adjustTree(newRootPath);
@@ -336,16 +327,16 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
         // get the parent and add the new split node
         E parentEntry = subtree.getParentPath().getLastPathComponent().getEntry();
         N parent = getNode(parentEntry);
-        if (getLogger().isDebugging()) {
+        if(getLogger().isDebugging()) {
           getLogger().debugFine("parent " + parent);
         }
-        double parentDistance2 = distance(parentEntry.getRoutingObjectID(), assignments.getSecondRoutingObject()).doubleValue();
+        double parentDistance2 = distance(parentEntry.getRoutingObjectID(), assignments.getSecondRoutingObject());
         // logger.warning("parent: "+parent.toString()+" split: " +
         // splitNode.toString()+ " dist:"+parentDistance2);
         parent.addDirectoryEntry(createNewDirectoryEntry(newNode, assignments.getSecondRoutingObject(), parentDistance2));
 
         // adjust the entry representing the (old) node, that has been split
-        double parentDistance1 = distance(parentEntry.getRoutingObjectID(), assignments.getFirstRoutingObject()).doubleValue();
+        double parentDistance1 = distance(parentEntry.getRoutingObjectID(), assignments.getFirstRoutingObject());
         // logger.warning("parent: "+parent.toString()+" node: " +
         // node.toString()+ " dist:"+parentDistance1);
         node.adjustEntry(parent.getEntry(nodeIndex), assignments.getFirstRoutingObject(), parentDistance1, this);
@@ -358,7 +349,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
     // no overflow, only adjust parameters of the entry representing the
     // node
     else {
-      if (!isRoot(node)) {
+      if(!isRoot(node)) {
         E parentEntry = subtree.getParentPath().getLastPathComponent().getEntry();
         N parent = getNode(parentEntry);
         int index = subtree.getLastPathComponent().getIndex();
@@ -385,7 +376,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
    *         otherwise
    */
   private boolean hasOverflow(N node) {
-    if (node.isLeaf()) {
+    if(node.isLeaf()) {
       return node.getNumEntries() == leafCapacity;
     }
 
@@ -411,9 +402,9 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
 
     // switch the ids
     oldRoot.setPageID(root.getPageID());
-    if (!oldRoot.isLeaf()) {
+    if(!oldRoot.isLeaf()) {
       // FIXME: what is happening here?
-      for (int i = 0; i < oldRoot.getNumEntries(); i++) {
+      for(int i = 0; i < oldRoot.getNumEntries(); i++) {
         N node = getNode(oldRoot.getEntry(i));
         writeNode(node);
       }
@@ -437,7 +428,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
     writeNode(root);
     writeNode(oldRoot);
     writeNode(newNode);
-    if (getLogger().isDebugging()) {
+    if(getLogger().isDebugging()) {
       String msg = "Create new Root: ID=" + root.getPageID();
       msg += "\nchild1 " + oldRoot;
       msg += "\nchild2 " + newNode;
@@ -451,13 +442,13 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
   public List<E> getLeaves() {
     List<E> result = new ArrayList<>();
     BreadthFirstEnumeration<N, E> enumeration = new BreadthFirstEnumeration<>(this, getRootPath());
-    while (enumeration.hasMoreElements()) {
+    while(enumeration.hasMoreElements()) {
       IndexTreePath<E> path = enumeration.nextElement();
       E entry = path.getLastPathComponent().getEntry();
-      if (!entry.isLeafEntry()) {
+      if(!entry.isLeafEntry()) {
         // TODO: any way to skip unnecessary reads?
         N node = getNode(entry);
-        if (node.isLeaf()) {
+        if(node.isLeaf()) {
           result.add(entry);
         }
       }
@@ -474,8 +465,8 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
     int levels = 0;
     N node = getRoot();
 
-    while (!node.isLeaf()) {
-      if (node.getNumEntries() > 0) {
+    while(!node.isLeaf()) {
+      if(node.getNumEntries() > 0) {
         E entry = node.getEntry(0);
         node = getNode(entry);
         levels++;
@@ -488,7 +479,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
   public void logStatistics() {
     super.logStatistics();
     Logging log = getLogger();
-    if (log.isStatistics()) {
+    if(log.isStatistics()) {
       log.statistics(new LongStatistic(this.getClass().getName() + ".height", getHeight()));
       statistics.logStatistics();
     }
@@ -532,7 +523,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
      * Count a distance computation.
      */
     public void countDistanceCalculation() {
-      if (distanceCalcs != null) {
+      if(distanceCalcs != null) {
         distanceCalcs.increment();
       }
     }
@@ -541,7 +532,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
      * Count a knn query invocation.
      */
     public void countKNNQuery() {
-      if (knnQueries != null) {
+      if(knnQueries != null) {
         knnQueries.increment();
       }
     }
@@ -550,7 +541,7 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
      * Count a range query invocation.
      */
     public void countRangeQuery() {
-      if (rangeQueries != null) {
+      if(rangeQueries != null) {
         rangeQueries.increment();
       }
     }
@@ -560,13 +551,13 @@ public abstract class AbstractMTree<O, D extends NumberDistance<D, ?>, N extends
      */
     public void logStatistics() {
       Logging log = getLogger();
-      if (statistics.distanceCalcs != null) {
+      if(statistics.distanceCalcs != null) {
         log.statistics(statistics.distanceCalcs);
       }
-      if (statistics.knnQueries != null) {
+      if(statistics.knnQueries != null) {
         log.statistics(statistics.knnQueries);
       }
-      if (statistics.rangeQueries != null) {
+      if(statistics.rangeQueries != null) {
         log.statistics(statistics.rangeQueries);
       }
     }
