@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.mkmax;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -30,20 +30,18 @@ import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDListIter;
+import de.lmu.ifi.dbs.elki.database.ids.KNNHeap;
+import de.lmu.ifi.dbs.elki.database.ids.KNNList;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDList;
-import de.lmu.ifi.dbs.elki.database.ids.distance.DistanceDBIDListIter;
-import de.lmu.ifi.dbs.elki.database.ids.distance.KNNHeap;
-import de.lmu.ifi.dbs.elki.database.ids.distance.KNNList;
-import de.lmu.ifi.dbs.elki.database.ids.distance.ModifiableDistanceDBIDList;
-import de.lmu.ifi.dbs.elki.database.ids.generic.GenericDistanceDBIDList;
+import de.lmu.ifi.dbs.elki.database.ids.ModifiableDoubleDBIDList;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.AbstractMkTreeUnified;
 import de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants.mktrees.MkTreeSettings;
 import de.lmu.ifi.dbs.elki.logging.Logging;
-import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.persistent.PageFile;
+import de.lmu.ifi.dbs.elki.utilities.io.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleIntPair;
 
 /**
@@ -57,9 +55,8 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleIntPair;
  * @apiviz.has MkMaxTreeNode oneway - - contains
  * 
  * @param <O> the type of DatabaseObject to be stored in the MkMaxTree
- * @param <D> the type of Distance used in the MkMaxTree
  */
-public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTreeUnified<O, D, MkMaxTreeNode<O, D>, MkMaxEntry, MkTreeSettings<O, D, MkMaxTreeNode<O, D>, MkMaxEntry>> {
+public class MkMaxTree<O> extends AbstractMkTreeUnified<O, MkMaxTreeNode<O>, MkMaxEntry, MkTreeSettings<O, MkMaxTreeNode<O>, MkMaxEntry>> {
   /**
    * The logger for this class.
    */
@@ -72,7 +69,7 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @param pagefile Page file
    * @param settings Tree settings
    */
-  public MkMaxTree(Relation<O> relation, PageFile<MkMaxTreeNode<O, D>> pagefile, MkTreeSettings<O, D, MkMaxTreeNode<O, D>, MkMaxEntry> settings) {
+  public MkMaxTree(Relation<O> relation, PageFile<MkMaxTreeNode<O>> pagefile, MkTreeSettings<O, MkMaxTreeNode<O>, MkMaxEntry> settings) {
     super(relation, pagefile, settings);
   }
 
@@ -83,13 +80,13 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * in a second step.
    */
   @Override
-  public DistanceDBIDList<D> reverseKNNQuery(DBIDRef id, int k) {
+  public DoubleDBIDList reverseKNNQuery(DBIDRef id, int k) {
     if (k > this.getKmax()) {
       throw new IllegalArgumentException("Parameter k has to be equal or less than " + "parameter k of the MkMax-Tree!");
     }
 
     // get the candidates
-    GenericDistanceDBIDList<D> candidates = new GenericDistanceDBIDList<>();
+    ModifiableDoubleDBIDList candidates = DBIDUtil.newDistanceDBIDList();
     doReverseKNNQuery(id, getRoot(), null, candidates);
 
     if (k == this.getKmax()) {
@@ -105,15 +102,15 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
     for (DBIDIter candidate = candidates.iter(); candidate.valid(); candidate.advance()) {
       candidateIDs.add(candidate);
     }
-    Map<DBID, KNNList<D>> knnLists = batchNN(getRoot(), candidateIDs, k);
+    Map<DBID, KNNList> knnLists = batchNN(getRoot(), candidateIDs, k);
 
-    GenericDistanceDBIDList<D> result = new GenericDistanceDBIDList<>();
+    ModifiableDoubleDBIDList result = DBIDUtil.newDistanceDBIDList();
     for (DBIDIter iter = candidateIDs.iter(); iter.valid(); iter.advance()) {
       DBID cid = DBIDUtil.deref(iter);
-      KNNList<D> cands = knnLists.get(cid);
-      for (DistanceDBIDListIter<D> iter2 = cands.iter(); iter2.valid(); iter2.advance()) {
+      KNNList cands = knnLists.get(cid);
+      for (DoubleDBIDListIter iter2 = cands.iter(); iter2.valid(); iter2.advance()) {
         if (DBIDUtil.equal(id, iter2)) {
-          result.add(iter2.getDistance(), cid);
+          result.add(iter2.doubleValue(), cid);
           break;
         }
       }
@@ -132,7 +129,7 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    */
   @Override
   protected void preInsert(MkMaxEntry entry) {
-    KNNHeap<D> knns_o = DBIDUtil.newHeap(getDistanceFactory(), getKmax());
+    KNNHeap knns_o = DBIDUtil.newHeap(getKmax());
     preInsert(entry, getRootEntry(), knns_o);
   }
 
@@ -140,13 +137,13 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * Adjusts the knn distance in the subtree of the specified root entry.
    */
   @Override
-  protected void kNNdistanceAdjustment(MkMaxEntry entry, Map<DBID, KNNList<D>> knnLists) {
-    MkMaxTreeNode<O, D> node = getNode(entry);
+  protected void kNNdistanceAdjustment(MkMaxEntry entry, Map<DBID, KNNList> knnLists) {
+    MkMaxTreeNode<O> node = getNode(entry);
     double knnDist_node = 0.;
     if (node.isLeaf()) {
       for (int i = 0; i < node.getNumEntries(); i++) {
         MkMaxEntry leafEntry = node.getEntry(i);
-        leafEntry.setKnnDistance(knnLists.get(leafEntry.getRoutingObjectID()).getKNNDistance().doubleValue());
+        leafEntry.setKnnDistance(knnLists.get(leafEntry.getRoutingObjectID()).getKNNDistance());
         knnDist_node = Math.max(knnDist_node, leafEntry.getKnnDistance());
       }
     } else {
@@ -170,13 +167,13 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @param node_entry the entry representing the node
    * @param result the list for the query result
    */
-  private void doReverseKNNQuery(DBIDRef q, MkMaxTreeNode<O, D> node, MkMaxEntry node_entry, ModifiableDistanceDBIDList<D> result) {
+  private void doReverseKNNQuery(DBIDRef q, MkMaxTreeNode<O> node, MkMaxEntry node_entry, ModifiableDoubleDBIDList result) {
     // data node
     if (node.isLeaf()) {
       for (int i = 0; i < node.getNumEntries(); i++) {
         MkMaxEntry entry = node.getEntry(i);
-        D distance = distance(entry.getRoutingObjectID(), q);
-        if (distance.doubleValue() <= entry.getKnnDistance()) {
+        double distance = distance(entry.getRoutingObjectID(), q);
+        if (distance <= entry.getKnnDistance()) {
           result.add(distance, entry.getRoutingObjectID());
         }
       }
@@ -188,11 +185,11 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
         MkMaxEntry entry = node.getEntry(i);
         double node_knnDist = node_entry != null ? node_entry.getKnnDistance() : Double.POSITIVE_INFINITY;
 
-        double distance = distance(entry.getRoutingObjectID(), q).doubleValue();
+        double distance = distance(entry.getRoutingObjectID(), q);
         double minDist = (entry.getCoveringRadius() > distance) ? 0.0 : distance - entry.getCoveringRadius();
 
         if (minDist <= node_knnDist) {
-          MkMaxTreeNode<O, D> childNode = getNode(entry);
+          MkMaxTreeNode<O> childNode = getNode(entry);
           doReverseKNNQuery(q, childNode, entry, result);
         }
       }
@@ -206,39 +203,39 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @param nodeEntry the entry representing the root of the current subtree
    * @param knns_q the knns of q
    */
-  private void preInsert(MkMaxEntry q, MkMaxEntry nodeEntry, KNNHeap<D> knns_q) {
+  private void preInsert(MkMaxEntry q, MkMaxEntry nodeEntry, KNNHeap knns_q) {
     if (LOG.isDebugging()) {
       LOG.debugFine("preInsert " + q + " - " + nodeEntry + "\n");
     }
 
-    double knnDist_q = knns_q.getKNNDistance().doubleValue();
-    MkMaxTreeNode<O, D> node = getNode(nodeEntry);
+    double knnDist_q = knns_q.getKNNDistance();
+    MkMaxTreeNode<O> node = getNode(nodeEntry);
     double knnDist_node = 0.;
 
     // leaf node
     if (node.isLeaf()) {
       for (int i = 0; i < node.getNumEntries(); i++) {
         MkMaxEntry p = node.getEntry(i);
-        D dist_pq = distance(p.getRoutingObjectID(), q.getRoutingObjectID());
+        double dist_pq = distance(p.getRoutingObjectID(), q.getRoutingObjectID());
 
         // p is nearer to q than the farthest kNN-candidate of q
         // ==> p becomes a knn-candidate
-        if (dist_pq.doubleValue() <= knnDist_q) {
+        if (dist_pq <= knnDist_q) {
           knns_q.insert(dist_pq, p.getRoutingObjectID());
           if (knns_q.size() >= getKmax()) {
-            knnDist_q = knns_q.getKNNDistance().doubleValue();
+            knnDist_q = knns_q.getKNNDistance();
             q.setKnnDistance(knnDist_q);
           }
         }
         // p is nearer to q than to its farthest knn-candidate
         // q becomes knn of p
-        if (dist_pq.doubleValue() <= p.getKnnDistance()) {
-          KNNList<D> knns_p = knnq.getKNNForDBID(p.getRoutingObjectID(), getKmax() - 1);
+        if (dist_pq <= p.getKnnDistance()) {
+          KNNList knns_p = knnq.getKNNForDBID(p.getRoutingObjectID(), getKmax() - 1);
 
           if (knns_p.size() + 1 < getKmax()) {
             p.setKnnDistance(Double.NaN);
           } else {
-            double knnDist_p = Math.max(dist_pq.doubleValue(), knns_p.getKNNDistance().doubleValue());
+            double knnDist_p = Math.max(dist_pq, knns_p.getKNNDistance());
             p.setKnnDistance(knnDist_p);
           }
         }
@@ -254,7 +251,7 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
 
         if (distEntry.second < entry_knnDist || distEntry.second < knnDist_q) {
           preInsert(q, dirEntry, knns_q);
-          knnDist_q = knns_q.getKNNDistance().doubleValue();
+          knnDist_q = knns_q.getKNNDistance();
         }
         knnDist_node = Math.max(knnDist_node, dirEntry.getKnnDistance());
       }
@@ -305,7 +302,7 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @return a new MkMaxTreeNode which is a leaf node
    */
   @Override
-  protected MkMaxTreeNode<O, D> createNewLeafNode() {
+  protected MkMaxTreeNode<O> createNewLeafNode() {
     return new MkMaxTreeNode<>(leafCapacity, true);
   }
 
@@ -313,7 +310,7 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @return a new MkMaxTreeNode which is a directory node
    */
   @Override
-  protected MkMaxTreeNode<O, D> createNewDirectoryNode() {
+  protected MkMaxTreeNode<O> createNewDirectoryNode() {
     return new MkMaxTreeNode<>(dirCapacity, false);
   }
 
@@ -321,7 +318,7 @@ public class MkMaxTree<O, D extends NumberDistance<D, ?>> extends AbstractMkTree
    * @return a new MkMaxDirectoryEntry representing the specified node
    */
   @Override
-  protected MkMaxEntry createNewDirectoryEntry(MkMaxTreeNode<O, D> node, DBID routingObjectID, double parentDistance) {
+  protected MkMaxEntry createNewDirectoryEntry(MkMaxTreeNode<O> node, DBID routingObjectID, double parentDistance) {
     return new MkMaxDirectoryEntry(routingObjectID, parentDistance, node.getPageID(), node.coveringRadius(routingObjectID, this), node.kNNDistance());
   }
 
