@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.deliclu;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -62,7 +62,7 @@ public class DeLiCluLeafEntry extends SpatialPointLeafEntry implements DeLiCluEn
    * @param id the unique id of the underlying data object
    * @param vector the vector to store
    */
-  public DeLiCluLeafEntry(DBID id, NumberVector<?> vector) {
+  public DeLiCluLeafEntry(DBID id, NumberVector vector) {
     super(id, vector);
     this.hasHandled = false;
     this.hasUnhandled = true;
