@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.vafile;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,8 +28,8 @@ import java.util.Arrays;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
-import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
+import de.lmu.ifi.dbs.elki.utilities.io.ByteArrayUtil;
 
 /**
  * Dimension approximation file, a one-dimensional part of the
@@ -66,7 +66,7 @@ public class DAFile {
    * @param dimension Dimension of this file
    * @param partitions Number of partitions
    */
-  public DAFile(Relation<? extends NumberVector<?>> relation, int dimension, int partitions) {
+  public DAFile(Relation<? extends NumberVector> relation, int dimension, int partitions) {
     final int size = relation.size();
     this.dimension = dimension;
     this.splitPositions = new double[partitions + 1];
