@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.logging.progress;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -42,7 +42,6 @@ public class StepProgress extends FiniteProgress {
    * 
    * @param total Total number of steps.
    */
-  @SuppressWarnings("deprecation")
   public StepProgress(int total) {
     super("Step", total);
   }
@@ -54,7 +53,6 @@ public class StepProgress extends FiniteProgress {
    * @param task Task title
    * @param total Total number of steps.
    */
-  @SuppressWarnings("deprecation")
   public StepProgress(String task, int total) {
     super(task, total);
   }
@@ -74,18 +72,6 @@ public class StepProgress extends FiniteProgress {
     return buf;
   }
 
-  /**
-   * Do a new step.
-   * 
-   * @param step Step number
-   * @param stepTitle Step title
-   */
-  @Deprecated
-  public void beginStep(int step, String stepTitle) {
-    setProcessed(step - 1);
-    this.stepTitle = stepTitle;
-  }
-
   /**
    * Do a new step and log it
    * 
@@ -99,14 +85,6 @@ public class StepProgress extends FiniteProgress {
     logger.progress(this);
   }
 
-  /**
-   * Mark the progress as completed.
-   */
-  @Deprecated
-  public void setCompleted() {
-    setProcessed(getTotal());
-  }
-
   /**
    * Mark the progress as completed and log it.
    *
