@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -105,6 +105,11 @@ public final class MathUtil {
    */
   public static final double ONE_BY_SQRTTWOPI = 1. / SQRTTWOPI;
 
+  /**
+   * 1. / log(2)
+   */
+  public static final double ONE_BY_LOG2 = 1. / Math.log(2.);
+
   /**
    * Logarithm of 2 to the basis e, for logarithm conversion.
    */
@@ -130,6 +135,11 @@ public final class MathUtil {
    */
   public static final double LOGPIHALF = LOGPI / 2.;
 
+  /**
+   * Math.log(2*Math.PI).
+   */
+  public static final double LOGTWOPI = Math.log(TWOPI);
+
   /**
    * Math.log(Math.sqrt(2*Math.PI)).
    */
@@ -157,6 +167,16 @@ public final class MathUtil {
     // Static methods only - do not instantiate!
   }
 
+  /**
+   * Compute the base 2 logarithm.
+   * 
+   * @param x X
+   * @return Logarithm base 2.
+   */
+  public static double log2(double x) {
+    return Math.log(x) * ONE_BY_LOG2;
+  }
+
   /**
    * Computes the square root of the sum of the squared arguments without under
    * or overflow.
@@ -169,19 +189,21 @@ public final class MathUtil {
    * @return {@code sqrt(a<sup>2</sup> + b<sup>2</sup>)}
    */
   public static double fastHypot(double a, double b) {
-    if (a < 0) {
+    if(a < 0) {
       a = -a;
     }
-    if (b < 0) {
+    if(b < 0) {
       b = -b;
     }
-    if (a > b) {
+    if(a > b) {
       final double r = b / a;
       return a * Math.sqrt(1 + r * r);
-    } else if (b != 0) {
+    }
+    else if(b != 0) {
       final double r = a / b;
       return b * Math.sqrt(1 + r * r);
-    } else {
+    }
+    else {
       return 0.0;
     }
   }
@@ -199,17 +221,17 @@ public final class MathUtil {
    * @return {@code sqrt(a<sup>2</sup> + b<sup>2</sup> + c<sup>2</sup>)}
    */
   public static double fastHypot3(double a, double b, double c) {
-    if (a < 0) {
+    if(a < 0) {
       a = -a;
     }
-    if (b < 0) {
+    if(b < 0) {
       b = -b;
     }
-    if (c < 0) {
+    if(c < 0) {
       c = -c;
     }
     double m = (a > b) ? ((a > c) ? a : c) : ((b > c) ? b : c);
-    if (m <= 0) {
+    if(m <= 0) {
       return 0.0;
     }
     a = a / m;
@@ -227,7 +249,7 @@ public final class MathUtil {
    */
   public static double mahalanobisDistance(Matrix weightMatrix, Vector o1_minus_o2) {
     double sqrDist = o1_minus_o2.transposeTimesTimes(weightMatrix, o1_minus_o2);
-    if (sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
+    if(sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
       sqrDist = Math.abs(sqrDist);
     }
     return Math.sqrt(sqrDist);
@@ -242,97 +264,196 @@ public final class MathUtil {
    */
   public static double mahalanobisDistance(double[][] weightMatrix, double[] o1_minus_o2) {
     double sqrDist = VMath.transposeTimesTimes(o1_minus_o2, weightMatrix, o1_minus_o2);
-    if (sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
+    if(sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
       sqrDist = Math.abs(sqrDist);
     }
     return Math.sqrt(sqrDist);
   }
 
   /**
-   * <p>
-   * Provides the Pearson product-moment correlation coefficient for two
-   * FeatureVectors.
-   * </p>
+   * Compute the Mahalanobis distance using the given weight matrix.
    * 
-   * @param x first FeatureVector
-   * @param y second FeatureVector
-   * @return the Pearson product-moment correlation coefficient for x and y
+   * @param weightMatrix Weight Matrix
+   * @param o1 First vector
+   * @param o2 Center vector
+   * @return Mahalanobis distance
    */
-  public static double pearsonCorrelationCoefficient(NumberVector<?> x, NumberVector<?> y) {
-    final int xdim = x.getDimensionality();
-    final int ydim = y.getDimensionality();
-    if (xdim != ydim) {
-      throw new IllegalArgumentException("Invalid arguments: feature vectors differ in dimensionality.");
+  public static double mahalanobisDistance(Matrix weightMatrix, Vector o1, Vector o2) {
+    double sqrDist = VMath.mahalanobisDistance(weightMatrix.getArrayRef(), o1.getArrayRef(), o2.getArrayRef());
+    if(sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
+      sqrDist = Math.abs(sqrDist);
     }
-    if (xdim <= 0) {
-      throw new IllegalArgumentException("Invalid arguments: dimensionality not positive.");
-    }
-    PearsonCorrelation pc = new PearsonCorrelation();
-    for (int i = 0; i < xdim; i++) {
-      pc.put(x.doubleValue(i), y.doubleValue(i), 1.0);
-    }
-    return pc.getCorrelation();
+    return Math.sqrt(sqrDist);
   }
 
   /**
-   * <p>
-   * Provides the Pearson product-moment correlation coefficient for two
+   * Compute the Mahalanobis distance using the given weight matrix.
+   * 
+   * @param weightMatrix Weight Matrix
+   * @param o1 First vector
+   * @param o2 Center vector
+   * @return Mahalanobis distance
+   */
+  public static double mahalanobisDistance(double[][] weightMatrix, double[] o1, double[] o2) {
+    double sqrDist = VMath.mahalanobisDistance(weightMatrix, o1, o2);
+    if(sqrDist < 0 && Math.abs(sqrDist) < 0.000000001) {
+      sqrDist = Math.abs(sqrDist);
+    }
+    return Math.sqrt(sqrDist);
+  }
+
+  /**
+   * Compute the Pearson product-moment correlation coefficient for two
+   * NumberVectors.
+   * 
+   * @param x first NumberVector
+   * @param y second NumberVector
+   * @return the Pearson product-moment correlation coefficient for x and y
+   */
+  public static double pearsonCorrelationCoefficient(NumberVector x, NumberVector y) {
+    final int xdim = x.getDimensionality();
+    final int ydim = y.getDimensionality();
+    if(xdim != ydim) {
+      throw new IllegalArgumentException("Invalid arguments: number vectors differ in dimensionality.");
+    }
+    // Old code, using an instance:
+    // PearsonCorrelation pc = new PearsonCorrelation();
+    // for(int i = 0; i < xdim; ++i) {
+    // final double xv = x.doubleValue(i), yv = y.doubleValue(i);
+    // pc.put(xv, yv, 1.);
+    // }
+    // return pc.getCorrelation();
+
+    // Inlined computation of Pearson correlation, to avoid allocating objects!
+    // This is a numerically stabilized version, avoiding sum-of-squares.
+    double sumXX = 0., sumYY = 0., sumXY = 0.;
+    double meanX = x.doubleValue(0), meanY = y.doubleValue(0);
+    int i = 1;
+    while(i < xdim) {
+      final double xv = x.doubleValue(i), yv = y.doubleValue(i);
+      // Delta to previous mean
+      final double deltaX = xv - meanX;
+      final double deltaY = yv - meanY;
+      // Increment count first
+      ++i;
+      // Update means
+      meanX += deltaX / i;
+      meanY += deltaY / i;
+      // Delta to new mean
+      final double neltaX = xv - meanX;
+      final double neltaY = yv - meanY;
+      // Update
+      sumXX += deltaX * neltaX;
+      sumYY += deltaY * neltaY;
+      // should equal deltaY * neltaX!
+      sumXY += deltaX * neltaY;
+    }
+    // One or both series were constant:
+    if(!(sumXX > 0. && sumYY > 0.)) {
+      return (sumXX == sumYY) ? 1. : 0.;
+    }
+    return sumXY / Math.sqrt(sumXX * sumYY);
+  }
+
+  /**
+   * Compute the Pearson product-moment correlation coefficient for two
+   * NumberVectors.
+   * 
+   * @param x first NumberVector
+   * @param y second NumberVector
+   * @param weights Weights
+   * @return the Pearson product-moment correlation coefficient for x and y
+   */
+  public static double weightedPearsonCorrelationCoefficient(NumberVector x, NumberVector y, double[] weights) {
+    final int xdim = x.getDimensionality();
+    final int ydim = y.getDimensionality();
+    if(xdim != ydim) {
+      throw new IllegalArgumentException("Invalid arguments: number vectors differ in dimensionality.");
+    }
+    if(xdim != weights.length) {
+      throw new IllegalArgumentException("Dimensionality doesn't agree to weights.");
+    }
+    // Inlined computation of Pearson correlation, to avoid allocating objects!
+    // This is a numerically stabilized version, avoiding sum-of-squares.
+    double sumXX = 0., sumYY = 0., sumXY = 0., sumWe = weights[0];
+    double meanX = x.doubleValue(0), meanY = y.doubleValue(0);
+    for(int i = 1; i < xdim; ++i) {
+      final double xv = x.doubleValue(i), yv = y.doubleValue(i), w = weights[i];
+      // Delta to previous mean
+      final double deltaX = xv - meanX;
+      final double deltaY = yv - meanY;
+      // Increment weight first
+      sumWe += w;
+      // Update means
+      meanX += deltaX * w / sumWe;
+      meanY += deltaY * w / sumWe;
+      // Delta to new mean
+      final double neltaX = xv - meanX;
+      final double neltaY = yv - meanY;
+      // Update
+      sumXX += w * deltaX * neltaX;
+      sumYY += w * deltaY * neltaY;
+      // should equal weight * deltaY * neltaX!
+      sumXY += w * deltaX * neltaY;
+    }
+    // One or both series were constant:
+    if(!(sumXX > 0. && sumYY > 0.)) {
+      return (sumXX == sumYY) ? 1. : 0.;
+    }
+    return sumXY / Math.sqrt(sumXX * sumYY);
+  }
+
+  /**
+   * Compute the Pearson product-moment correlation coefficient for two
    * FeatureVectors.
-   * </p>
    * 
    * @param x first FeatureVector
    * @param y second FeatureVector
    * @param weights Weights
    * @return the Pearson product-moment correlation coefficient for x and y
    */
-  public static double weightedPearsonCorrelationCoefficient(NumberVector<?> x, NumberVector<?> y, double[] weights) {
+  public static double weightedPearsonCorrelationCoefficient(NumberVector x, NumberVector y, NumberVector weights) {
     final int xdim = x.getDimensionality();
     final int ydim = y.getDimensionality();
-    if (xdim != ydim) {
+    if(xdim != ydim) {
       throw new IllegalArgumentException("Invalid arguments: feature vectors differ in dimensionality.");
     }
-    if (xdim != weights.length) {
+    if(xdim != weights.getDimensionality()) {
       throw new IllegalArgumentException("Dimensionality doesn't agree to weights.");
     }
-    PearsonCorrelation pc = new PearsonCorrelation();
-    for (int i = 0; i < xdim; i++) {
-      pc.put(x.doubleValue(i), y.doubleValue(i), weights[i]);
+    // Inlined computation of Pearson correlation, to avoid allocating objects!
+    // This is a numerically stabilized version, avoiding sum-of-squares.
+    double sumXX = 0., sumYY = 0., sumXY = 0., sumWe = weights.doubleValue(0);
+    double meanX = x.doubleValue(0), meanY = y.doubleValue(0);
+    for(int i = 1; i < xdim; ++i) {
+      final double xv = x.doubleValue(i), yv = y.doubleValue(i), w = weights.doubleValue(i);
+      // Delta to previous mean
+      final double deltaX = xv - meanX;
+      final double deltaY = yv - meanY;
+      // Increment weight first
+      sumWe += w;
+      // Update means
+      meanX += deltaX * w / sumWe;
+      meanY += deltaY * w / sumWe;
+      // Delta to new mean
+      final double neltaX = xv - meanX;
+      final double neltaY = yv - meanY;
+      // Update
+      sumXX += w * deltaX * neltaX;
+      sumYY += w * deltaY * neltaY;
+      // should equal weight * deltaY * neltaX!
+      sumXY += w * deltaX * neltaY;
     }
-    return pc.getCorrelation();
+    // One or both series were constant:
+    if(!(sumXX > 0. && sumYY > 0.)) {
+      return (sumXX == sumYY) ? 1. : 0.;
+    }
+    return sumXY / Math.sqrt(sumXX * sumYY);
   }
 
   /**
-   * <p>
-   * Provides the Pearson product-moment correlation coefficient for two
+   * Compute the Pearson product-moment correlation coefficient for two
    * FeatureVectors.
-   * </p>
-   * 
-   * @param x first FeatureVector
-   * @param y second FeatureVector
-   * @param weights Weights
-   * @return the Pearson product-moment correlation coefficient for x and y
-   */
-  public static double weightedPearsonCorrelationCoefficient(NumberVector<?> x, NumberVector<?> y, NumberVector<?> weights) {
-    final int xdim = x.getDimensionality();
-    final int ydim = y.getDimensionality();
-    if (xdim != ydim) {
-      throw new IllegalArgumentException("Invalid arguments: feature vectors differ in dimensionality.");
-    }
-    if (xdim != weights.getDimensionality()) {
-      throw new IllegalArgumentException("Dimensionality doesn't agree to weights.");
-    }
-    PearsonCorrelation pc = new PearsonCorrelation();
-    for (int i = 0; i < xdim; i++) {
-      pc.put(x.doubleValue(i), y.doubleValue(i), weights.doubleValue(i));
-    }
-    return pc.getCorrelation();
-  }
-
-  /**
-   * <p>
-   * Provides the Pearson product-moment correlation coefficient for two
-   * FeatureVectors.
-   * </p>
    * 
    * @param x first FeatureVector
    * @param y second FeatureVector
@@ -341,21 +462,43 @@ public final class MathUtil {
   public static double pearsonCorrelationCoefficient(double[] x, double[] y) {
     final int xdim = x.length;
     final int ydim = y.length;
-    if (xdim != ydim) {
-      throw new IllegalArgumentException("Invalid arguments: feature vectors differ in dimensionality.");
+    if(xdim != ydim) {
+      throw new IllegalArgumentException("Invalid arguments: arrays differ in length.");
     }
-    PearsonCorrelation pc = new PearsonCorrelation();
-    for (int i = 0; i < xdim; i++) {
-      pc.put(x[i], y[i], 1.0);
+    // Inlined computation of Pearson correlation, to avoid allocating objects!
+    // This is a numerically stabilized version, avoiding sum-of-squares.
+    double sumXX = 0., sumYY = 0., sumXY = 0.;
+    double meanX = x[0], meanY = y[0];
+    int i = 1;
+    while(i < xdim) {
+      final double xv = x[i], yv = y[i];
+      // Delta to previous mean
+      final double deltaX = xv - meanX;
+      final double deltaY = yv - meanY;
+      // Increment count first
+      ++i;
+      // Update means
+      meanX += deltaX / i;
+      meanY += deltaY / i;
+      // Delta to new mean
+      final double neltaX = xv - meanX;
+      final double neltaY = yv - meanY;
+      // Update
+      sumXX += deltaX * neltaX;
+      sumYY += deltaY * neltaY;
+      // should equal deltaY * neltaX!
+      sumXY += deltaX * neltaY;
     }
-    return pc.getCorrelation();
+    // One or both series were constant:
+    if(!(sumXX > 0. && sumYY > 0.)) {
+      return (sumXX == sumYY) ? 1. : 0.;
+    }
+    return sumXY / Math.sqrt(sumXX * sumYY);
   }
 
   /**
-   * <p>
-   * Provides the Pearson product-moment correlation coefficient for two
+   * Compute the Pearson product-moment correlation coefficient for two
    * FeatureVectors.
-   * </p>
    * 
    * @param x first FeatureVector
    * @param y second FeatureVector
@@ -365,17 +508,40 @@ public final class MathUtil {
   public static double weightedPearsonCorrelationCoefficient(double[] x, double[] y, double[] weights) {
     final int xdim = x.length;
     final int ydim = y.length;
-    if (xdim != ydim) {
-      throw new IllegalArgumentException("Invalid arguments: feature vectors differ in dimensionality.");
+    if(xdim != ydim) {
+      throw new IllegalArgumentException("Invalid arguments: arrays differ in length.");
     }
-    if (xdim != weights.length) {
+    if(xdim != weights.length) {
       throw new IllegalArgumentException("Dimensionality doesn't agree to weights.");
     }
-    PearsonCorrelation pc = new PearsonCorrelation();
-    for (int i = 0; i < xdim; i++) {
-      pc.put(x[i], y[i], weights[i]);
+    // Inlined computation of Pearson correlation, to avoid allocating objects!
+    // This is a numerically stabilized version, avoiding sum-of-squares.
+    double sumXX = 0., sumYY = 0., sumXY = 0., sumWe = weights[0];
+    double meanX = x[0], meanY = y[0];
+    for(int i = 1; i < xdim; ++i) {
+      final double xv = x[i], yv = y[i], w = weights[i];
+      // Delta to previous mean
+      final double deltaX = xv - meanX;
+      final double deltaY = yv - meanY;
+      // Increment weight first
+      sumWe += w;
+      // Update means
+      meanX += deltaX * w / sumWe;
+      meanY += deltaY * w / sumWe;
+      // Delta to new mean
+      final double neltaX = xv - meanX;
+      final double neltaY = yv - meanY;
+      // Update
+      sumXX += w * deltaX * neltaX;
+      sumYY += w * deltaY * neltaY;
+      // should equal weight * deltaY * neltaX!
+      sumXY += w * deltaX * neltaY;
     }
-    return pc.getCorrelation();
+    // One or both series were constant:
+    if(!(sumXX > 0. && sumYY > 0.)) {
+      return (sumXX == sumYY) ? 1. : 0.;
+    }
+    return sumXY / Math.sqrt(sumXX * sumYY);
   }
 
   /**
@@ -390,10 +556,10 @@ public final class MathUtil {
    * @return n * (n-1) * (n-2) * ... * 1
    */
   public static BigInteger factorial(BigInteger n) {
-    BigInteger nFac = BigInteger.valueOf(1);
-    while (n.compareTo(BigInteger.valueOf(1)) > 0) {
+    BigInteger nFac = BigInteger.ONE;
+    while(n.compareTo(BigInteger.ONE) > 0) {
       nFac = nFac.multiply(n);
-      n = n.subtract(BigInteger.valueOf(1));
+      n = n.subtract(BigInteger.ONE);
     }
     return nFac;
   }
@@ -407,7 +573,7 @@ public final class MathUtil {
    */
   public static long factorial(int n) {
     long nFac = 1;
-    for (long i = n; i > 0; i--) {
+    for(long i = n; i > 0; i--) {
       nFac *= i;
     }
     return nFac;
@@ -426,7 +592,7 @@ public final class MathUtil {
   public static long binomialCoefficient(long n, long k) {
     final long m = Math.max(k, n - k);
     double temp = 1;
-    for (long i = n, j = 1; i > m; i--, j++) {
+    for(long i = n, j = 1; i > m; i--, j++) {
       temp = temp * i / j;
     }
     return (long) temp;
@@ -441,7 +607,7 @@ public final class MathUtil {
    */
   public static double approximateFactorial(int n) {
     double nFac = 1.0;
-    for (int i = n; i > 0; i--) {
+    for(int i = n; i > 0; i--) {
       nFac *= i;
     }
     return nFac;
@@ -458,7 +624,7 @@ public final class MathUtil {
   public static double approximateBinomialCoefficient(int n, int k) {
     final int m = Math.max(k, n - k);
     long temp = 1;
-    for (int i = n, j = 1; i > m; i--, j++) {
+    for(int i = n, j = 1; i > m; i--, j++) {
       temp = temp * i / j;
     }
     return temp;
@@ -493,7 +659,7 @@ public final class MathUtil {
    */
   public static double[] randomDoubleArray(int len, Random r) {
     final double[] ret = new double[len];
-    for (int i = 0; i < len; i++) {
+    for(int i = 0; i < len; i++) {
       ret[i] = r.nextDouble();
     }
     return ret;
@@ -549,18 +715,18 @@ public final class MathUtil {
     // v1.transposeTimes(v2) / (v1.euclideanLength() * v2.euclideanLength());
     // We can just compute all three in parallel.
     double s = 0, e1 = 0, e2 = 0;
-    for (int k = 0; k < mindim; k++) {
+    for(int k = 0; k < mindim; k++) {
       final double r1 = v1[k];
       final double r2 = v2[k];
       s += r1 * r2;
       e1 += r1 * r1;
       e2 += r2 * r2;
     }
-    for (int k = mindim; k < v1.length; k++) {
+    for(int k = mindim; k < v1.length; k++) {
       final double r1 = v1[k];
       e1 += r1 * r1;
     }
-    for (int k = mindim; k < v2.length; k++) {
+    for(int k = mindim; k < v2.length; k++) {
       final double r2 = v2[k];
       e2 += r2 * r2;
     }
@@ -594,7 +760,7 @@ public final class MathUtil {
     // v1'.transposeTimes(v2') / (v1'.euclideanLength()*v2'.euclideanLength());
     // We can just compute all three in parallel.
     double s = 0, e1 = 0, e2 = 0;
-    for (int k = 0; k < mindim; k++) {
+    for(int k = 0; k < mindim; k++) {
       final double ok = (k < o.length) ? o[k] : 0;
       final double r1 = v1[k] - ok;
       final double r2 = v2[k] - ok;
@@ -602,12 +768,12 @@ public final class MathUtil {
       e1 += r1 * r1;
       e2 += r2 * r2;
     }
-    for (int k = mindim; k < v1.length; k++) {
+    for(int k = mindim; k < v1.length; k++) {
       final double ok = (k < o.length) ? o[k] : 0;
       final double r1 = v1[k] - ok;
       e1 += r1 * r1;
     }
-    for (int k = mindim; k < v2.length; k++) {
+    for(int k = mindim; k < v2.length; k++) {
       final double ok = (k < o.length) ? o[k] : 0;
       final double r2 = v2[k] - ok;
       e2 += r2 * r2;
@@ -623,11 +789,7 @@ public final class MathUtil {
    */
   public static double normAngle(double x) {
     x %= TWOPI;
-    if (x > 0) {
-      return x;
-    } else {
-      return x + TWOPI;
-    }
+    return (x > 0) ? x : x + TWOPI;
   }
 
   /**
@@ -639,15 +801,12 @@ public final class MathUtil {
    */
   public static double sinToCos(double angle, double sin) {
     // Numerics of the formula below aren't too good.
-    if ((-1e-5 < sin && sin < 1e-5) || sin > 0.99999 || sin < -0.99999) {
+    if((-1e-5 < sin && sin < 1e-5) || sin > 0.99999 || sin < -0.99999) {
       return Math.cos(angle);
     }
     angle = normAngle(angle);
-    if (angle < HALFPI || angle > ONEHALFPI) {
-      return Math.sqrt(1 - sin * sin);
-    } else {
-      return -Math.sqrt(1 - sin * sin);
-    }
+    final double s = Math.sqrt(1 - sin * sin);
+    return (angle < HALFPI || angle > ONEHALFPI) ? s : -s;
   }
 
   /**
@@ -659,15 +818,12 @@ public final class MathUtil {
    */
   public static double cosToSin(double angle, double cos) {
     // Numerics of the formula below aren't too good.
-    if ((-1e-5 < cos && cos < 1e-5) || cos > 0.99999 || cos < -0.99999) {
+    if((-1e-5 < cos && cos < 1e-5) || cos > 0.99999 || cos < -0.99999) {
       return Math.sin(angle);
     }
     angle = normAngle(angle);
-    if (angle < Math.PI) {
-      return Math.sqrt(1 - cos * cos);
-    } else {
-      return -Math.sqrt(1 - cos * cos);
-    }
+    final double s = Math.sqrt(1 - cos * cos);
+    return (angle < Math.PI) ? s : -s;
   }
 
   /**
@@ -754,48 +910,52 @@ public final class MathUtil {
    * @return Double value
    */
   public static double floatToDoubleUpper(float f) {
-    if (Float.isNaN(f)) {
+    if(Float.isNaN(f)) {
       return Double.NaN;
     }
-    if (Float.isInfinite(f)) {
-      if (f > 0) {
+    if(Float.isInfinite(f)) {
+      if(f > 0) {
         return Double.POSITIVE_INFINITY;
-      } else {
+      }
+      else {
         return Double.longBitsToDouble(0xc7efffffffffffffL);
       }
     }
     long bits = Double.doubleToRawLongBits((double) f);
-    if ((bits & 0x8000000000000000L) == 0) { // Positive
-      if (bits == 0L) {
+    if((bits & 0x8000000000000000L) == 0) { // Positive
+      if(bits == 0L) {
         return Double.longBitsToDouble(0x3690000000000000L);
       }
-      if (f == Float.MIN_VALUE) {
+      if(f == Float.MIN_VALUE) {
         // bits += 0x7_ffff_ffff_ffffl;
         return Double.longBitsToDouble(0x36a7ffffffffffffL);
       }
-      if (Float.MIN_NORMAL > f && f >= Double.MIN_NORMAL) {
+      if(Float.MIN_NORMAL > f && f >= Double.MIN_NORMAL) {
         // The most tricky case:
         // a denormalized float, but a normalized double
         final long bits2 = Double.doubleToRawLongBits((double) Math.nextUp(f));
         bits = (bits >>> 1) + (bits2 >>> 1) - 1L;
-      } else {
+      }
+      else {
         bits += 0xfffffffL; // 28 extra bits
       }
       return Double.longBitsToDouble(bits);
-    } else {
-      if (bits == 0x8000000000000000L) {
+    }
+    else {
+      if(bits == 0x8000000000000000L) {
         return -0.0d;
       }
-      if (f == -Float.MIN_VALUE) {
+      if(f == -Float.MIN_VALUE) {
         // bits -= 0xf_ffff_ffff_ffffl;
         return Double.longBitsToDouble(0xb690000000000001L);
       }
-      if (-Float.MIN_NORMAL < f && f <= -Double.MIN_NORMAL) {
+      if(-Float.MIN_NORMAL < f && f <= -Double.MIN_NORMAL) {
         // The most tricky case:
         // a denormalized float, but a normalized double
         final long bits2 = Double.doubleToRawLongBits((double) Math.nextUp(f));
         bits = (bits >>> 1) + (bits2 >>> 1) + 1L;
-      } else {
+      }
+      else {
         bits -= 0xfffffffL; // 28 extra bits
       }
       return Double.longBitsToDouble(bits);
@@ -812,48 +972,52 @@ public final class MathUtil {
    * @return Double value
    */
   public static double floatToDoubleLower(float f) {
-    if (Float.isNaN(f)) {
+    if(Float.isNaN(f)) {
       return Double.NaN;
     }
-    if (Float.isInfinite(f)) {
-      if (f < 0) {
+    if(Float.isInfinite(f)) {
+      if(f < 0) {
         return Double.NEGATIVE_INFINITY;
-      } else {
+      }
+      else {
         return Double.longBitsToDouble(0x47efffffffffffffL);
       }
     }
     long bits = Double.doubleToRawLongBits((double) f);
-    if ((bits & 0x8000000000000000L) == 0) { // Positive
-      if (bits == 0L) {
+    if((bits & 0x8000000000000000L) == 0) { // Positive
+      if(bits == 0L) {
         return +0.0d;
       }
-      if (f == Float.MIN_VALUE) {
+      if(f == Float.MIN_VALUE) {
         // bits -= 0xf_ffff_ffff_ffffl;
         return Double.longBitsToDouble(0x3690000000000001L);
       }
-      if (Float.MIN_NORMAL > f /* && f >= Double.MIN_NORMAL */) {
+      if(Float.MIN_NORMAL > f /* && f >= Double.MIN_NORMAL */) {
         // The most tricky case:
         // a denormalized float, but a normalized double
         final long bits2 = Double.doubleToRawLongBits((double) -Math.nextUp(-f));
         bits = (bits >>> 1) + (bits2 >>> 1) + 1L; // + (0xfff_ffffL << 18);
-      } else {
+      }
+      else {
         bits -= 0xfffffffL; // 28 extra bits
       }
       return Double.longBitsToDouble(bits);
-    } else {
-      if (bits == 0x8000000000000000L) {
+    }
+    else {
+      if(bits == 0x8000000000000000L) {
         return Double.longBitsToDouble(0xb690000000000000L);
       }
-      if (f == -Float.MIN_VALUE) {
+      if(f == -Float.MIN_VALUE) {
         // bits += 0x7_ffff_ffff_ffffl;
         return Double.longBitsToDouble(0xb6a7ffffffffffffL);
       }
-      if (-Float.MIN_NORMAL < f /* && f <= -Double.MIN_NORMAL */) {
+      if(-Float.MIN_NORMAL < f /* && f <= -Double.MIN_NORMAL */) {
         // The most tricky case:
         // a denormalized float, but a normalized double
         final long bits2 = Double.doubleToRawLongBits((double) -Math.nextUp(-f));
         bits = (bits >>> 1) + (bits2 >>> 1) - 1L;
-      } else {
+      }
+      else {
         bits += 0xfffffffL; // 28 extra bits
       }
       return Double.longBitsToDouble(bits);
@@ -878,12 +1042,12 @@ public final class MathUtil {
    * @return {@code Math.pow(x, p)}
    */
   public static double powi(double x, int p) {
-    if (p < 0) { // Fallback for negative integers.
+    if(p < 0) { // Fallback for negative integers.
       return Math.pow(x, p);
     }
     double ret = 1.;
-    for (; p > 0; p >>= 1) {
-      if ((p & 1) == 1) {
+    for(; p > 0; p >>= 1) {
+      if((p & 1) == 1) {
         ret *= x;
       }
       x *= x;
@@ -900,16 +1064,39 @@ public final class MathUtil {
    * @return {@code Math.pow(x, p)}
    */
   public static int ipowi(int x, int p) {
-    if (p < 0) { // Fallback for negative integers.
+    if(p < 0) { // Fallback for negative integers.
       return (int) Math.pow(x, p);
     }
     int ret = 1;
-    for (; p > 0; p >>= 1) {
-      if ((p & 1) == 1) {
+    for(; p > 0; p >>= 1) {
+      if((p & 1) == 1) {
         ret *= x;
       }
       x *= x;
     }
     return ret;
   }
+
+  /**
+   * Empty integer array.
+   */
+  public static final int[] EMPTY_INTS = new int[0];
+
+  /**
+   * Generate an array of integers.
+   * 
+   * @param start First integer
+   * @param end Last integer (exclusive!)
+   * @return Array of integers of length end-start
+   */
+  public static int[] sequence(int start, int end) {
+    if(start >= end) {
+      return EMPTY_INTS;
+    }
+    int[] ret = new int[end - start];
+    for(int j = 0; start < end; start++, j++) {
+      ret[j] = start;
+    }
+    return ret;
+  }
 }
