@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.dimensionsimilarity;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,8 +23,6 @@ package de.lmu.ifi.dbs.elki.math.dimensionsimilarity;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.BitSet;
-
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
@@ -33,8 +31,8 @@ import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.SubspaceEuclideanDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.math.Mean;
+import de.lmu.ifi.dbs.elki.utilities.BitsUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 
@@ -70,7 +68,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
  * @apiviz.uses SubspaceEuclideanDistanceFunction
  */
 @Reference(authors = "Elke Achtert, Hans-Peter Kriegel, Erich Schubert, Arthur Zimek", title = "Interactive Data Mining with 3D-Parallel-Coordinate-Trees", booktitle = "Proc. of the 2013 ACM International Conference on Management of Data (SIGMOD)", url = "http://dx.doi.org/10.1145/2463676.2463696")
-public class SURFINGDimensionSimilarity implements DimensionSimilarity<NumberVector<?>> {
+public class SURFINGDimensionSimilarity implements DimensionSimilarity<NumberVector> {
   /**
    * Static instance.
    */
@@ -83,9 +81,12 @@ public class SURFINGDimensionSimilarity implements DimensionSimilarity<NumberVec
     super();
   }
 
-  @Reference(authors = "Christian Baumgartner, Claudia Plant, Karin Kailing, Hans-Peter Kriegel, and Peer Kröger", title = "Subspace Selection for Clustering High-Dimensional Data", booktitle = "IEEE International Conference on Data Mining, 2004", url = "http://dx.doi.org/10.1109/ICDM.2004.10112")
+  @Reference(authors = "Christian Baumgartner, Claudia Plant, Karin Kailing, Hans-Peter Kriegel, and Peer Kröger", //
+  title = "Subspace Selection for Clustering High-Dimensional Data", //
+  booktitle = "IEEE International Conference on Data Mining, 2004", //
+  url = "http://dx.doi.org/10.1109/ICDM.2004.10112")
   @Override
-  public void computeDimensionSimilarites(Database database, Relation<? extends NumberVector<?>> relation, DBIDs subset, DimensionSimilarityMatrix matrix) {
+  public void computeDimensionSimilarites(Database database, Relation<? extends NumberVector> relation, DBIDs subset, DimensionSimilarityMatrix matrix) {
     final int dim = matrix.size();
     Mean kdistmean = new Mean();
     final int k = Math.max(1, subset.size() / 10);
@@ -93,20 +94,20 @@ public class SURFINGDimensionSimilarity implements DimensionSimilarity<NumberVec
     double[] knns = new double[subset.size()];
 
     // TODO: optimize by using 1d indexes?
-    for (int x = 0; x < dim; x++) {
+    for(int x = 0; x < dim; x++) {
       final int i = matrix.dim(x);
-      for (int y = x + 1; y < dim; y++) {
+      for(int y = x + 1; y < dim; y++) {
         final int j = matrix.dim(y);
-        BitSet dims = new BitSet(dim);
-        dims.set(i);
-        dims.set(j);
-        DistanceQuery<? extends NumberVector<?>, DoubleDistance> dq = database.getDistanceQuery(relation, new SubspaceEuclideanDistanceFunction(dims));
-        KNNQuery<? extends NumberVector<?>, DoubleDistance> knnq = database.getKNNQuery(dq, k);
+        long[] dims = BitsUtil.zero(dim);
+        BitsUtil.setI(dims, i);
+        BitsUtil.setI(dims, j);
+        DistanceQuery<? extends NumberVector> dq = database.getDistanceQuery(relation, new SubspaceEuclideanDistanceFunction(dims));
+        KNNQuery<? extends NumberVector> knnq = database.getKNNQuery(dq, k);
 
         kdistmean.reset();
         int knn = 0;
-        for (DBIDIter id1 = subset.iter(); id1.valid(); id1.advance(), knn++) {
-          final double kdist = knnq.getKNNForDBID(id1, k).getKNNDistance().doubleValue();
+        for(DBIDIter id1 = subset.iter(); id1.valid(); id1.advance(), knn++) {
+          final double kdist = knnq.getKNNForDBID(id1, k).getKNNDistance();
           kdistmean.put(kdist);
           knns[knn] = kdist;
         }
@@ -114,9 +115,9 @@ public class SURFINGDimensionSimilarity implements DimensionSimilarity<NumberVec
         // Deviation from mean:
         double diff = 0.;
         int below = 0;
-        for (int l = 0; l < knns.length; l++) {
+        for(int l = 0; l < knns.length; l++) {
           diff += Math.abs(mean - knns[l]);
-          if (knns[l] < mean) {
+          if(knns[l] < mean) {
             below++;
           }
         }
