@@ -8,7 +8,7 @@ import de.lmu.ifi.dbs.elki.math.MathUtil;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -270,7 +270,7 @@ public class AffineTransformation {
    * @return copy of the transformation matrix
    */
   public Matrix getTransformation() {
-    return trans.copy();
+    return trans;
   }
 
   /**
@@ -282,7 +282,7 @@ public class AffineTransformation {
     if(inv == null) {
       updateInverse();
     }
-    return inv.copy();
+    return inv;
   }
 
   /**
