@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.statistics;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -86,8 +86,7 @@ public class MultipleLinearRegression {
   private final double sst;
 
   /**
-   * Provides a new multiple linear regression model with the specified
-   * parameters.
+   * Constructor.
    * 
    * @param y the (n x 1) - vector holding the response values (y1, ..., yn)^T.
    * @param x the (n x p+1)-matrix holding the explanatory values, where the
@@ -144,7 +143,7 @@ public class MultipleLinearRegression {
     msg.append("\ny = ").append(FormatUtil.format(y, 9, 4));
     msg.append("\nb = ").append(FormatUtil.format(b, 9, 4));
     msg.append("\ne = ").append(FormatUtil.format(e, 9, 4));
-    msg.append("error variance = ").append(FormatUtil.format(variance, 4));
+    msg.append("error variance = ").append(FormatUtil.NF4.format(variance));
     return msg.toString();
   }
 
