@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.statistics.distribution.estimator.meta;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -199,7 +199,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
     }
     if (mom.getMax() <= mom.getMin()) {
       LOG.warning("Constant distribution detected. Cannot fit.");
-      return new UniformDistribution(mom.getMin() - .1, mom.getMax() + .1);
+      return new UniformDistribution(mom.getMin() - 1., mom.getMax() + 1.);
     }
     // Sort: for L-Moments, but getting the median is now also cheap.
     Arrays.sort(x);
@@ -253,9 +253,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           LOG.debugFine("Fitting distribution " + est.getClass().getSimpleName() + " failed: " + e.getMessage());
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     for (MADDistributionEstimator<?> est : madests) {
       try {
@@ -274,9 +272,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           LOG.debugFine("Fitting distribution " + est.getClass().getSimpleName() + " failed: " + e.getMessage());
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     for (LMMDistributionEstimator<?> est : lmmests) {
       if (lmm != null) {
@@ -297,9 +293,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           }
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     for (LogMOMDistributionEstimator<?> est : logmomests) {
       try {
@@ -318,9 +312,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           LOG.debugFine("Fitting distribution " + est.getClass().getSimpleName() + " failed: " + e.getMessage());
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     for (LogMADDistributionEstimator<?> est : logmadests) {
       try {
@@ -339,9 +331,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           LOG.debugFine("Fitting distribution " + est.getClass().getSimpleName() + " failed: " + e.getMessage());
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     { // Uniform estimators.
       final UniformMinMaxEstimator est = UniformMinMaxEstimator.STATIC;
@@ -361,9 +351,7 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           LOG.debugFine("Fitting distribution " + est.getClass().getSimpleName() + " failed: " + e.getMessage());
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
+      LOG.incrementProcessed(prog);
     }
     { // Uniform estimators.
       final UniformEnhancedMinMaxEstimator est = UniformEnhancedMinMaxEstimator.STATIC;
@@ -383,13 +371,9 @@ public class BestFitEstimator implements DistributionEstimator<Distribution> {
           LOG.debugFine("Fitting distribution " + est.getClass().getSimpleName() + " failed: " + e.getMessage());
         }
       }
-      if (prog != null) {
-        prog.incrementProcessed(LOG);
-      }
-    }
-    if (prog != null) {
-      prog.ensureCompleted(LOG);
+      LOG.incrementProcessed(prog);
     }
+    LOG.ensureCompleted(prog);
 
     if (LOG.isVeryVerbose()) {
       LOG.veryverbose("Best distribution fit: " + bestscore + " " + best.toString() + " via " + bestest);
