@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.statistics.tests;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2012
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,18 +23,16 @@ package de.lmu.ifi.dbs.elki.math.statistics.tests;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable;
-
 /**
  * Interface for the statistical test used by HiCS.
  * 
- * Provides a single method that calculates the deviation between two data
+ * Constists of a single method that calculates the deviation between two data
  * samples, given as arrays of double values
  * 
  * @author Jan Brusis
  * @author Erich Schubert
  */
-public interface GoodnessOfFitTest extends Parameterizable {
+public interface GoodnessOfFitTest {
   /**
    * Measure the deviation of a full sample from a conditional sample.
    * 
