@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.persistent;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -304,24 +304,6 @@ public class PersistentPageFile<P extends ExternalizablePage> extends AbstractSt
     return header;
   }
 
-  /**
-   * Increases the {@link AbstractStoringPageFile#readAccess readAccess} counter by
-   * one.
-   */
-  @Deprecated
-  public void increaseReadAccess() {
-    countRead();
-  }
-
-  /**
-   * Increases the {@link AbstractStoringPageFile#writeAccess writeAccess} counter by
-   * one.
-   */
-  @Deprecated
-  public void increaseWriteAccess() {
-    countWrite();
-  }
-
   /**
    * Set the next page id to the given value. If this means that any page ids
    * stored in <code>emptyPages</code> are smaller than
