@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.result;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -252,7 +252,7 @@ public class ResultUtil {
    * @param rel Relation
    * @return associated scales result
    */
-  public static ScalesResult getScalesResult(final Relation<? extends NumberVector<?>> rel) {
+  public static ScalesResult getScalesResult(final Relation<? extends NumberVector> rel) {
     Collection<ScalesResult> scas = ResultUtil.filterResults(rel, ScalesResult.class);
     if (scas.size() == 0) {
       final ScalesResult newsca = new ScalesResult(rel);
