@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.result;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,8 +25,7 @@ package de.lmu.ifi.dbs.elki.result;
 
 import java.util.Collection;
 
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Parameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.TrackedParameter;
 
 /**
  * Result that keeps track of settings that were used in generating this
@@ -38,14 +37,14 @@ public class SettingsResult extends BasicResult {
   /**
    * Settings storage.
    */
-  Collection<Pair<Object, Parameter<?>>> settings;
+  Collection<TrackedParameter> settings;
 
   /**
    * Constructor.
    * 
    * @param settings Settings to store
    */
-  public SettingsResult(Collection<Pair<Object, Parameter<?>>> settings) {
+  public SettingsResult(Collection<TrackedParameter> settings) {
     super("Settings", "settings");
     this.settings = settings;
   }
@@ -55,7 +54,7 @@ public class SettingsResult extends BasicResult {
    * 
    * @return the settings
    */
-  public Collection<Pair<Object, Parameter<?>>> getSettings() {
+  public Collection<TrackedParameter> getSettings() {
     return settings;
   }
 }
