@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,6 +23,8 @@ package de.lmu.ifi.dbs.elki.utilities;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import gnu.trove.strategy.HashingStrategy;
+
 import java.util.Arrays;
 
 /**
@@ -60,6 +62,26 @@ public final class BitsUtil {
   3125, 15625, 78125, 390625, 1953125,//
   9765625, 48828125, 244140625, 1220703125 };
 
+  /**
+   * Hashing strategy to use with Trove.
+   */
+  public static final HashingStrategy<long[]> TROVE_HASH_STRATEGY = new HashingStrategy<long[]>() {
+    /**
+     * Serial version number.
+     */
+    private static final long serialVersionUID = 1L;
+
+    @Override
+    public int computeHashCode(long[] arg0) {
+      return BitsUtil.hashCode(arg0);
+    }
+
+    @Override
+    public boolean equals(long[] arg0, long[] arg1) {
+      return equal(arg0, arg1);
+    }
+  };
+
   /**
    * Allocate a new long[].
    * 
@@ -67,7 +89,7 @@ public final class BitsUtil {
    * @return New array
    */
   public static long[] zero(int bits) {
-    return new long[((bits - 1) >>> LONG_LOG2_SIZE) + 1];
+    return new long[(bits > 0) ? ((bits - 1) >>> LONG_LOG2_SIZE) + 1 : 1];
   }
 
   /**
@@ -120,7 +142,7 @@ public final class BitsUtil {
    */
   public static long[] copy(long[] v, int mincap) {
     int words = ((mincap - 1) >>> LONG_LOG2_SIZE) + 1;
-    if (v.length == words) {
+    if(v.length == words) {
       return Arrays.copyOf(v, v.length);
     }
     long[] ret = new long[words];
@@ -140,15 +162,15 @@ public final class BitsUtil {
    */
   public static long[] copy(long[] v, int mincap, int shift) {
     int words = ((mincap - 1) >>> LONG_LOG2_SIZE) + 1;
-    if (v.length == words && shift == 0) {
+    if(v.length == words && shift == 0) {
       return Arrays.copyOf(v, v.length);
     }
     long[] ret = new long[words];
     final int shiftWords = shift >>> LONG_LOG2_SIZE;
     final int shiftBits = shift & LONG_LOG2_MASK;
     // Simple case - multiple of word size
-    if (shiftBits == 0) {
-      for (int i = shiftWords; i < ret.length; i++) {
+    if(shiftBits == 0) {
+      for(int i = shiftWords; i < ret.length; i++) {
         ret[i] |= v[i - shiftWords];
       }
       return ret;
@@ -156,7 +178,7 @@ public final class BitsUtil {
     // Overlapping case
     final int unshiftBits = Long.SIZE - shiftBits;
     final int end = Math.min(ret.length, v.length + shiftWords) - 1;
-    for (int i = end; i > shiftWords; i--) {
+    for(int i = end; i > shiftWords; i--) {
       final int src = i - shiftWords;
       ret[i] |= (v[src] << shiftBits) | (v[src - 1] >>> unshiftBits);
     }
@@ -214,15 +236,15 @@ public final class BitsUtil {
     final int last = v.length - 1;
     int o;
     // Sub word level:
-    for (o = 1; o < Long.SIZE; o <<= 1) {
-      for (int i = 0; i < last; i++) {
+    for(o = 1; o < Long.SIZE; o <<= 1) {
+      for(int i = 0; i < last; i++) {
         v[i] ^= (v[i] >>> o) ^ (v[i + 1] << (Long.SIZE - o));
       }
       v[last] ^= (v[last] >>> o);
     }
     // Word level:
-    for (o = 1; o <= last; o <<= 1) {
-      for (int i = o; i <= last; i++) {
+    for(o = 1; o <= last; o <<= 1) {
+      for(int i = o; i <= last; i++) {
         v[i - o] ^= v[i];
       }
     }
@@ -236,8 +258,8 @@ public final class BitsUtil {
    * @return true when all zero
    */
   public static boolean isZero(long[] v) {
-    for (int i = 0; i < v.length; i++) {
-      if (v[i] != 0) {
+    for(int i = 0; i < v.length; i++) {
+      if(v[i] != 0) {
         return false;
       }
     }
@@ -262,9 +284,9 @@ public final class BitsUtil {
    * @param v Value
    * @return Number of bits set in long[]
    */
-  public static long cardinality(long[] v) {
+  public static int cardinality(long[] v) {
     int sum = 0;
-    for (int i = 0; i < v.length; i++) {
+    for(int i = 0; i < v.length; i++) {
       sum += Long.bitCount(v[i]);
     }
     return sum;
@@ -320,6 +342,22 @@ public final class BitsUtil {
     return v;
   }
 
+  /**
+   * Put o onto v inplace, i.e. v = o
+   * 
+   * @param v Primary object
+   * @param o data to initialize to.
+   * @return v
+   */
+  public static long[] setI(long[] v, long[] o) {
+    assert (o.length <= v.length) : "Bit set sizes do not agree.";
+    final int max = Math.min(v.length, o.length);
+    for(int i = 0; i < max; i++) {
+      v[i] = o[i];
+    }
+    return v;
+  }
+
   /**
    * Clear bit number "off" in v.
    * 
@@ -365,7 +403,7 @@ public final class BitsUtil {
    */
   public static boolean get(long[] v, int off) {
     final int wordindex = off >>> LONG_LOG2_SIZE;
-    return (v[wordindex] & (1L << off)) != 0;
+    return (wordindex < v.length) && (v[wordindex] & (1L << off)) != 0;
   }
 
   /**
@@ -390,7 +428,7 @@ public final class BitsUtil {
    */
   public static long[] xorI(long[] v, long[] o) {
     assert (o.length <= v.length) : "Bit set sizes do not agree.";
-    for (int i = 0; i < o.length; i++) {
+    for(int i = 0; i < o.length; i++) {
       v[i] ^= o[i];
     }
     return v;
@@ -405,23 +443,23 @@ public final class BitsUtil {
    * @return v
    */
   public static long[] xorI(long[] v, long[] o, int off) {
-    if (off == 0) {
+    if(off == 0) {
       return xorI(v, o);
     }
-    if (off < 0) {
+    if(off < 0) {
       throw new UnsupportedOperationException("Negative shifts are not supported.");
     }
     // Break shift into integers to shift and bits to shift
     final int shiftWords = off >>> LONG_LOG2_SIZE;
     final int shiftBits = off & LONG_LOG2_MASK;
 
-    if (shiftWords >= v.length) {
+    if(shiftWords >= v.length) {
       return v;
     }
     // Simple case - multiple of word size
-    if (shiftBits == 0) {
+    if(shiftBits == 0) {
       final int end = Math.min(v.length, o.length + shiftWords);
-      for (int i = shiftWords; i < end; i++) {
+      for(int i = shiftWords; i < end; i++) {
         v[i] ^= o[i - shiftWords];
       }
       return v;
@@ -429,7 +467,7 @@ public final class BitsUtil {
     // Overlapping case
     final int unshiftBits = Long.SIZE - shiftBits;
     final int end = Math.min(v.length, o.length + shiftWords) - 1;
-    for (int i = end; i > shiftWords; i--) {
+    for(int i = end; i > shiftWords; i--) {
       final int src = i - shiftWords;
       v[i] ^= (o[src] << shiftBits) | (o[src - 1] >>> unshiftBits);
     }
@@ -447,7 +485,7 @@ public final class BitsUtil {
   public static long[] orI(long[] v, long[] o) {
     assert (o.length <= v.length) : "Bit set sizes do not agree.";
     final int max = Math.min(v.length, o.length);
-    for (int i = 0; i < max; i++) {
+    for(int i = 0; i < max; i++) {
       v[i] |= o[i];
     }
     return v;
@@ -464,23 +502,23 @@ public final class BitsUtil {
    * @return v
    */
   public static long[] orI(long[] v, long[] o, int off) {
-    if (off == 0) {
+    if(off == 0) {
       return orI(v, o);
     }
-    if (off < 0) {
+    if(off < 0) {
       throw new UnsupportedOperationException("Negative shifts are not supported.");
     }
     // Break shift into integers to shift and bits to shift
     final int shiftWords = off >>> LONG_LOG2_SIZE;
     final int shiftBits = off & LONG_LOG2_MASK;
 
-    if (shiftWords >= v.length) {
+    if(shiftWords >= v.length) {
       return v;
     }
     // Simple case - multiple of word size
-    if (shiftBits == 0) {
+    if(shiftBits == 0) {
       final int end = Math.min(v.length, o.length + shiftWords);
-      for (int i = shiftWords; i < end; i++) {
+      for(int i = shiftWords; i < end; i++) {
         v[i] |= o[i - shiftWords];
       }
       return v;
@@ -488,7 +526,7 @@ public final class BitsUtil {
     // Overlapping case
     final int unshiftBits = Long.SIZE - shiftBits;
     final int end = Math.min(v.length, o.length + shiftWords) - 1;
-    for (int i = end; i > shiftWords; i--) {
+    for(int i = end; i > shiftWords; i--) {
       final int src = i - shiftWords;
       v[i] |= (o[src] << shiftBits) | (o[src - 1] >>> unshiftBits);
     }
@@ -505,8 +543,8 @@ public final class BitsUtil {
    */
   public static long[] andI(long[] v, long[] o) {
     int i = 0;
-    for (; i < o.length; i++) {
-      v[i] |= o[i];
+    for(; i < o.length; i++) {
+      v[i] &= o[i];
     }
     // Zero higher words
     Arrays.fill(v, i, v.length, 0);
@@ -522,23 +560,23 @@ public final class BitsUtil {
    * @return v
    */
   public static long[] andI(long[] v, long[] o, int off) {
-    if (off == 0) {
+    if(off == 0) {
       return andI(v, o);
     }
-    if (off < 0) {
+    if(off < 0) {
       throw new UnsupportedOperationException("Negative shifts are not supported.");
     }
     // Break shift into integers to shift and bits to shift
     final int shiftWords = off >>> LONG_LOG2_SIZE;
     final int shiftBits = off & LONG_LOG2_MASK;
 
-    if (shiftWords >= v.length) {
+    if(shiftWords >= v.length) {
       return v;
     }
     // Simple case - multiple of word size
-    if (shiftBits == 0) {
+    if(shiftBits == 0) {
       final int end = Math.min(v.length, o.length + shiftWords);
-      for (int i = shiftWords; i < end; i++) {
+      for(int i = shiftWords; i < end; i++) {
         v[i] &= o[i - shiftWords];
       }
       // Clear bottom words
@@ -549,7 +587,7 @@ public final class BitsUtil {
     final int unshiftBits = Long.SIZE - shiftBits;
     final int end = Math.min(v.length, o.length + shiftWords) - 1;
     Arrays.fill(v, end + 1, v.length, 0);
-    for (int i = end; i > shiftWords; i--) {
+    for(int i = end; i > shiftWords; i--) {
       final int src = i - shiftWords;
       v[i] &= (o[src] << shiftBits) | (o[src - 1] >>> unshiftBits);
     }
@@ -559,6 +597,67 @@ public final class BitsUtil {
     return v;
   }
 
+  /**
+   * AND o onto v in a copy, i.e. v & o
+   * 
+   * The resulting array will have the shorter length of the two.
+   * 
+   * @param v Primary object
+   * @param o data to and
+   * @return Copy of v and o
+   */
+  public static long[] andCMin(long[] v, long[] o) {
+    final int min = Math.min(v.length, o.length);
+    long[] out = new long[min];
+    int i = 0;
+    for(; i < min; i++) {
+      out[i] = v[i] & o[i];
+    }
+    return out;
+  }
+
+  /**
+   * AND o onto v in a copy, i.e. v & o
+   * 
+   * The resulting array will have the shorter length of the two.
+   * 
+   * @param v Primary object
+   * @param o data to and
+   * @return Copy of v and o
+   */
+  public static long[] andCMax(long[] v, long[] o) {
+    final int min, max;
+    if(v.length < o.length) {
+      min = v.length;
+      max = o.length;
+    }
+    else {
+      min = o.length;
+      max = v.length;
+    }
+    long[] out = new long[max];
+    int i = 0;
+    for(; i < min; i++) {
+      out[i] = v[i] & o[i];
+    }
+    return out;
+  }
+
+  /**
+   * NOTAND o onto v inplace, i.e. v &= ~o
+   * 
+   * @param v Primary object
+   * @param o data to and
+   * @return v
+   */
+  public static long[] nandI(long[] v, long[] o) {
+    int i = 0;
+    for(; i < o.length; i++) {
+      v[i] &= ~o[i];
+    }
+    return v;
+  }
+
   /**
    * Invert v inplace.
    * 
@@ -566,7 +665,7 @@ public final class BitsUtil {
    * @return v
    */
   public static long[] invertI(long[] v) {
-    for (int i = 0; i < v.length; i++) {
+    for(int i = 0; i < v.length; i++) {
       v[i] = ~v[i];
     }
     return v;
@@ -582,21 +681,21 @@ public final class BitsUtil {
    * @return Bitset
    */
   public static long[] shiftRightI(long[] v, int off) {
-    if (off == 0) {
+    if(off == 0) {
       return v;
     }
-    if (off < 0) {
+    if(off < 0) {
       return shiftLeftI(v, -off);
     }
     // Break shift into integers to shift and bits to shift
     final int shiftWords = off >>> LONG_LOG2_SIZE;
     final int shiftBits = off & LONG_LOG2_MASK;
 
-    if (shiftWords >= v.length) {
+    if(shiftWords >= v.length) {
       return zeroI(v);
     }
     // Simple case - multiple of word size
-    if (shiftBits == 0) {
+    if(shiftBits == 0) {
       // Move whole words down
       System.arraycopy(v, shiftWords, v, 0, v.length - shiftWords);
       // Fill top words with zeros
@@ -606,7 +705,7 @@ public final class BitsUtil {
     // Overlapping case
     final int unshiftBits = Long.SIZE - shiftBits;
     // Bottom-up to not overlap the operations.
-    for (int i = 0; i < v.length - shiftWords - 1; i++) {
+    for(int i = 0; i < v.length - shiftWords - 1; i++) {
       final int src = i + shiftWords;
       v[i] = (v[src + 1] << unshiftBits) | (v[src] >>> shiftBits);
     }
@@ -627,21 +726,21 @@ public final class BitsUtil {
    * @return Bitset
    */
   public static long[] shiftLeftI(long[] v, int off) {
-    if (off == 0) {
+    if(off == 0) {
       return v;
     }
-    if (off < 0) {
+    if(off < 0) {
       return shiftRightI(v, -off);
     }
     // Break shift into integers to shift and bits to shift
     final int shiftWords = off >>> LONG_LOG2_SIZE;
     final int shiftBits = off & LONG_LOG2_MASK;
 
-    if (shiftWords >= v.length) {
+    if(shiftWords >= v.length) {
       return zeroI(v);
     }
     // Simple case - multiple of word size
-    if (shiftBits == 0) {
+    if(shiftBits == 0) {
       // Move whole words up
       System.arraycopy(v, 0, v, shiftWords, v.length - shiftWords);
       // Fill the initial words with zeros
@@ -651,7 +750,7 @@ public final class BitsUtil {
     // Overlapping case
     final int unshiftBits = Long.SIZE - shiftBits;
     // Top-Down to not overlap the operations.
-    for (int i = v.length - 1; i > shiftWords; i--) {
+    for(int i = v.length - 1; i > shiftWords; i--) {
       final int src = i - shiftWords;
       v[i] = (v[src] << shiftBits) | (v[src - 1] >>> unshiftBits);
     }
@@ -670,10 +769,10 @@ public final class BitsUtil {
    * @return cycled bit set
    */
   public static long cycleRightC(long v, int shift, int len) {
-    if (shift == 0) {
+    if(shift == 0) {
       return v;
     }
-    if (shift < 0) {
+    if(shift < 0) {
       return cycleLeftC(v, -shift, len);
     }
     final long ones = (1 << len) - 1;
@@ -706,7 +805,7 @@ public final class BitsUtil {
     final int zapWords = (zap >>> LONG_LOG2_SIZE);
     final int zapbits = zap & LONG_LOG2_MASK;
     Arrays.fill(v, v.length - zapWords, v.length, 0);
-    if (zapbits > 0) {
+    if(zapbits > 0) {
       v[v.length - zapWords - 1] &= (LONG_ALL_BITS >>> zapbits);
     }
     return v;
@@ -721,10 +820,10 @@ public final class BitsUtil {
    * @return cycled bit set
    */
   public static long cycleLeftC(long v, int shift, int len) {
-    if (shift == 0) {
+    if(shift == 0) {
       return v;
     }
-    if (shift < 0) {
+    if(shift < 0) {
       return cycleRightC(v, -shift, len);
     }
     final long ones = (1 << len) - 1;
@@ -753,25 +852,30 @@ public final class BitsUtil {
    * @return String representation
    */
   public static String toString(long[] v) {
+    if(v == null) {
+      return "null";
+    }
     final int mag = magnitude(v);
-    if (v.length == 0 || mag == 0) {
+    if(mag == 0) {
       return "0";
     }
-    final int words = ((mag - 1) >>> LONG_LOG2_SIZE) + 1;
     char[] digits = new char[mag];
-
     int pos = mag - 1;
-    for (int w = 0; w < words; w++) {
+
+    outer: for(int w = 0; w < v.length; w++) {
       long f = 1L;
-      for (int i = 0; i < Long.SIZE; i++) {
+      for(int i = 0; i < Long.SIZE; i++) {
         digits[pos] = ((v[w] & f) == 0) ? '0' : '1';
-        pos--;
         f <<= 1;
-        if (pos < 0) {
-          break;
+        --pos;
+        if(pos < 0) {
+          break outer;
         }
       }
     }
+    for(; pos >= 0; --pos) {
+      digits[pos] = '0';
+    }
     return new String(digits);
   }
 
@@ -783,25 +887,30 @@ public final class BitsUtil {
    * @return String representation
    */
   public static String toString(long[] v, int minw) {
+    if(v == null) {
+      return "null";
+    }
     final int mag = Math.max(magnitude(v), minw);
-    if (v.length == 0 || mag == 0) {
+    if(mag == 0) {
       return "0";
     }
-    final int words = ((mag - 1) >>> LONG_LOG2_SIZE) + 1;
     char[] digits = new char[mag];
-
     int pos = mag - 1;
-    for (int w = 0; w < words; w++) {
+
+    outer: for(int w = 0; w < v.length; w++) {
       long f = 1L;
-      for (int i = 0; i < Long.SIZE; i++) {
+      for(int i = 0; i < Long.SIZE; i++) {
         digits[pos] = ((v[w] & f) == 0) ? '0' : '1';
-        pos--;
         f <<= 1;
-        if (pos < 0) {
-          break;
+        --pos;
+        if(pos < 0) {
+          break outer;
         }
       }
     }
+    for(; pos >= 0; --pos) {
+      digits[pos] = '0';
+    }
     return new String(digits);
   }
 
@@ -813,24 +922,128 @@ public final class BitsUtil {
    */
   public static String toString(long v) {
     final int mag = magnitude(v);
-    if (mag == 0) {
+    if(mag == 0) {
       return "0";
     }
     char[] digits = new char[mag];
 
-    int pos = mag - 1;
     long f = 1L;
-    for (int i = 0; i < Long.SIZE; i++) {
+    for(int pos = mag - 1; pos >= 0; --pos, f <<= 1) {
       digits[pos] = ((v & f) == 0) ? '0' : '1';
-      pos--;
-      f <<= 1;
-      if (pos < 0) {
-        break;
-      }
     }
     return new String(digits);
   }
 
+  /**
+   * Convert bitset to a string consisting of "0" and "1", in low-endian order.
+   * 
+   * @param v Value to process
+   * @return String representation
+   */
+  public static String toStringLow(long[] v) {
+    if(v == null) {
+      return "null";
+    }
+    final int mag = magnitude(v);
+    if(mag == 0) {
+      return "0";
+    }
+    char[] digits = new char[mag];
+    int pos = 0;
+
+    outer: for(int w = 0; w < v.length; w++) {
+      long f = 1L;
+      for(int i = 0; i < Long.SIZE; i++) {
+        digits[pos] = ((v[w] & f) == 0) ? '0' : '1';
+        f <<= 1;
+        ++pos;
+        if(pos >= mag) {
+          break outer;
+        }
+      }
+    }
+    for(; pos < mag; ++pos) {
+      digits[pos] = '0';
+    }
+    return new String(digits);
+  }
+
+  /**
+   * Convert bitset to a string consisting of "0" and "1", in low-endian order.
+   * 
+   * @param v Value to process
+   * @param minw Minimum width
+   * @return String representation
+   */
+  public static String toStringLow(long[] v, int minw) {
+    if(v == null) {
+      return "null";
+    }
+    final int mag = Math.max(magnitude(v), minw);
+    if(mag == 0) {
+      return "0";
+    }
+    char[] digits = new char[mag];
+    int pos = 0;
+
+    outer: for(int w = 0; w < v.length; w++) {
+      long f = 1L;
+      for(int i = 0; i < Long.SIZE; i++) {
+        digits[pos] = ((v[w] & f) == 0) ? '0' : '1';
+        f <<= 1;
+        ++pos;
+        if(pos >= mag) {
+          break outer;
+        }
+      }
+    }
+    for(; pos < mag; ++pos) {
+      digits[pos] = '0';
+    }
+    return new String(digits);
+  }
+
+  /**
+   * Convert bitset to a string consisting of "0" and "1", in low-endian order.
+   * 
+   * @param v Value to process
+   * @return String representation
+   */
+  public static String toStringLow(long v) {
+    final int mag = magnitude(v);
+    if(mag == 0) {
+      return "0";
+    }
+    char[] digits = new char[mag];
+
+    long f = 1L;
+    for(int pos = 0; pos < mag; ++pos, f <<= 1) {
+      digits[pos] = ((v & f) == 0) ? '0' : '1';
+    }
+    return new String(digits);
+  }
+
+  /**
+   * Convert the bitset into a decimal representation, e.g. <tt>0, 3, 5</tt>
+   * 
+   * @param v Value
+   * @param sep Value separator
+   * @param offset Counting offset (usually, 0 or 1)
+   * @return String representation
+   */
+  public static String toString(long[] v, String sep, int offset) {
+    int p = nextSetBit(v, 0);
+    if(p < 0) {
+      return "";
+    }
+    StringBuilder buf = new StringBuilder();
+    buf.append(p + offset);
+    for(p = nextSetBit(v, p + 1); p >= 0; p = nextSetBit(v, p + 1)) {
+      buf.append(sep).append(p + offset);
+    }
+    return buf.toString();
+  }
+
   /**
    * Find the number of trailing zeros.
    * 
@@ -838,11 +1051,11 @@ public final class BitsUtil {
    * @return Position of first set bit, -1 if no set bit was found.
    */
   public static int numberOfTrailingZerosSigned(long[] v) {
-    for (int p = 0;; p++) {
-      if (p == v.length) {
+    for(int p = 0;; p++) {
+      if(p == v.length) {
         return -1;
       }
-      if (v[p] != 0) {
+      if(v[p] != 0) {
         return Long.numberOfTrailingZeros(v[p]) + p * Long.SIZE;
       }
     }
@@ -855,11 +1068,11 @@ public final class BitsUtil {
    * @return Position of first set bit, v.length * 64 if no set bit was found.
    */
   public static int numberOfTrailingZeros(long[] v) {
-    for (int p = 0;; p++) {
-      if (p == v.length) {
+    for(int p = 0;; p++) {
+      if(p == v.length) {
         return p * Long.SIZE;
       }
-      if (v[p] != 0) {
+      if(v[p] != 0) {
         return Long.numberOfTrailingZeros(v[p]) + p * Long.SIZE;
       }
     }
@@ -909,11 +1122,11 @@ public final class BitsUtil {
    * @return Position of first set bit, -1 if no set bit was found.
    */
   public static int numberOfLeadingZerosSigned(long[] v) {
-    for (int p = 0, ip = v.length - 1;; p++, ip--) {
-      if (p == v.length) {
+    for(int p = 0, ip = v.length - 1;; p++, ip--) {
+      if(p == v.length) {
         return -1;
       }
-      if (v[ip] != 0) {
+      if(v[ip] != 0) {
         return Long.numberOfLeadingZeros(v[ip]) + p * Long.SIZE;
       }
     }
@@ -926,11 +1139,11 @@ public final class BitsUtil {
    * @return Position of first set bit, v.length * 64 if no set bit was found.
    */
   public static int numberOfLeadingZeros(long[] v) {
-    for (int p = 0, ip = v.length - 1;; p++, ip--) {
-      if (p == v.length) {
+    for(int p = 0, ip = v.length - 1;; p++, ip--) {
+      if(p == v.length) {
         return p * Long.SIZE;
       }
-      if (v[ip] != 0) {
+      if(v[ip] != 0) {
         return Long.numberOfLeadingZeros(v[ip]) + p * Long.SIZE;
       }
     }
@@ -946,7 +1159,7 @@ public final class BitsUtil {
    * @return Position of first set bit, -1 if no set bit was found.
    */
   public static int numberOfLeadingZerosSigned(long v) {
-    if (v == 0) {
+    if(v == 0) {
       return -1;
     }
     return Long.numberOfLeadingZeros(v);
@@ -962,7 +1175,7 @@ public final class BitsUtil {
    * @return Position of first set bit, -1 if no set bit was found.
    */
   public static int numberOfLeadingZerosSigned(int v) {
-    if (v == 0) {
+    if(v == 0) {
       return -1;
     }
     return Integer.numberOfLeadingZeros(v);
@@ -992,6 +1205,24 @@ public final class BitsUtil {
     return Integer.numberOfLeadingZeros(v);
   }
 
+  /**
+   * Find the previous set bit.
+   * 
+   * @param v Values to process
+   * @param start Start position (inclusive)
+   * @return Position of previous set bit, or -1.
+   */
+  public static int previousSetBit(long v, int start) {
+    if(start == -1 || start >= Long.SIZE) {
+      return -1;
+    }
+    long cur = v & (LONG_ALL_BITS >>> start);
+    if(cur == 0) {
+      return -1;
+    }
+    return Long.SIZE - 1 - Long.numberOfLeadingZeros(cur);
+  }
+
   /**
    * Find the previous set bit.
    * 
@@ -1000,21 +1231,21 @@ public final class BitsUtil {
    * @return Position of previous set bit, or -1.
    */
   public static int previousSetBit(long[] v, int start) {
-    if (start == -1) {
+    if(start == -1) {
       return -1;
     }
     int wordindex = start >>> LONG_LOG2_SIZE;
-    if (wordindex >= v.length) {
+    if(wordindex >= v.length) {
       return magnitude(v) - 1;
     }
     // Initial word
     final int off = Long.SIZE - 1 - (start & LONG_LOG2_MASK);
     long cur = v[wordindex] & (LONG_ALL_BITS >>> off);
-    for (;;) {
-      if (cur != 0) {
+    for(;;) {
+      if(cur != 0) {
         return (wordindex + 1) * Long.SIZE - 1 - Long.numberOfLeadingZeros(cur);
       }
-      if (wordindex == 0) {
+      if(wordindex == 0) {
         return -1;
       }
       wordindex--;
@@ -1022,6 +1253,24 @@ public final class BitsUtil {
     }
   }
 
+  /**
+   * Find the previous clear bit.
+   * 
+   * @param v Values to process
+   * @param start Start position (inclusive)
+   * @return Position of previous clear bit, or -1.
+   */
+  public static int previousClearBit(long v, int start) {
+    if(start < 0 || start >= Long.SIZE) {
+      return -1;
+    }
+    long cur = ~v & (LONG_ALL_BITS >>> start);
+    if(cur == 0) {
+      return -1;
+    }
+    return Long.SIZE - 1 - Long.numberOfTrailingZeros(cur);
+  }
+
   /**
    * Find the previous clear bit.
    * 
@@ -1030,21 +1279,21 @@ public final class BitsUtil {
    * @return Position of previous clear bit, or -1.
    */
   public static int previousClearBit(long[] v, int start) {
-    if (start == -1) {
+    if(start == -1) {
       return -1;
     }
     int wordindex = start >>> LONG_LOG2_SIZE;
-    if (wordindex >= v.length) {
+    if(wordindex >= v.length) {
       return magnitude(v);
     }
     final int off = Long.SIZE + 1 - (start & LONG_LOG2_MASK);
     // Initial word
     long cur = ~v[wordindex] & (LONG_ALL_BITS >>> off);
-    for (;;) {
-      if (cur != 0) {
+    for(;;) {
+      if(cur != 0) {
         return (wordindex + 1) * Long.SIZE - 1 - Long.numberOfTrailingZeros(cur);
       }
-      if (wordindex == 0) {
+      if(wordindex == 0) {
         return -1;
       }
       wordindex--;
@@ -1052,6 +1301,24 @@ public final class BitsUtil {
     }
   }
 
+  /**
+   * Find the next set bit.
+   * 
+   * @param v Value to process
+   * @param start Start position (inclusive)
+   * @return Position of next set bit, or -1.
+   */
+  public static int nextSetBit(long v, int start) {
+    if(start >= Long.SIZE) {
+      return -1;
+    }
+    long cur = v & (LONG_ALL_BITS << start);
+    if(cur == 0) {
+      return -1;
+    }
+    return Long.numberOfTrailingZeros(cur);
+  }
+
   /**
    * Find the next set bit.
    * 
@@ -1061,24 +1328,42 @@ public final class BitsUtil {
    */
   public static int nextSetBit(long[] v, int start) {
     int wordindex = start >>> LONG_LOG2_SIZE;
-    if (wordindex >= v.length) {
+    if(wordindex >= v.length) {
       return -1;
     }
 
     // Initial word
     long cur = v[wordindex] & (LONG_ALL_BITS << start);
-    for (;;) {
-      if (cur != 0) {
+    for(;;) {
+      if(cur != 0) {
         return (wordindex * Long.SIZE) + Long.numberOfTrailingZeros(cur);
       }
       wordindex++;
-      if (wordindex == v.length) {
+      if(wordindex == v.length) {
         return -1;
       }
       cur = v[wordindex];
     }
   }
 
+  /**
+   * Find the next clear bit.
+   * 
+   * @param v Value to process
+   * @param start Start position (inclusive)
+   * @return Position of next clear bit, or -1.
+   */
+  public static int nextClearBit(long v, int start) {
+    if(start >= Long.SIZE) {
+      return -1;
+    }
+    long cur = ~v & (LONG_ALL_BITS << start);
+    if(cur == 0) {
+      return -1;
+    }
+    return Long.numberOfTrailingZeros(cur);
+  }
+
   /**
    * Find the next clear bit.
    * 
@@ -1088,14 +1373,14 @@ public final class BitsUtil {
    */
   public static int nextClearBit(long[] v, int start) {
     int wordindex = start >>> LONG_LOG2_SIZE;
-    if (wordindex >= v.length) {
+    if(wordindex >= v.length) {
       return -1;
     }
 
     // Initial word
     long cur = ~v[wordindex] & (LONG_ALL_BITS << start);
-    for (; wordindex < v.length;) {
-      if (cur != 0) {
+    for(; wordindex < v.length;) {
+      if(cur != 0) {
         return (wordindex * Long.SIZE) + Long.numberOfTrailingZeros(cur);
       }
       wordindex++;
@@ -1135,6 +1420,132 @@ public final class BitsUtil {
     return Integer.SIZE - Integer.numberOfLeadingZeros(v);
   }
 
+  /**
+   * Test whether two Bitsets intersect.
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return {@code true} when the bitsets intersect.
+   */
+  public static boolean intersect(long x, long y) {
+    return (x & y) != 0L;
+  }
+
+  /**
+   * Test whether two Bitsets intersect.
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return {@code true} when the bitsets intersect.
+   */
+  public static boolean intersect(long[] x, long[] y) {
+    final int min = (x.length < y.length) ? x.length : y.length;
+    for(int i = 0; i < min; i++) {
+      if((x[i] & y[i]) != 0L) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  /**
+   * Compute the intersection size of two Bitsets.
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return Intersection size
+   */
+  public static int intersectionSize(long x, long y) {
+    return Long.bitCount(x & y);
+  }
+
+  /**
+   * Compute the intersection size of two Bitsets.
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return Intersection size
+   */
+  public static int intersectionSize(long[] x, long[] y) {
+    final int lx = x.length, ly = y.length;
+    final int min = (lx < ly) ? lx : ly;
+    int res = 0;
+    for(int i = 0; i < min; i++) {
+      res += Long.bitCount(x[i] & y[i]);
+    }
+    return res;
+  }
+
+  /**
+   * Compute the union size of two Bitsets.
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return Union size
+   */
+  public static int unionSize(long x, long y) {
+    return Long.bitCount(x | y);
+  }
+
+  /**
+   * Compute the union size of two Bitsets.
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return Union size
+   */
+  public static int unionSize(long[] x, long[] y) {
+    final int lx = x.length, ly = y.length;
+    final int min = (lx < ly) ? lx : ly;
+    int i = 0, res = 0;
+    for(; i < min; i++) {
+      res += Long.bitCount(x[i] | y[i]);
+    }
+    for(; i < lx; i++) {
+      res += Long.bitCount(x[i]);
+    }
+    for(; i < ly; i++) {
+      res += Long.bitCount(y[i]);
+    }
+    return res;
+  }
+
+  /**
+   * Compute the Hamming distance (Size of symmetric difference), i.e.
+   * {@code cardinality(a ^ b)}.
+   * 
+   * @param b1 First vector
+   * @param b2 Second vector
+   * @return Cardinality of symmetric difference
+   */
+  public static int hammingDistance(long b1, long b2) {
+    return Long.bitCount(b1 ^ b2);
+  }
+
+  /**
+   * Compute the Hamming distance (Size of symmetric difference), i.e.
+   * {@code cardinality(a ^ b)}.
+   * 
+   * @param x First vector
+   * @param y Second vector
+   * @return Cardinality of symmetric difference
+   */
+  public static int hammingDistance(long[] x, long[] y) {
+    final int lx = x.length, ly = y.length;
+    final int min = (lx < ly) ? lx : ly;
+    int i = 0, h = 0;
+    for(; i < min; i++) {
+      h += Long.bitCount(x[i] ^ y[i]);
+    }
+    for(; i < lx; i++) {
+      h += Long.bitCount(x[i]);
+    }
+    for(; i < ly; i++) {
+      h += Long.bitCount(y[i]);
+    }
+    return h;
+  }
+
   /**
    * Capacity of the vector v.
    * 
@@ -1145,6 +1556,36 @@ public final class BitsUtil {
     return v.length * Long.SIZE;
   }
 
+  /**
+   * Test two bitsets for equality
+   * 
+   * @param x First bitset
+   * @param y Second bitset
+   * @return {@code true} when the bitsets are equal
+   */
+  public static boolean equal(long[] x, long[] y) {
+    if(x == null || y == null) {
+      return (x == null) && (y == null);
+    }
+    int p = Math.min(x.length, y.length) - 1;
+    for(int i = x.length - 1; i > p; i--) {
+      if(x[i] != 0L) {
+        return false;
+      }
+    }
+    for(int i = y.length - 1; i > p; i--) {
+      if(y[i] != 0L) {
+        return false;
+      }
+    }
+    for(; p >= 0; p--) {
+      if(x[p] != y[p]) {
+        return false;
+      }
+    }
+    return true;
+  }
+
   /**
    * Compare two bitsets.
    * 
@@ -1153,31 +1594,40 @@ public final class BitsUtil {
    * @return Comparison result
    */
   public static int compare(long[] x, long[] y) {
+    if(x == null) {
+      return (y == null) ? 0 : -1;
+    }
+    if(y == null) {
+      return +1;
+    }
     int p = Math.min(x.length, y.length) - 1;
-    for (int i = x.length - 1; i > p; i--) {
-      if (x[i] != 0) {
+    for(int i = x.length - 1; i > p; i--) {
+      if(x[i] != 0) {
         return +1;
       }
     }
-    for (int i = y.length - 1; i > p; i--) {
-      if (y[i] != 0) {
+    for(int i = y.length - 1; i > p; i--) {
+      if(y[i] != 0) {
         return -1;
       }
     }
-    for (; p >= 0; p--) {
+    for(; p >= 0; p--) {
       final long xp = x[p];
       final long yp = y[p];
-      if (xp != yp) {
-        if (xp < 0) {
-          if (yp < 0) {
+      if(xp != yp) {
+        if(xp < 0) {
+          if(yp < 0) {
             return (yp < xp) ? -1 : ((yp == xp) ? 0 : 1);
-          } else {
+          }
+          else {
             return +1;
           }
-        } else {
-          if (yp < 0) {
+        }
+        else {
+          if(yp < 0) {
             return -1;
-          } else {
+          }
+          else {
             return (xp < yp) ? -1 : ((xp == yp) ? 0 : 1);
           }
         }
@@ -1186,25 +1636,63 @@ public final class BitsUtil {
     return 0;
   }
 
+  /**
+   * Compute a hash code for the given bitset.
+   * 
+   * @param x Bitset bitset
+   * @return Hash code
+   */
+  public static int hashCode(long x) {
+    // We use almost the same hash code function as Java BitSet.
+    // Optimized for speed only, not protected against collision attacks
+    // If you need that, consider using murmur hashing with custom seeds.
+    long hash = 0x76543210L ^ x;
+    return (int) ((hash >> 32) ^ hash);
+  }
+
+  /**
+   * Compute a hash code for the given bitset.
+   * 
+   * @param x Bitset bitset
+   * @return Hash code
+   */
+  public static int hashCode(long[] x) {
+    // We use almost the same hash code function as Java BitSet.
+    // Optimized for speed only, not protected against collision attacks
+    // If you need that, consider using murmur hashing with custom seeds.
+    long hash = 0x76543210L;
+    for(int i = 0; i < x.length;) {
+      hash ^= x[i] * ++i;
+    }
+    return (int) ((hash >> 32) ^ hash);
+  }
+
+  /**
+   * Compute <code>m * pow(2., n)</code> using bit operations.
+   * 
+   * @param m Mantissa
+   * @param n Exponent
+   * @return Double value
+   */
   public static double lpow2(long m, int n) {
-    if (m == 0) {
+    if(m == 0) {
       return 0.0;
     }
-    if (m == Long.MIN_VALUE) {
+    if(m == Long.MIN_VALUE) {
       return lpow2(Long.MIN_VALUE >> 1, n + 1);
     }
-    if (m < 0) {
+    if(m < 0) {
       return -lpow2(-m, n);
     }
-    assert(m >= 0);
+    assert (m >= 0);
     int bitLength = magnitude(m);
     int shift = bitLength - 53;
     long exp = 1023L + 52 + n + shift; // Use long to avoid overflow.
-    if (exp >= 0x7FF) {
+    if(exp >= 0x7FF) {
       return Double.POSITIVE_INFINITY;
     }
-    if (exp <= 0) { // Degenerated number (subnormal, assume 0 for bit 52)
-      if (exp <= -54) {
+    if(exp <= 0) { // Degenerated number (subnormal, assume 0 for bit 52)
+      if(exp <= -54) {
         return 0.0;
       }
       return lpow2(m, n + 54) / 18014398509481984L; // 2^54 Exact.
@@ -1212,7 +1700,7 @@ public final class BitsUtil {
     // Normal number.
     long bits = (shift > 0) ? (m >> shift) + ((m >> (shift - 1)) & 1) : // Rounding.
     m << -shift;
-    if (((bits >> 52) != 1) && (++exp >= 0x7FF)) {
+    if(((bits >> 52) != 1) && (++exp >= 0x7FF)) {
       return Double.POSITIVE_INFINITY;
     }
     bits &= 0x000fffffffffffffL; // Clears MSB (bit 52)
@@ -1229,17 +1717,17 @@ public final class BitsUtil {
    * @return Double value.
    */
   public static double lpow10(long m, int n) {
-    if (m == 0) {
+    if(m == 0) {
       return 0.0;
     }
-    if (m == Long.MIN_VALUE) {
+    if(m == Long.MIN_VALUE) {
       return lpow10(Long.MIN_VALUE / 10, n + 1);
     }
-    if (m < 0) {
+    if(m < 0) {
       return -lpow10(-m, n);
     }
-    if (n >= 0) { // Positive power.
-      if (n > 308) {
+    if(n >= 0) { // Positive power.
+      if(n > 308) {
         return Double.POSITIVE_INFINITY;
       }
       // Works with 4 x 32 bits registers (x3:x2:x1:x0)
@@ -1248,14 +1736,14 @@ public final class BitsUtil {
       long x2 = m & LONG_32_BITS; // 32 bits.
       long x3 = m >>> 32; // 32 bits.
       int pow2 = 0;
-      while (n != 0) {
+      while(n != 0) {
         int i = (n >= POW5_INT.length) ? POW5_INT.length - 1 : n;
         int coef = POW5_INT[i]; // 31 bits max.
 
-        if (((int) x0) != 0) {
+        if(((int) x0) != 0) {
           x0 *= coef; // 63 bits max.
         }
-        if (((int) x1) != 0) {
+        if(((int) x1) != 0) {
           x1 *= coef; // 63 bits max.
         }
         x2 *= coef; // 63 bits max.
@@ -1276,7 +1764,7 @@ public final class BitsUtil {
 
         // Normalizes (x3 should be 32 bits max).
         long carry = x3 >>> 32;
-        if (carry != 0) { // Shift.
+        if(carry != 0) { // Shift.
           x0 = x1;
           x1 = x2;
           x2 = x3 & LONG_32_BITS;
@@ -1286,14 +1774,15 @@ public final class BitsUtil {
       }
 
       // Merges registers to a 63 bits mantissa.
-      assert(x3 >= 0);
+      assert (x3 >= 0);
       int shift = 31 - magnitude(x3); // -1..30
       pow2 -= shift;
       long mantissa = (shift < 0) ? (x3 << 31) | (x2 >>> 1) : // x3 is 32 bits.
       (((x3 << 32) | x2) << shift) | (x1 >>> (32 - shift));
       return lpow2(mantissa, pow2);
-    } else { // n < 0
-      if (n < -324 - 20) {
+    }
+    else { // n < 0
+      if(n < -324 - 20) {
         return 0.;
       }
 
@@ -1301,9 +1790,9 @@ public final class BitsUtil {
       long x1 = m; // 63 bits.
       long x0 = 0; // 63 bits.
       int pow2 = 0;
-      while (true) {
+      while(true) {
         // Normalizes x1:x0
-        assert(x1 >= 0);
+        assert (x1 >= 0);
         int shift = 63 - magnitude(x1);
         x1 <<= shift;
         x1 |= x0 >>> (63 - shift);
@@ -1311,7 +1800,7 @@ public final class BitsUtil {
         pow2 -= shift;
 
         // Checks if division has to be performed.
-        if (n == 0) {
+        if(n == 0) {
           break; // Done.
         }
 
