@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.datastructures.heap;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -233,8 +233,9 @@ public class ComparableMinHeap<K extends Comparable<? super K>> implements Objec
     }
 
     @Override
-    public void advance() {
+    public de.lmu.ifi.dbs.elki.utilities.datastructures.iterator.Iter advance() {
       pos++;
+      return this;
     }
 
     @SuppressWarnings("unchecked")
