@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.datastructures.heap;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -226,8 +226,9 @@ public class DoubleMinHeap implements DoubleHeap {
     }
 
     @Override
-    public void advance() {
+    public de.lmu.ifi.dbs.elki.utilities.datastructures.iterator.Iter advance() {
       pos++;
+      return this;
     }
 
     @Override
