@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.datastructures.heap;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,7 +23,6 @@ package de.lmu.ifi.dbs.elki.utilities.datastructures.heap;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import de.lmu.ifi.dbs.elki.utilities.pairs.PairInterface;
 
 /**
  * Object for a priority queue with integer priority. Can be used in the
@@ -35,7 +34,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.PairInterface;
  * 
  * @param <O> Stored object type.
  */
-public class IntegerPriorityObject<O> implements PairInterface<Integer, O>, Comparable<IntegerPriorityObject<?>> {
+public class IntegerPriorityObject<O> implements Comparable<IntegerPriorityObject<?>> {
   /**
    * Priority.
    */
@@ -78,17 +77,6 @@ public class IntegerPriorityObject<O> implements PairInterface<Integer, O>, Comp
     return object;
   }
 
-  @Override
-  @Deprecated
-  public Integer getFirst() {
-    return Integer.valueOf(priority);
-  }
-
-  @Override
-  public O getSecond() {
-    return object;
-  }
-
   @Override
   public int hashCode() {
     return ((object == null) ? 0 : object.hashCode());
