@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -26,6 +26,7 @@ package de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters;
 import java.lang.reflect.InvocationTargetException;
 import java.util.List;
 
+import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
@@ -50,6 +51,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameteriz
 // TODO: add additional constructors with parameter constraints.
 // TODO: turn restrictionClass into a constraint?
 public class ClassParameter<C> extends AbstractParameter<ClassParameter<C>, Class<? extends C>> {
+  /**
+   * The class logger.
+   */
+  private static final Logging LOG = Logging.getLogger(ClassParameter.class);
+
   /**
    * The restriction class for this class parameter.
    */
@@ -176,7 +182,27 @@ public class ClassParameter<C> extends AbstractParameter<ClassParameter<C>, Clas
   @Override
   public String getValuesDescription() {
     if(restrictionClass != null && restrictionClass != Object.class) {
-      return restrictionString();
+      StringBuilder info = new StringBuilder();
+      if(restrictionClass.isInterface()) {
+        info.append("Implementing ");
+      }
+      else {
+        info.append("Extending ");
+      }
+      info.append(restrictionClass.getName());
+      info.append(FormatUtil.NEWLINE);
+      
+      List<Class<?>> known = getKnownImplementations();
+      if(!known.isEmpty()) {
+        info.append("Known classes (default package " + restrictionClass.getPackage().getName() + "):");
+        info.append(FormatUtil.NEWLINE);
+        for(Class<?> c : known) {
+          info.append("->").append(FormatUtil.NONBREAKING_SPACE);
+          info.append(canonicalClassName(c, getRestrictionClass()));
+          info.append(FormatUtil.NEWLINE);
+        }
+      }
+      return info.toString();
     }
     return "";
   }
@@ -214,6 +240,9 @@ public class ClassParameter<C> extends AbstractParameter<ClassParameter<C>, Clas
         throw new WrongParameterValueException(this, getValue().getCanonicalName(), "Error instantiating class - no usable public constructor.");
       }
       catch(Exception e) {
+        if(LOG.isVerbose()) {
+          LOG.exception("Class initialization failed.", e);
+        }
         throw new WrongParameterValueException(this, getValue().getCanonicalName(), "Error instantiating class.", e);
       }
       return instance;
@@ -242,37 +271,6 @@ public class ClassParameter<C> extends AbstractParameter<ClassParameter<C>, Clas
     return InspectionUtil.cachedFindAllImplementations(getRestrictionClass());
   }
 
-  /**
-   * Provides a description string listing all classes for the given superclass
-   * or interface as specified in the properties.
-   * 
-   * @return a description string listing all classes for the given superclass
-   *         or interface as specified in the properties
-   */
-  public String restrictionString() {
-    StringBuilder info = new StringBuilder();
-    if(restrictionClass.isInterface()) {
-      info.append("Implementing ");
-    }
-    else {
-      info.append("Extending ");
-    }
-    info.append(restrictionClass.getName());
-    info.append(FormatUtil.NEWLINE);
-
-    List<Class<?>> known = getKnownImplementations();
-    if(!known.isEmpty()) {
-      info.append("Known classes (default package " + restrictionClass.getPackage().getName() + "):");
-      info.append(FormatUtil.NEWLINE);
-      for(Class<?> c : known) {
-        info.append("->" + FormatUtil.NONBREAKING_SPACE);
-        info.append(canonicalClassName(c, getRestrictionClass()));
-        info.append(FormatUtil.NEWLINE);
-      }
-    }
-    return info.toString();
-  }
-
   /**
    * Get the "simple" form of a class name.
    * 
