@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.referencepoints;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,22 +25,20 @@ package de.lmu.ifi.dbs.elki.utilities.referencepoints;
 
 import java.util.Collection;
 
+import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable;
 
 /**
  * Simple Interface for an heuristic to pick reference points.
  * 
  * @author Erich Schubert
- * 
- * @param <O> Object type
  */
-public interface ReferencePointsHeuristic<O> extends Parameterizable {
+public interface ReferencePointsHeuristic {
   /**
    * Get the reference points for the given database.
    * 
    * @param db Database
    * @return Collection of reference points.
    */
-  public <T extends O> Collection<O> getReferencePoints(Relation<T> db);
+  public Collection<? extends NumberVector> getReferencePoints(Relation<? extends NumberVector> db);
 }
\ No newline at end of file
