@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.scaling.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -26,7 +26,7 @@ package de.lmu.ifi.dbs.elki.utilities.scaling.outlier;
 import java.util.ArrayList;
 
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
-import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.DoubleRelation;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.math.MeanVariance;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
@@ -149,9 +149,9 @@ public class OutlierLinearScaling implements OutlierScalingFunction {
       MeanVariance mv = new MeanVariance();
       DoubleMinMax mm = (max == null) ? new DoubleMinMax() : null;
       boolean skippedzeros = false;
-      Relation<Double> scores = or.getScores();
+      DoubleRelation scores = or.getScores();
       for (DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
-        double val = scores.get(id);
+        double val = scores.doubleValue(id);
         if (nozeros && val == 0.0) {
           skippedzeros = true;
           continue;
@@ -175,9 +175,9 @@ public class OutlierLinearScaling implements OutlierScalingFunction {
       if (min == null || max == null) {
         boolean skippedzeros = false;
         DoubleMinMax mm = new DoubleMinMax();
-        Relation<Double> scores = or.getScores();
+        DoubleRelation scores = or.getScores();
         for (DBIDIter id = scores.iterDBIDs(); id.valid(); id.advance()) {
-          double val = scores.get(id);
+          double val = scores.doubleValue(id);
           if (nozeros && val == 0.0) {
             skippedzeros = true;
             continue;
