@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities.scaling.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -102,7 +102,7 @@ public class TopKOutlierScaling implements OutlierScalingFunction {
     }
     DBIDIter order = or.getOrdering().iter(or.getOrdering().getDBIDs()).iter();
     for(int i = 0; i < k && order.valid(); i++, order.advance()) {
-      cutoff = or.getScores().get(order);
+      cutoff = or.getScores().doubleValue(order);
     }
     max = or.getOutlierMeta().getActualMaximum();
     ground = or.getOutlierMeta().getTheoreticalBaseline();
