@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -129,7 +129,7 @@ public class VisualizerContext implements DataStoreListener, Result {
       // FIXME: this is a really ugly workaround. :-(
       if (TypeUtil.NUMBER_VECTOR_FIELD.isAssignableFromType(rel.getDataTypeInformation())) {
         @SuppressWarnings("unchecked")
-        Relation<? extends NumberVector<?>> vrel = (Relation<? extends NumberVector<?>>) rel;
+        Relation<? extends NumberVector> vrel = (Relation<? extends NumberVector>) rel;
         ResultUtil.getScalesResult(vrel);
       }
     }
