@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.projections;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -173,7 +173,7 @@ public class AffineProjection extends AbstractFullProjection implements Projecti
   }
 
   @Override
-  public double[] fastProjectDataToRenderSpace(NumberVector<?> data) {
+  public double[] fastProjectDataToRenderSpace(NumberVector data) {
     return fastProjectScaledToRenderSpace(fastProjectDataToScaledSpace(data));
   }
 
@@ -184,7 +184,7 @@ public class AffineProjection extends AbstractFullProjection implements Projecti
   }
 
   @Override
-  public double[] fastProjectDataToScaledSpace(NumberVector<?> data) {
+  public double[] fastProjectDataToScaledSpace(NumberVector data) {
     // FIXME: implement with less objects?
     return projectDataToScaledSpace(data).getArrayRef();
   }
@@ -222,7 +222,7 @@ public class AffineProjection extends AbstractFullProjection implements Projecti
   }
 
   @Override
-  public double[] fastProjectRelativeDataToRenderSpace(NumberVector<?> data) {
+  public double[] fastProjectRelativeDataToRenderSpace(NumberVector data) {
     // FIXME: implement with less objects?
     return fastProjectRelativeScaledToRenderSpace(projectRelativeDataToScaledSpace(data).getArrayRef());
   }
