@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.projections;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -55,7 +55,7 @@ public class Simple1D extends AbstractSimpleProjection implements Projection1D {
   }
 
   @Override
-  public double fastProjectDataToRenderSpace(NumberVector<?> data) {
+  public double fastProjectDataToRenderSpace(NumberVector data) {
     return (scales[dnum].getScaled(data.doubleValue(dnum)) - 0.5) * SCALE;
   }
 
@@ -70,7 +70,7 @@ public class Simple1D extends AbstractSimpleProjection implements Projection1D {
   }
 
   @Override
-  public double fastProjectRelativeDataToRenderSpace(NumberVector<?> data) {
+  public double fastProjectRelativeDataToRenderSpace(NumberVector data) {
     return (data.doubleValue(dnum) - 0.5) * SCALE;
   }
 
