@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.projector;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -50,7 +50,7 @@ import de.lmu.ifi.dbs.elki.visualization.visualizers.visunproj.LabelVisualizatio
  * 
  * @param <V> Vector type
  */
-public class HistogramProjector<V extends NumberVector<?>> extends AbstractHierarchicalResult implements Projector {
+public class HistogramProjector<V extends NumberVector> extends AbstractHierarchicalResult implements Projector {
   /**
    * Relation we project.
    */
