@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.optics;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,8 +28,7 @@ import java.util.Collection;
 import org.apache.batik.util.SVGConstants;
 import org.w3c.dom.Element;
 
-import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.optics.ClusterOrderEntry;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.result.HierarchicalResult;
 import de.lmu.ifi.dbs.elki.result.Result;
@@ -59,8 +58,7 @@ public class OPTICSPlotVisualizer extends AbstractVisFactory {
   private static final String NAME = "OPTICS Plot";
 
   /**
-   * Constructor, adhering to
-   * {@link de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable}
+   * Constructor.
    */
   public OPTICSPlotVisualizer() {
     super();
@@ -79,7 +77,7 @@ public class OPTICSPlotVisualizer extends AbstractVisFactory {
 
   @Override
   public Visualization makeVisualization(VisualizationTask task) {
-    return new Instance<DoubleDistance>(task);
+    return new Instance(task);
   }
 
   @Override
@@ -92,10 +90,8 @@ public class OPTICSPlotVisualizer extends AbstractVisFactory {
    * Instance.
    * 
    * @author Erich Schubert
-   * 
-   * @param <D> Distance type
    */
-  public class Instance<D extends Distance<D>> extends AbstractOPTICSVisualization<D> {
+  public class Instance extends AbstractOPTICSVisualization<ClusterOrderEntry<?>> {
     /**
      * Constructor.
      * 
@@ -110,7 +106,7 @@ public class OPTICSPlotVisualizer extends AbstractVisFactory {
       makeLayerElement();
       // addCSSClasses();
 
-      OPTICSPlot<D> opticsplot = optics.getOPTICSPlot(context);
+      OPTICSPlot<?> opticsplot = optics.getOPTICSPlot(context);
       String ploturi = opticsplot.getSVGPlotURI();
 
       Element itag = svgp.svgElement(SVGConstants.SVG_IMAGE_TAG);
