@@ -3,7 +3,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.parallel;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -89,7 +89,7 @@ public class AxisVisibilityVisualization extends AbstractVisFactory {
    * @author Robert Rödler
    * @author Erich Schubert
    */
-  public class Instance extends AbstractParallelVisualization<NumberVector<?>> {
+  public class Instance extends AbstractParallelVisualization<NumberVector> {
     /**
      * Generic tags to indicate the type of element. Used in IDs, CSS-Classes
      * etc.
