@@ -64,8 +64,7 @@ public class ClusterParallelMeanVisualization extends AbstractVisFactory {
   private static final String NAME = "Cluster Means";
 
   /**
-   * Constructor, adhering to
-   * {@link de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable}.
+   * Constructor.
    */
   public ClusterParallelMeanVisualization() {
     super();
@@ -83,7 +82,7 @@ public class ClusterParallelMeanVisualization extends AbstractVisFactory {
     for (Clustering<?> c : clusterings) {
       if (c.getAllClusters().size() > 0) {
         // Does the cluster have a model with cluster means?
-        Clustering<MeanModel<? extends NumberVector<?>>> mcls = findMeanModel(c);
+        Clustering<MeanModel> mcls = findMeanModel(c);
         if (mcls != null) {
           Collection<ParallelPlotProjector<?>> ps = ResultUtil.filterResults(baseResult, ParallelPlotProjector.class);
           for (ParallelPlotProjector<?> p : ps) {
@@ -104,9 +103,9 @@ public class ClusterParallelMeanVisualization extends AbstractVisFactory {
    * @return the clustering cast to return a mean model, null otherwise.
    */
   @SuppressWarnings("unchecked")
-  private static Clustering<MeanModel<? extends NumberVector<?>>> findMeanModel(Clustering<?> c) {
-    if (c.getAllClusters().get(0).getModel() instanceof MeanModel<?>) {
-      return (Clustering<MeanModel<? extends NumberVector<?>>>) c;
+  private static Clustering<MeanModel> findMeanModel(Clustering<?> c) {
+    if (c.getAllClusters().get(0).getModel() instanceof MeanModel) {
+      return (Clustering<MeanModel>) c;
     }
     return null;
   }
@@ -123,7 +122,7 @@ public class ClusterParallelMeanVisualization extends AbstractVisFactory {
    * @author Robert Rödler
    * 
    */
-  public class Instance extends AbstractParallelVisualization<NumberVector<?>> implements DataStoreListener {
+  public class Instance extends AbstractParallelVisualization<NumberVector> implements DataStoreListener {
     /**
      * Generic tags to indicate the type of element. Used in IDs, CSS-Classes
      * etc.
@@ -133,7 +132,7 @@ public class ClusterParallelMeanVisualization extends AbstractVisFactory {
     /**
      * The result we visualize.
      */
-    private Clustering<MeanModel<? extends NumberVector<?>>> clustering;
+    private Clustering<MeanModel> clustering;
 
     /**
      * Constructor.
@@ -159,13 +158,13 @@ public class ClusterParallelMeanVisualization extends AbstractVisFactory {
     protected void redraw() {
       addCSSClasses(svgp);
 
-      Iterator<Cluster<MeanModel<? extends NumberVector<?>>>> ci = clustering.getAllClusters().iterator();
+      Iterator<Cluster<MeanModel>> ci = clustering.getAllClusters().iterator();
       for (int cnum = 0; cnum < clustering.getAllClusters().size(); cnum++) {
-        Cluster<MeanModel<? extends NumberVector<?>>> clus = ci.next();
+        Cluster<MeanModel> clus = ci.next();
         if (clus.getModel() == null) {
           continue;
         }
-        NumberVector<?> mean = clus.getModel().getMean();
+        NumberVector mean = clus.getModel().getMean();
         if (mean == null) {
           continue;
         }
