@@ -77,8 +77,7 @@ public class SelectionToolLineVisualization extends AbstractVisFactory {
   }
 
   /**
-   * Constructor, adhering to
-   * {@link de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable}
+   * Constructor.
    */
   public SelectionToolLineVisualization() {
     super();
@@ -115,7 +114,7 @@ public class SelectionToolLineVisualization extends AbstractVisFactory {
    * @apiviz.has SelectionResult oneway - - updates
    * @apiviz.has DBIDSelection oneway - - updates
    */
-  public class Instance extends AbstractParallelVisualization<NumberVector<?>> implements DragableArea.DragListener {
+  public class Instance extends AbstractParallelVisualization<NumberVector> implements DragableArea.DragListener {
     /**
      * CSS class of the selection rectangle while selecting.
      */
