@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.scatterplot.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -152,7 +152,7 @@ public class COPVectorVisualization extends AbstractVisFactory {
         if (VMath.euclideanLength(ev) < 0.01) {
           continue;
         }
-        final NumberVector<?> vec = rel.get(objId);
+        final NumberVector vec = rel.get(objId);
         if (vec == null) {
           continue;
         }
