@@ -76,8 +76,7 @@ public class SelectionToolCubeVisualization extends AbstractVisFactory {
   private static final String NAME = "Range Selection";
 
   /**
-   * Constructor, adhering to
-   * {@link de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable}.
+   * Constructor.
    */
   public SelectionToolCubeVisualization() {
     super();
@@ -261,7 +260,7 @@ public class SelectionToolCubeVisualization extends AbstractVisFactory {
 
       selection.clear();
       candidates: for (DBIDIter iditer = rel.iterDBIDs(); iditer.valid(); iditer.advance()) {
-        NumberVector<?> dbTupel = rel.get(iditer);
+        NumberVector dbTupel = rel.get(iditer);
         for (int i = 0; i < dim; i++) {
           if (ranges != null && ranges[i] != null) {
             if (dbTupel.doubleValue(i) < ranges[i].first || dbTupel.doubleValue(i) > ranges[i].second) {
