@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.visualizers.visunproj;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -71,8 +71,7 @@ public class HistogramVisualization extends AbstractVisFactory {
   private static final String SERIESID = "series";
 
   /**
-   * Constructor, adhering to
-   * {@link de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable}
+   * Constructor.
    */
   public HistogramVisualization() {
     super();
@@ -82,7 +81,7 @@ public class HistogramVisualization extends AbstractVisFactory {
   public Visualization makeVisualization(VisualizationTask task) {
     VisualizerContext context = task.getContext();
     SVGPlot svgp = task.getPlot();
-    HistogramResult<? extends NumberVector<?>> curve = task.getResult();
+    HistogramResult<? extends NumberVector> curve = task.getResult();
 
     final StyleLibrary style = context.getStyleResult().getStyleLibrary();
     final double sizex = StyleLibrary.SCALE;
@@ -96,7 +95,7 @@ public class HistogramVisualization extends AbstractVisFactory {
     int dim = -1;
     DoubleMinMax xminmax = new DoubleMinMax();
     DoubleMinMax yminmax = new DoubleMinMax();
-    for (NumberVector<?> vec : curve) {
+    for (NumberVector vec : curve) {
       xminmax.put(vec.doubleValue(0));
       if (dim < 0) {
         dim = vec.getDimensionality();
@@ -104,7 +103,7 @@ public class HistogramVisualization extends AbstractVisFactory {
         // TODO: test and throw always
         assert (dim == vec.getDimensionality());
       }
-      for (int i = 0; i < dim; i++) {
+      for (int i = 1; i < dim; i++) {
         yminmax.put(vec.doubleValue(i));
       }
     }
@@ -117,7 +116,7 @@ public class HistogramVisualization extends AbstractVisFactory {
     double range = xminmax.getMax() - xminmax.getMin();
     double binwidth = range / (size - 1);
 
-    LinearScale xscale = new LinearScale(xminmax.getMin() - binwidth * .5, xminmax.getMax() + binwidth * .5);
+    LinearScale xscale = new LinearScale(xminmax.getMin() - binwidth * .49999, xminmax.getMax() + binwidth * .49999);
     LinearScale yscale = new LinearScale(yminmax.getMin(), yminmax.getMax());
 
     SVGPath[] path = new SVGPath[dim];
@@ -126,7 +125,7 @@ public class HistogramVisualization extends AbstractVisFactory {
     }
 
     // draw curves.
-    for (NumberVector<?> vec : curve) {
+    for (NumberVector vec : curve) {
       for (int d = 0; d < dim; d++) {
         path[d].lineTo(sizex * (xscale.getScaled(vec.doubleValue(0) - binwidth * .5)), sizey * (1 - yscale.getScaled(vec.doubleValue(d + 1))));
         path[d].lineTo(sizex * (xscale.getScaled(vec.doubleValue(0) + binwidth * .5)), sizey * (1 - yscale.getScaled(vec.doubleValue(d + 1))));
@@ -164,8 +163,8 @@ public class HistogramVisualization extends AbstractVisFactory {
 
   @Override
   public void processNewResult(HierarchicalResult baseResult, Result newResult) {
-    List<HistogramResult<? extends NumberVector<?>>> histograms = ResultUtil.filterResults(newResult, HistogramResult.class);
-    for (HistogramResult<? extends NumberVector<?>> histogram : histograms) {
+    List<HistogramResult<? extends NumberVector>> histograms = ResultUtil.filterResults(newResult, HistogramResult.class);
+    for (HistogramResult<? extends NumberVector> histogram : histograms) {
       final VisualizationTask task = new VisualizationTask(NAME, histogram, null, this);
       task.width = 2.0;
       task.height = 1.0;
