@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.workflow;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2013
+ Copyright (C) 2014
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -91,14 +91,11 @@ public class AlgorithmStep implements WorkflowStep {
       }
     }
     for (Algorithm algorithm : algorithms) {
-      Duration duration = LOG.isStatistics() ? LOG.newDuration(algorithm.getClass().getName()+".runtime") : null;
-      if (duration != null) {
-        duration.begin();
-      }
+      Thread.currentThread().setName(algorithm.toString());
+      Duration duration = LOG.isStatistics() ? LOG.newDuration(algorithm.getClass().getName()+".runtime").begin() : null;
       Result res = algorithm.run(database);
       if (duration != null) {
-        duration.end();
-        LOG.statistics(duration);
+        LOG.statistics(duration.end());
       }
       if (LOG.isStatistics() && database.getIndexes().size() > 0) {
         LOG.statistics("Index statistics after running algorithms:");
